/*
Stretch.cpp
*/

#include <memory.h>

__inline static void StretchLine422(long DestWidth, long StartX, long SourceWidth,
						  short * DestPtr, short *SourcePtr);


void StretchYUV422(short * DestPtr, long DestWidth, long DestHeight,
				  short * SourcePtr, long SourceWidth,
				  long RectStartX, long RectStartY,
				  long RectWidth, long RectHeight)
/*
This function stretchs or shrinks the specified rectanble in the
specifed YUV 422 bitmap to specified output YUV 422 bitmap.

NOTE: since we are using 422 coding, both the input and the output bitmaps
must have widths of an even number of pixels.  (RectStartX can be odd.)

	xs1,ys1 - first point of source rectangle
	xs2,ys2 - second point of source rectangle
	xd1,yd1 - first point of destination rectangle
	xd2,yd2 - second point of destination rectangle
*/
{

	long dx,dy,e,d,dx2;
	dx=DestHeight-1;
	dy=RectHeight-1;
	e=(dy<<1)-dx;
	dx2=dx<<1;
	dy<<=1;
	long ys1 = RectStartY;
	for(d=0;d<=dx;d++) 
	{
		StretchLine422(DestWidth, RectStartX, RectWidth, &DestPtr[d*DestWidth],
											&SourcePtr[ys1*SourceWidth]);
		while (e<0 && d<dx)
		{
			e+=dy;
			memcpy(&DestPtr[(d+1)*DestWidth],&DestPtr[d*DestWidth],DestWidth*2);
			d++;
		}
		while(e>=0)	
		{
			ys1++;
			e-=dx2;
		}
		e+=dy;
	}

}


__inline static void StretchLine422(long DestWidth, long StartX, long SourceWidth,
						  short * DestPtr, short *SourcePtr)

/*
stretch or shrink the specified input line.  Writes to the specified output line.
*/
{
	long dx,dy,e,d,dx2;
	dx=DestWidth-1;
	dy=SourceWidth-1;
	e=(dy<<1)-dx;
	dx2=dx<<1;
	dy<<=1;
	long y1 = StartX;
	for(d=0;d<=dx;d+=2)
	{
		long FirstY1 = y1;

		//find input pixel for next point
		while(e>=0)	{
			y1++;
			e-=dx2;
		}
		e+=dy;

		//copy a pair of points
		if (FirstY1 & 1) {
			DestPtr[d] = (0xff00 & SourcePtr[FirstY1]) + (0x00ff & SourcePtr[FirstY1-1]);
			DestPtr[d+1] =(0xff00 & SourcePtr[y1]) + (0x00ff & SourcePtr[FirstY1]);
		}
		else {
			DestPtr[d] = SourcePtr[FirstY1];
			DestPtr[d+1] =(0xff00 & SourcePtr[y1]) + (0x00ff & SourcePtr[FirstY1+1]);
		}

		//find input pixel for next point
		while(e>=0)	{
			y1++;
			e-=dx2;
		}
		e+=dy;
	}
}

