// YUV frame conversion routines


class FrameConverter
{
public:
	int Init(int shres,int svres,int dhres,int dvres);
	int InitEx(ICDECOMPRESSEX* pIc); 
	virtual void ProcessFrame(UCHAR *Output,const UCHAR *Input);
	virtual DWORD DecodeFrame(DECODE_FRAME *SwDecoder,UCHAR *Output, UCHAR *Input,DWORD InputSizeBytes);
	virtual ~FrameConverter();
	FrameConverter();

//	virtual Begin()=0;
	virtual void ProcessPixel(UCHAR *Output,const UCHAR *Input);
	DWORD GetOutputFrameSize();
	DWORD GetInputFrameSize();
	int GetInputBitsPerPixel() {return InputBitsPerPixel;}  
	int GetOutputBitsPerPixel() {return OutputBitsPerPixel;}  
	FOURCC GetCompression() {return Compression;};
	int xres;  // image width
	int yres;	// image height
protected:
	virtual void ProcessLine(UCHAR *pOutput,const UCHAR *pInput);
	void StretchLineUYVY(UCHAR *pOutput,const UCHAR *pInput);
	void StretchFirstProcessLine(UCHAR *pOutput,const UCHAR *pInput,UCHAR *buffer);
	void ProcessLineFirstStretch(UCHAR *pOutput,const UCHAR *pInput,UCHAR *buffer);
	virtual void StretchProcessLine(UCHAR *pOutput,const UCHAR *pInput,UCHAR *buffer)
		{StretchFirstProcessLine(pOutput,pInput,buffer);} // usable for UYVY input
	virtual void InitBlankLine() {};

	int hdisplay;  // display width
	int vdisplay; // display width
	int hstart; // display start  for windowing
	int vstart; // display start
	int dyDst;  // ouput size to stretch to
	int dxDst; // ouput size to stretch to
	int InputBytesPerPixel;    
	int OutputBytesPerPixel;    
	int PixelsPerProcess;
	int InputBitsPerPixel;
	int OutputBitsPerPixel;
	int Compression;
	int Direction; // 1=from dib 2=to dib
	BOOL mMustStretch;
	BOOL mCanBlank;
	BOOL invert;
	UCHAR *TempBuffer;
	BOOL SoftwareDecodePostprocNeeded;
	ESWDecodeOutputType SwDecodeOutputType;
	static unsigned char ClipTable[1024];	//table to clip to 8 bits
	static unsigned char *clip;				//0 point of table;
	static short CrToR[256];				//look up Cr to red value (YUV to RGB conversion)
	static short CrToG[256];				//look up Cr to green value (YUV to RGB conversion)
	static short CbToB[256];				//look up Cb to red blue (YUV to RGB conversion)
	static short CbToG[256];				//look up Cb to red green (YUV to RGB conversion)
	static int ClipInited;

	UCHAR *mBlankLine;
};
/*
class GenericConverter: public FrameConverter
{
public:
	GenericConverter();
	virtual void ProcessFrame(UCHAR *Output,const UCHAR *Input);
protected:
	virtual void ProcessPixel(UCHAR *Output,const UCHAR *Input);
	int srcPixelSize;
	int destPixelSize;
};
*/




class FromRGBToYuv: public FrameConverter
{
public:
	FromRGBToYuv();
//	virtual Begin()=0;
//	virtual ProcessFrame(UCHAR *Output,const UCHAR *Input);
protected:
	virtual void StretchProcessLine(UCHAR *pOutput,const UCHAR *pInput,UCHAR *buffer)
		{ProcessLineFirstStretch(pOutput,pInput,buffer);} 
	virtual void ProcessPixel(UCHAR *Output,const UCHAR *Input);
//	virtual ~FromRGBToYuv();
	inline void  rgb2yuv422(const unsigned char *lppic , unsigned char *w );
};

class FromRGB16ToYuv: public FromRGBToYuv 
{
public:
	FromRGB16ToYuv();
//	virtual void ProcessPixel(UCHAR *Output,const UCHAR *Input);
//	virtual Begin()=0;
	//virtual ProcessFrame(UCHAR *Output,const UCHAR *Input);
//	virtual ~FromRGB32ToYuv();
protected:
	virtual void ProcessPixel(UCHAR *Output,const UCHAR *Input);
//	virtual void ProcessLine(UCHAR *pOutput,const UCHAR *pInput);
private:
};

class FromRGB32ToYuv: public FromRGBToYuv 
{
public:
	FromRGB32ToYuv();
//	virtual void ProcessPixel(UCHAR *Output,const UCHAR *Input);
//	virtual Begin()=0;
	//virtual ProcessFrame(UCHAR *Output,const UCHAR *Input);
//	virtual ~FromRGB32ToYuv();
protected:
	virtual void ProcessLine(UCHAR *pOutput,const UCHAR *pInput);
};

class FromYuvToRGB: public FrameConverter
{
public:
	FromYuvToRGB();
//	virtual void ProcessFrame(UCHAR *Output,const UCHAR *Input);
private:
//	inline void OutputPixel(UCHAR *Output,const UCHAR *Input);
protected:
	virtual void ProcessLine(UCHAR *pOutput,const UCHAR *pInput);
};

class FromYuvToRGB32: public FrameConverter
{
public:
	FromYuvToRGB32();
//	virtual void ProcessFrame(UCHAR *Output,const UCHAR *Input);
private:
//	inline void OutputPixel(UCHAR *Output,const UCHAR *Input);
protected:
	virtual void ProcessLine(UCHAR *pOutput,const UCHAR *pInput);
};

class FromYUY2ToRGB24: public FromYuvToRGB
{
public:
	FromYUY2ToRGB24();
	virtual void InitBlankLine();
	int lastxres;
protected:
	virtual void ProcessLine(UCHAR *pOutput,const UCHAR *pInput);
};

class FromYUY2ToRGB32: public FromYuvToRGB32
{
public:
	FromYUY2ToRGB32();
	virtual void InitBlankLine();
	int lastxres;
protected:
	virtual void ProcessLine(UCHAR *pOutput,const UCHAR *pInput);
};




class FromYuvToYUY2: public FrameConverter
{
public:
	FromYuvToYUY2();
protected:
	virtual void ProcessLine(UCHAR *pOutput,const UCHAR *pInput);
};

class FromYuvToUYVY: public FrameConverter
{
public:
	FromYuvToUYVY();
protected:
	virtual void StretchProcessLine(UCHAR *pOutput,const UCHAR *pInput,UCHAR *buffer)
		{StretchLineUYVY(pOutput,pInput);}
	virtual void ProcessLine(UCHAR *pOutput,const UCHAR *pInput);
};

class FromYUY2ToUYVY: public FrameConverter
{
public:
	FromYUY2ToUYVY();
	virtual void InitBlankLine();
	int lastxres;
protected:
	virtual void ProcessLine(UCHAR *pOutput,const UCHAR *pInput);
	virtual void StretchProcessLine(UCHAR *pOutput,const UCHAR *pInput,UCHAR *buffer)
		{ProcessLineFirstStretch(pOutput,pInput,buffer);} // usable for UYVY input
};

class FromYUY2ToYUY2: public FromYUY2ToUYVY
{
public:
	FromYUY2ToYUY2();
	virtual void InitBlankLine();
	int lastxres;
protected:
	virtual void ProcessLine(UCHAR *pOutput,const UCHAR *pInput);
};

class FromYuvToRGB16: public FrameConverter
{
public:
	FromYuvToRGB16(BOOL BuildTable=TRUE);
	~FromYuvToRGB16();
//	virtual void ProcessFrame(UCHAR *Output,const UCHAR *Input);
protected:
	WORD *Table;
#ifdef TWO16BITTABLES
	WORD *TableA;
#endif
	UCHAR YDitherTable[256];
	UCHAR RoundClipShift3[256];
	void BuildDitherTable();
	int DitherCount;
	virtual void ProcessLine(UCHAR *pOutput,const UCHAR *pInput);
	void BuildConversionTable();
};

class FromYuvToRGBBF16: public FromYuvToRGB16
	// currently differs only in table from FromYuvToRGB16

{
public:
	FromYuvToRGBBF16();
//	~FromYuvToRGBBF16();
//	virtual void ProcessFrame(UCHAR *Output,const UCHAR *Input);
protected:
	void BuildConversionTable();
//	WORD *Table;
//	UCHAR YDitherTable[256];
//	int DitherCount;
//	virtual void ProcessLine(UCHAR *pOutput,const UCHAR *pInput);
};

class FromYUY2ToRGBBF16: public FromYuvToRGBBF16
{
public:
	FromYUY2ToRGBBF16();
	virtual void InitBlankLine();
	int lastxres;
protected:
	virtual void ProcessLine(UCHAR *pOutput,const UCHAR *pInput);
};

class FromYUY2ToRGB16: public FromYuvToRGB16
{
public:
	FromYUY2ToRGB16();
	virtual void InitBlankLine();
	int lastxres;
protected:
	virtual void ProcessLine(UCHAR *pOutput,const UCHAR *pInput);
};