/* getvlc.c, variable length decoding                                       */

/* Copyright (C) 1994, MPEG Software Simulation Group. All Rights Reserved. */

/*
 * Disclaimer of Warranty
 *
 * These software programs are available to the user without any license fee or
 * royalty on an "as is" basis.  The MPEG Software Simulation Group disclaims
 * any and all warranties, whether express, implied, or statuary, including any
 * implied warranties or merchantability or of fitness for a particular
 * purpose.  In no event shall the copyright-holder be liable for any
 * incidental, punitive, or consequential damages of any kind whatsoever
 * arising from the use of these programs.
 *
 * This disclaimer of warranty extends to the user of these programs and user's
 * customers, employees, agents, transferees, successors, and assigns.
 *
 * The MPEG Software Simulation Group does not represent or warrant that the
 * programs furnished hereunder are free of infringement of any third-party
 * patents.
 *
 * Commercial implementations of MPEG-1 and MPEG-2 video, including shareware,
 * are subject to royalty fees to patent holders.  Many of these patents are
 * general enough such that they are unavoidable regardless of implementation
 * design.
 *
 */

#include <stdio.h>

#include "config.h"
#include "global.h"
#include "getvlc.h"


int getMBA()
{
  int code, val;

  if (trace)
    printf("macroblock_address_increment (");

  val = 0;

  while ((code = showbits(11))<24)
  {
    if (code!=15) /* if not macroblock_stuffing */
    {
      if (code==8) /* if macroblock_escape */
      {
        if (trace)
          printf("00000001000 ");

        val+= 33;
      }
      else
      {
        if (!quiet)
          fprintf(stderr,"Invalid macroblock_address_increment code\n");

        fault = 1;
        return 1;
      }
    }
    else /* macroblock suffing */
    {
      if (trace)
        printf("00000001111 ");
    }

    flushbits(11);
  }

  if (code>=1024)
  {
    flushbits(1);
    if (trace)
      printf("1): %d\n",val+1);
    return val + 1;
  }

  if (code>=128)
  {
    code >>= 6;
    flushbits(MBAtab1[code].len);

    if (trace)
    {
      printbits(code,5,MBAtab1[code].len);
      printf("): %d\n",val+MBAtab1[code].val);
    }

    return val + MBAtab1[code].val;
  }

  code-= 24;
  flushbits(MBAtab2[code].len);

  if (trace)
  {
    printbits(code+24,11,MBAtab2[code].len);
    printf("): %d\n",val+MBAtab2[code].val);
  }

  return val + MBAtab2[code].val;
}

int getDClum()
{
  int code, size, val;

  if (trace)
    printf("dct_dc_size_luminance: (");

  /* decode length */
  code = showbits(5);

  if (code<31)
  {
    size = DClumtab0[code].val;
    flushbits(DClumtab0[code].len);

    if (trace)
    {
      printbits(code,5,DClumtab0[code].len);
      printf("): %d",size);
    }
  }
  else
  {
    code = showbits(9) - 0x1f0;
    size = DClumtab1[code].val;
    flushbits(DClumtab1[code].len);

    if (trace)
    {
      printbits(code+0x1f0,9,DClumtab1[code].len);
      printf("): %d",size);
    }
  }

  if (trace)
    printf(", dct_dc_differential (");

  if (size==0)
    val = 0;
  else
  {
    val = getbits(size);
    if (trace)
      printbits(val,size,size);
    if ((val & (1<<(size-1)))==0)
      val-= (1<<size) - 1;
  }

  if (trace)
    printf("): %d\n",val);

  return val;
}

int getDCchrom()
{
  int code, size, val;

  if (trace)
    printf("dct_dc_size_chrominance: (");

  /* decode length */
  code = showbits(5);

  if (code<31)
  {
    size = DCchromtab0[code].val;
    flushbits(DCchromtab0[code].len);

    if (trace)
    {
      printbits(code,5,DCchromtab0[code].len);
      printf("): %d",size);
    }
  }
  else
  {
    code = showbits(10) - 0x3e0;
    size = DCchromtab1[code].val;
    flushbits(DCchromtab1[code].len);

    if (trace)
    {
      printbits(code+0x3e0,10,DCchromtab1[code].len);
      printf("): %d",size);
    }
  }

  if (trace)
    printf(", dct_dc_differential (");

  if (size==0)
    val = 0;
  else
  {
    val = getbits(size);
    if (trace)
      printbits(val,size,size);
    if ((val & (1<<(size-1)))==0)
      val-= (1<<size) - 1;
  }

  if (trace)
    printf("): %d\n",val);

  return val;
}
