#include <stdio.h>
#include <stdlib.h>

#include "config.h"
#include "global.h"


void InitRGBTabels(void)
/*
This function inits the 4 256 point tables that are used in YUV to RGB conversion.
It returns 1 if successfull, 0 otherwise.

*/
{
	int val;

	for (val=0; val<256; val++) {
		CrToR[val] = 128 + (short)( 1.371*(val-128));
		CrToG[val] = 128 + (short)(-0.698*(val-128));
		CbToB[val] = 128 + (short)( 1.732*(val-128));
		CbToG[val] =       (short)(-0.336*(val-128));
	}
	
}

__inline static void OutputRGB24(RGB24_PIXEL* pOut, short Y, 
													short CrR, short CrG, short CbB, short CbG)
/*
function to output a 24 bit RGB value given the input lum and the crom factors
*/
{
	pOut->red = clp[Y + CrR];
	pOut->green = clp[Y + CrG + CbG];
	pOut->blue = clp[Y + CbB];
}

void CovertBlockToRGB24(int bx, int by)
/*
This function is used to convert the 6 decoded blocks associated with the current
macroblock into RGB color values.  The functions outputs the data in the correct
locattion on the global RGB frame array.  This code computs 4 points at a time
so that we can pre-compute part of the color calc once for all 4 points.

This function makes use of lookup tables to compute:
	R = Y + 1.371 V
	G = Y - 0.698 V - 0.336 U
	B = Y + 1.732 U
where V = Cr and U = Cb
*/

{
	short *CrBlock;
	short *CbBlock;
	short *IBlock;
	RGB24_PIXEL *rfp;
	int comp,i,j;

#if 1
	//set values to account for windows upside-down dibs
	int iincr = -coded_picture_width;
	int TopRight = 	coded_picture_width * (coded_picture_height-1);
#else
	//set values for upside-right output
	int iincr = coded_picture_width;
	int TopRight = 	0;
#endif

	for (comp=0; comp<4; comp++) { //for each lum block

		//compute pointers
		IBlock = ld->block[comp];
		rfp = &RGB24Frame[TopRight + iincr*(by+ ((comp&2)<<2)) + bx + ((comp&1)<<3)];
		CbBlock = ld->block[4] + ((comp&1)<<2) + ((comp&2)<<4);
		CrBlock = ld->block[5] + ((comp&1)<<2) + ((comp&2)<<4);

		for (i=0; i<8; i+=2) {  //for each pair of lines
			for (j=0; j<8; j+=2) {	//for each pair of points

				//pre-compute croma offsets
				short Cr = clp[CrBlock[(i<<2) + (j>>1)]+128];
				short Cb = clp[CbBlock[(i<<2) + (j>>1)]+128];
				short CrR = CrToR[Cr];
				short CrG = CrToG[Cr];
				short CbB = CbToB[Cb];
				short CbG = CbToG[Cb];

				//compute 4 output points
				OutputRGB24(&rfp[j],        IBlock[(i<<3) + j  ],    CrR,CrG,CbB,CbG);
				OutputRGB24(&rfp[j+1],      IBlock[(i<<3) + j+1],    CrR,CrG,CbB,CbG);
				OutputRGB24(&rfp[iincr+j],  IBlock[((i+1)<<3) + j  ],CrR,CrG,CbB,CbG);
				OutputRGB24(&rfp[iincr+j+1],IBlock[((i+1)<<3) + j+1],CrR,CrG,CbB,CbG);
			}
			rfp += iincr<<1;

		}
	}
}

__inline static unsigned short BuildRGB16(short Y, short CrR, short CrG, short CbB, short CbG)
/*
function to compute a 555 RGB value given the input lum and the crom factors
*/
{
	unsigned short R,G,B;
	R = ClpRGB16[Y + CrR];
	G = ClpRGB16[Y + CrG + CbG];
	B = ClpRGB16[Y + CbB];

	return (R << 10) | (G << 5) | B;
}
__inline static unsigned short BuildRGB16o(short Y, short CrR, short CrG, short CbB, short CbG)
/*
function to compute a 555 RGB value given the input lum and the crom factors
This function is intended to be used on odd pixels, it adds a dithering bit
= 1/2 the LSB before truncating
*/
{
	unsigned short R,G,B;
	Y+=4;
	R = ClpRGB16[Y + CrR];
	G = ClpRGB16[Y + CrG + CbG];
	B = ClpRGB16[Y + CbB];

	return (R << 10) | (G << 5) | B;
}

void CovertBlockToRGB16(int bx, int by)
/*
This function is used to convert the 6 decoded blocks associated with the current
macroblock into RGB color values.  The functions outputs the data in the correct
locattion on the global RGB frame array.  This code computs 4 points at a time
so that we can pre-compute part of the color calc once for all 4 points.

This function makes use of lookup tables to compute:
	R = Y + 1.371 V
	G = Y - 0.698 V - 0.336 U
	B = Y + 1.732 U
where V = Cr and U = Cb
*/

{
	short *CrBlock;
	short *CbBlock;
	short *IBlock;
	short *rfp;
	int comp,i,j;

#if 1
	//set values to account for windows upside-down dibs
	int iincr = -coded_picture_width;
	int TopRight = 	coded_picture_width * (coded_picture_height-1);
#else
	//set values for upside-right output
	int iincr = coded_picture_width;
	int TopRight = 	0;
#endif

	for (comp=0; comp<4; comp++) { //for each lum block

		//compute pointers
		IBlock = ld->block[comp];
		rfp = &Output16Frame[TopRight + iincr*(by+ ((comp&2)<<2)) + bx + ((comp&1)<<3)];
		CbBlock = ld->block[4] + ((comp&1)<<2) + ((comp&2)<<4);
		CrBlock = ld->block[5] + ((comp&1)<<2) + ((comp&2)<<4);

		for (i=0; i<8; i+=2) {  //for each pair of lines
			for (j=0; j<8; j+=2) {	//for each pair of points

				//pre-compute croma offsets
				short Cr = clp[CrBlock[(i<<2) + (j>>1)]+128];
				short Cb = clp[CbBlock[(i<<2) + (j>>1)]+128];
				short CrR = CrToR[Cr];
				short CrG = CrToG[Cr];
				short CbB = CbToB[Cb];
				short CbG = CbToG[Cb];

				//compute 4 output points
				rfp[j] =   BuildRGB16(IBlock[(i<<3) + j  ],CrR,CrG,CbB,CbG);
				rfp[j+1] = BuildRGB16o(IBlock[(i<<3) + j+1],CrR,CrG,CbB,CbG);
				rfp[iincr+j] =   BuildRGB16o(IBlock[((i+1)<<3) + j  ],CrR,CrG,CbB,CbG);
				rfp[iincr+j+1] = BuildRGB16(IBlock[((i+1)<<3) + j+1],CrR,CrG,CbB,CbG);
			}
			rfp += iincr<<1;

		}
	}
}

__inline static unsigned short Range(short val,short _min,short _max)
{
	return ( val>_max ? _max : (val <_min ? _min : val));
}
__inline static unsigned short BuildRGBBF16(short Y, short CrR, short CrG, short CbB, short CbG)
/*
function to compute a 555 RGB value given the input lum and the crom factors
*/
{
	unsigned short R,G,B;
	R = ClpRGB16[Y + CrR];
	G = Range((short)((Y + CrG + CbG)>>2),0,63);
	B = ClpRGB16[Y + CbB];

	return (R << 11) | (G << 5) | B;
}
__inline static unsigned short BuildRGBBF16o(short Y, short CrR, short CrG, short CbB, short CbG)
/*
function to compute a 555 RGB value given the input lum and the crom factors
This function is intended to be used on odd pixels, it adds a dithering bit
= 1/2 the LSB before truncating
*/
{
	unsigned short R,G,B;
	Y+=4;
	R = ClpRGB16[Y + CrR];
	G = Range((short)((Y + CrG + CbG)>>2),0,63);
	B = ClpRGB16[Y + CbB];

	return (R << 11) | (G << 5) | B;
}
void CovertBlockToRGBBF16(int bx, int by)
/*
This function is used to convert the 6 decoded blocks associated with the current
macroblock into RGB color values.  The functions outputs the data in the correct
locattion on the global RGB frame array.  This code computs 4 points at a time
so that we can pre-compute part of the color calc once for all 4 points.

This function makes use of lookup tables to compute:
	R = Y + 1.371 V
	G = Y - 0.698 V - 0.336 U
	B = Y + 1.732 U
where V = Cr and U = Cb
*/

{
	short *CrBlock;
	short *CbBlock;
	short *IBlock;
	short *rfp;
	int comp,i,j;

#if 1
	//set values to account for windows upside-down dibs
	int iincr = -coded_picture_width;
	int TopRight = 	coded_picture_width * (coded_picture_height-1);
#else
	//set values for upside-right output
	int iincr = coded_picture_width;
	int TopRight = 	0;
#endif

	for (comp=0; comp<4; comp++) { //for each lum block

		//compute pointers
		IBlock = ld->block[comp];
		rfp = &Output16Frame[TopRight + iincr*(by+ ((comp&2)<<2)) + bx + ((comp&1)<<3)];
		CbBlock = ld->block[4] + ((comp&1)<<2) + ((comp&2)<<4);
		CrBlock = ld->block[5] + ((comp&1)<<2) + ((comp&2)<<4);

		for (i=0; i<8; i+=2) {  //for each pair of lines
			for (j=0; j<8; j+=2) {	//for each pair of points

				//pre-compute croma offsets
				short Cr = clp[CrBlock[(i<<2) + (j>>1)]+128];
				short Cb = clp[CbBlock[(i<<2) + (j>>1)]+128];
				short CrR = CrToR[Cr];
				short CrG = CrToG[Cr];
				short CbB = CbToB[Cb];
				short CbG = CbToG[Cb];

				//compute 4 output points
				rfp[j] =   BuildRGBBF16(IBlock[(i<<3) + j  ],CrR,CrG,CbB,CbG);
				rfp[j+1] = BuildRGBBF16o(IBlock[(i<<3) + j+1],CrR,CrG,CbB,CbG);
				rfp[iincr+j] =   BuildRGBBF16o(IBlock[((i+1)<<3) + j  ],CrR,CrG,CbB,CbG);
				rfp[iincr+j+1] = BuildRGBBF16(IBlock[((i+1)<<3) + j+1],CrR,CrG,CbB,CbG);
			}
			rfp += iincr<<1;

		}
	}
}



void CovertBlockToUYVY(int bx, int by)
/*
This function is used to convert the 6 decoded blocks associated with the current
macroblock into UYVY (YUV 422) color values.  The functions outputs the data in the correct
locattion on the global YUV frame array.  This code computs 4 points at a time
so that we can pre-compute part of the color calc once for all 4 points.

note: V = Cr and U = Cb
*/

{
	short *CrBlock;
	short *CbBlock;
	short *IBlock;
	short *rfp;
	int comp,i,j;

#if 0
	//set values to account for windows upside-down dibs
	int iincr = -coded_picture_width;
	int TopRight = 	coded_picture_width * (coded_picture_height-1);
#else
	//set values for upside-right output
	int iincr = coded_picture_width;
	int TopRight = 	0;
#endif

	for (comp=0; comp<4; comp++) { //for each lum block

		//compute pointers
		IBlock = ld->block[comp];
		rfp = &Output16Frame[TopRight + iincr*(by+ ((comp&2)<<2)) + bx + ((comp&1)<<3)];
		CbBlock = ld->block[4] + ((comp&1)<<2) + ((comp&2)<<4);
		CrBlock = ld->block[5] + ((comp&1)<<2) + ((comp&2)<<4);

		for (i=0; i<8; i+=2) {  //for each pair of lines
			for (j=0; j<8; j+=2) {	//for each pair of points

				//fetch chrom values
				short Cb = clp[CbBlock[(i<<2) + (j>>1)]+128];
				short Cr = clp[CrBlock[(i<<2) + (j>>1)]+128];

				//compute 4 output points
				rfp[j] =         (clp[ IBlock[(i<<3)     + j  ] + 128] << 8) + Cb;
				rfp[j+1] =       (clp[ IBlock[(i<<3)     + j+1] + 128] << 8) + Cr;
				rfp[iincr+j] =   (clp[ IBlock[((i+1)<<3) + j  ] + 128] << 8) + Cb;
				rfp[iincr+j+1] = (clp[ IBlock[((i+1)<<3) + j+1] + 128] << 8) + Cr;
			}
			rfp += iincr<<1;

		}
	}
}

