/*delay.cpp
*/

#include <memory.h>
#include "delay.h"


CDelay::CDelay(int BufferSize/*=36*32*/, int DelaySize/*=256-64*/)
{
	BuffSize = BufferSize;
	delay = DelaySize;

	//set delay line size to number of full buffers >= buffsize + delay
	LineSize = BuffSize + delay;
	LineSize = BuffSize * ((LineSize + BuffSize - 1)/BuffSize);

	//init memory and pointers
	DelayLine = new float[LineSize];
	int i;
	for (i=0; i<LineSize; i++)
		DelayLine[i] = 0.0F;

	LastOutBuff = &DelayLine[LineSize-BuffSize];			//pointer to the last output buffer
	LastCopySize = LineSize - delay - BuffSize;

	inptr = &DelayLine[delay];
	outptr = LastOutBuff;
}



CDelay::~CDelay()
{
	delete [] DelayLine;
}

void CDelay::AddData(float *NewInput)
/*
add input to the delay line
*/
{
	//copy data into delay line
	if (inptr <= LastOutBuff) {
		memcpy(inptr, NewInput, BuffSize * sizeof(*inptr));
		inptr += BuffSize;
	}
	else {
		//last buffer, copy in 2 chunks
		memcpy(inptr, NewInput, LastCopySize * sizeof(*inptr));
		inptr = DelayLine;
		memcpy(inptr, &NewInput[LastCopySize], delay * sizeof(*inptr));
		inptr += delay;
	}

	//adjust output pointer
	outptr += BuffSize;
	if (outptr > LastOutBuff)
		outptr = DelayLine;
}

