#ifndef __CONVAUD__
#define __CONVAUD__
/*ConvAud.h
This file contains interface of the code to convert audio formats
*/
#include <math.h>

typedef float SAMPLE[2];


BOOL ConvertAudio(PAVISTREAM InStream, PAVISTREAM OutStream);
long ConvertAudioSamples(SAMPLE *buff, void *RawOutData, 
						 WAVEFORMATEX * pFmtData, long NumSamples);

//class to convert sample rates
class CConvertRate {
	//interface
	public:
	CConvertRate(double InRate, double OutRate);
	~CConvertRate();
	long Convert(SAMPLE *InBuff, SAMPLE *OutBuff, int channel, long InSize);
	long GetMaxOutSize(int InSize) {return (long)ceil(m_OutputScale * InSize);}
	long GetReqInSize(int OutSize) {return (long)ceil(m_InputScale * OutSize);}
	enum {FILTER_LENGTH = 32};

	//helper functions
	private:
	void InitFilter(float cutoff, float CenterFreq, float FilterGain);
	BOOL filter (SAMPLE *InBuff, SAMPLE *OutBuff, int channel, long InputSize);

	//member data
	double m_OutputScale;				//out sample rate / in sample rate
	double m_InputScale;				//in sample rate / out sample rate
	double m_CurInSample;				//exact input sample locatoin tha corresponds to the 
										//current output sample
	SAMPLE m_InterpolSave;				//saved value from prev block for interpolation
	float m_coefs[FILTER_LENGTH];		//filter coefs
	SAMPLE m_FilterData[FILTER_LENGTH];	//filter state data (1 set for each channel)
	SAMPLE *m_FiltBuff;					//storage for filter input data
	long m_FiltBuffSize;				//size of m_FiltData
};


/*
class to access samples to & from an audio avi stream
samples are read and written as floating point stereo,
and are converted to the stream data type:
8/16 bit, mono/sterio
*/
class CAccessSamples {

	//interface functions
	public:
	CAccessSamples() {m_stream = NULL; m_RawData = NULL; m_RawDataSize = 0;}
	~CAccessSamples() {if (m_RawData != NULL) free(m_RawData);}
	BOOL init(PAVISTREAM AudioStream);
	long outVert(SAMPLE *buff, long size);
	long read(SAMPLE *buff, long StartSample, long size);
	long write(SAMPLE *buff, long StartSample, long size);
	double GetSampleRate(void) {return (double)m_FmtData.nSamplesPerSec;}
    BOOL IsMono(void) {return (m_FmtData.nChannels == 1);}
	long	GetRawDataSize(){return m_RawDataSize;}
	char*	GetRawData(){return m_RawData;}

    //helper functions
    private:
    void ReadChannel(int channel, long NumSamples, SAMPLE *buff);

	//member data
	private:
    char *m_RawData;
    long m_RawDataSize;
	PAVISTREAM m_stream;
    WAVEFORMATEX m_FmtData;
};

/*
class to read data form a AVI audio stream.  This class automatacly
performs sample rate conversion
*/
class CConvertStream {
	//interface
	public:
	CConvertStream()  {m_RateConvert = NULL;}
	~CConvertStream();
	BOOL init(PAVISTREAM AudioStream, double OutputRate);
	long read(SAMPLE *buff, long size);	 //NOTE: BUFF MUST BE AT LEAST size+1 SAMPLES LONG!
	long GetSamplesRead() {return m_sample;}

	//member data
	private:
	CAccessSamples m_access;		//data access object
	CConvertRate *m_RateConvert;	//rate convert object
	long m_sample;					//current input sample
	double m_OutputRate;			//output sample sample rate
	SAMPLE m_OldSample;				//sample from last read
	BOOL m_OldSampleValid;			//TRUE if m_OldSample is valid
	SAMPLE *m_ConvData;				//convertion data buffer
	long m_ConvDataSize;			//size of conversion buffer in samples
	BOOL m_SameRate;                //TRUE if output rate = input rate
};


#endif //#ifndef __CONVAUD__
