#ifndef __IBPRECODE__
#define __IBPRECODE__
/* recode.h
This file contains the interface for the CIBPRecode class. This class is used to 
compress an I-frame mpeg avi file into a full mpeg system stream file.
*/
#include "video.h"
#include "stream\critsec.h"
#include "audio\encode.h"
#include "rcuser.h"
#include "vcstruct.h"
#include "vcuser.h"
#include "bwuser.h"
#include "vidtool.h"
#include "bwdlgs.h"
#include <time.h>

#ifdef BUILD_RECODE_DLL
#define RECODE_INTERFACE __declspec(dllexport) __stdcall
#else
#define RECODE_INTERFACE __declspec(dllimport) __stdcall
#endif

class CAudioEncode;

class CIBPRecode {
	public:

	//constructor, takes board and dma buffer info 
	RECODE_INTERFACE CIBPRecode (			// dma info no longer needed or used
                 DEVID DevID);				//handle to VT_access board object

	
	//distructor
	RECODE_INTERFACE ~CIBPRecode(void);

	/*
	worker function, takes parametes as specifed.	This funciton returns
	the appropriate error code, VT_NO_ERROR (0) if success
	*/
	ECODE RECODE_INTERFACE Compress(
	//interface parameters
		const char *AviFileName, //full path name of input avi file
		const char *MpgFileName, //full path name of output mpg file
		HWND hWnd,				 //window handle of parrent window for progress dialog

		//audio re-compression params
	    int AudioMode,          //none, mono, stereo, daul channel, joint stereo (see bwuser.h)
		int AudioDataRate,		//output audio data rate in kbits/second (1000 bits/second)

		//video re-compression params
		int VideoDataRate,		//output video rate in kbits/second	(1000 bits/second)
		BOOL UseQSIF,			//TRUE to select QSIF resolution, FALSE to select SIF

		//stream mix parameters
        EStreamMixType StreamMixType,  //type of mixing, normal, none, or VideoCD
		double StreamDataRate,  //total stream data rate in bytes/second, 0 = use optimum rate
		int PackSize,			//in bytes
		int PacketsPerPack,		//number of data packets per pack:
								// 0 => use 1 audio packet and 1 video packet in each pack
		double AudioDelay);		//delay audio form video (in seconds, can be negative)

	/*
	worker function, takes parametes as specifed.	This funciton returns
	the appropriate error code, VT_NO_ERROR (0) if success
	*/
	ECODE RECODE_INTERFACE Compress(
	//interface parameters
		PAVISTREAM pAudioStream, //input audio stream
		PAVISTREAM pVideoStream, //input video stream
		const char *MpgFileName, //full path name of output mpg file
		HWND hWnd,				 //window handle of parrent window for progress dialog

		//audio re-compression params
	    int AudioMode,          //none, mono, stereo, daul channel, joint stereo (see bwuser.h)
		int AudioDataRate,		//output audio data rate in kbits/second (1000 bits/second)

		//video re-compression params
		int VideoDataRate,		//output video rate in kbits/second	(1000 bits/second)
		BOOL UseQSIF,			//TRUE to select QSIF resolution, FALSE to select SIF

		//stream mix parameters
        EStreamMixType StreamMixType,  //type of mixing, normal, none, or VideoCD
		double StreamDataRate,  //total stream data rate in bytes/second, 0 = use optimum rate
		int PackSize,			//in bytes
		int PacketsPerPack,		//number of data packets per pack:
								// 0 => use 1 audio packet and 1 video packet in each pack
		double AudioDelay);		//delay audio form video (in seconds, can be negative)


	//wait for compression to finish, return compleation status
	ECODE RECODE_INTERFACE WaitForCompress(void);
    
	private:

    //helper functions
    ECODE SetupAudioEncoding(int AudioDataRate, BOOL UseJointStereo);
    static DWORD WINAPI AudioEncodeThread(LPVOID ThrdInfo);
    static DWORD WINAPI StreamMix(LPVOID ThrdInfo);
	static DWORD WINAPI CompressThread(LPVOID ThrdInfo);	   //outputs an audio & a video file
	static DWORD WINAPI CompressThreadMixed(LPVOID ThrdInfo);  //outputs a single system stream file
	void KillThreads(void);


	//progress dialog functions
	static BOOL CALLBACK ProgressDialogProc(HWND hwndDlg, UINT uMsg,
												WPARAM wParam, LPARAM lParam);
	BOOL OnInitDialog(HWND hwndDlg);
	void UpdateProgress(HWND hwndDlg);
	void OnCancel(HWND hwndDlg);


    //member data
	ECODE mStatus;					//compleation status of last compresion
    CRecodeVideo mRecodeVideo;      //object to recode the video
    PAVIFILE mAviFile;              //pointer to the AviFile object
	PAVISTREAM mAudioStream;        //pointer to audio stream object
	PAVISTREAM mVideoStream;        //pointer to video stream object
    CAudioEncode *mAudioEncode;     //pointer to audio encoder object
    HANDLE mhMpgFile;               //file handle to output mpeg system stream file
	HANDLE mAudioThread;			//handle of audio compression thread
	HANDLE mStreamThread;			//handle of stream compression thread
	HANDLE mCompressThread;			//handle of main active compression thread (video is
									//compressed in this thread)
    CCritSec mCritSec;				//critical section object
	int mStreamMixTime;				//estimate of stream mixing time in seconds

	int mAudioFractionDone;			//fraction done * 0x10000
	time_t	mStartTime;				//starting time of compress
	int mSecondsRemaining;			//seconds left in compressinn
	BOOL	mTimeReset;     // used when recodeding video to reset the start time once
	BOOL	mSecondsAvg;	// mSeconds remaining is inited for averidgeing

	BOOL mAbort;					//TRUE if processing should be aborted
	HCURSOR mOldCursor;				//saves orig cursor when we show wait cursor
	char mAudioFileName[300];		//full path name of output audio mpg file (if \0, don't
									//use temp files)
	BOOL mbStreamBased;             //TRUE if AVI stream based, FALSE if AVI file based

	//compress params
	char mAviFileName[300]; //full path name of input avi file
	char mMpgFileName[300]; //full path name of output mpg file
    BOOL mEnableAudio;      //TRUE if audio processing is enabled
	BOOL mEnableVideo;		//TRUE if video processing is enabled
	int mAudioDataRate;		//output audio data rate in kbits/second (1000 bits/second)
	EStereoType mAudioType;	//audio stereo mode
	BOOL mUseJointStereo;	//if input is stereo, encode in joint-stereo mode if TRUE
	int mVideoDataRate;		//output video rate in bits/second
	BOOL mUseQSIF;			//TRUE to select QSIF resolution, FALSE to select SIF
    EStreamMixType mStreamMixType; //type of mixing, normal, none, or VideoCD
	double mStreamDataRate;  //total stream data rate in bytes/second, 0 = use optimum rate
	int mPackSize;			//in bytes
	int mPacketsPerPack;		//number of data packets per pack
	double mAudioDelay;		//delay audio form video (in seconds, can be negative)
    BW_RECODEOPTIONS mRecodeOptions;  // these are the registry stored values
	BW_RECODE_PREFERENCES mBroadwayPref; // these are the registry stored values
	DEVID    mDevID;  
	CBasePin *mVideoPin;	// direct stream mix pin
	CBasePin *mAudioPin;	// direct stream mix pin

};


#endif  //#ifndef __IBPRECODE__
