// BwHelper.cpp Helper functions for use by the Broadway application.
#ifdef BUILD_DLL
#include "stdincs.h"
extern HANDLE hModule;  // Global handle
#include "bwhelper.h"
#include "resource.h"
#elif defined(NO_MFC)  
#include "bldhelper.h"
extern HANDLE hModule;  // Global handle
#else
#include "stdafx.h"
#define hModule AfxGetResourceHandle( )
#include "bldhelper.h"
#endif

#include <stdlib.h>
//typedef unsigned int uint;
//#define DEVNODE DWORD
//#include <devioctl.h>
//#include <vtdioc.h>

//#include "vt_acc.h"
//#include "vt_regs.h"
//#include "device.h"

//#include "bwhelper.h"
//#include "resource.h"

#if (defined(_DEBUG) && defined(BUILD_DLL))
#ifndef BUILD_DLL
#ifdef _CONSOLE
#define dprintf printf
#include <stdio.h>
#else
#define dprintf / ## / // does not work need to fix
#endif
#else   // ifndef BUILD_DLL
extern "C" void cdecl dprintf(LPSTR szFormat, ...);


#endif

//static char* DebugMessage= "Error %d returned from %s line %d\n";
//static char DebugBuffer[120];

#define ReturnIfError(__func) {ECODE __ecode=__func; \
if (__ecode) {dprintf(DebugMessage,__ecode,__FILE__,__LINE__);return __ecode;} } 


#else  // ifdef _DEBUG
//#define dprintf / ## /
//#define ReturnIfError(__func) {ECODE __ecode=__func; if (__ecode) return __ecode;} 
#endif




#define HARDWARE_TYPE_BROADWAY 0
#define HARDWARE_TYPE_BEGINNER 1

#define MAX_PATHNAME 256
/*
 * read a string parameter from the device's profile. returns FALSE
 * if it fails to read the string.
 */
BOOL 
HW_ReadProfileString(
    LPTSTR pDriverName,
    LPTSTR ValueName,
    LPTSTR ValueString,
    DWORD ValueLength
)
{
    LONG lRet;
    DWORD dwType;
    HKEY hkParams, hkSystem;
    TCHAR RegistryPath[MAX_PATH];

    /*
     * fill the whole string with 0s to ensure that it is null
     * terminated on return
     */
    FillMemory(ValueString, ValueLength, 0);

     /* Create the path to our node */
    lstrcpy(RegistryPath, TEXT("SYSTEM\\CurrentControlSet\\Services\\"));
	lstrcat(RegistryPath, pDriverName);
	lstrcat(RegistryPath, TEXT("\\"));
	lstrcat(RegistryPath, PARMS_SUBKEY);
    hkSystem = HKEY_LOCAL_MACHINE;

    /* See if we can get a registry handle to our device data */
    if (RegOpenKeyEx(
            HKEY_LOCAL_MACHINE,
            RegistryPath,
            0L,
            KEY_QUERY_VALUE,
            &hkParams) != ERROR_SUCCESS) {

        return FALSE;
    }

    lRet = RegQueryValueEx(
            hkParams,
            ValueName,
            NULL,
            &dwType,
            (PUCHAR) ValueString,
            &ValueLength);

        RegCloseKey(hkParams);


    if ((lRet != ERROR_SUCCESS) || (dwType != REG_SZ)) {
        return(FALSE);
    } else {
        return(TRUE);
    }
}

ECODE BW_INTERFACE BroadwayOpenDevice(DEVID *dwDevice,int DeviceIndex)
{
 

//ECODE VT_INTERFACE VT_OpenDevice(LPTSTR pDriverName, DWORD DeviceIndex, DEVID *dwDevice)
    
    PDEVICE_INFO pDevInfo;   
    DWORD dwCount;
    TCHAR szTablePath[MAX_PATHNAME];
    HANDLE hDriver;
	TCHAR FullName[MAX_PATHNAME]=TEXT("\\\\.\\");
	TCHAR pDriverName[]="vtool";
    OSVERSIONINFO VerInfo;
	//dprintf("Open device\n");
	*dwDevice=NULL;
    
    /* 
     * We need to get the platform information to determine 
     * if we are running on WinNT or Win95
     */
    VerInfo.dwOSVersionInfoSize = sizeof(OSVERSIONINFO);
    if (!GetVersionEx(&VerInfo)) {
		return VT_ERR_OTHER;
    }

    /* 
     * NT doesn't recognize the 
     * Win95 Plug & Play device node.
     */
    if (VerInfo.dwPlatformId == VER_PLATFORM_WIN32_NT) {

	    TCHAR DeviceName[MAX_VIDCAP_NAME_LENGTH];

            if (DeviceIndex != 0) {
		        return VT_ERR_BOARD_NOT_FOUND;
            }

            DeviceName[0] = TEXT('\0');
            if (!HW_ReadProfileString(pDriverName, REG_DEVNAME, DeviceName, sizeof(DeviceName))) {
		        return VT_ERR_BOARD_NOT_FOUND;
            }
        
            if (DeviceName[0] == TEXT('\0')) {
		        return VT_ERR_BOARD_NOT_FOUND;
            }

			/* create the full filename */
			lstrcat(FullName, DeviceName);



    } else {
        /*
         * For Win95 the device is identified by the device node, not
         * by the name created by the driver for the device.
         *
         */
		lstrcat(FullName,pDriverName);

		/* vxd name must allways be devicename+d+.vxd */
    	lstrcat(FullName,TEXT("d.vxd"));
    }


	GetSystemDirectory (szTablePath, sizeof(szTablePath));
    lstrcat (szTablePath, "\\BWTABLES\\");

    /*
     * open the kernel driver - we need the overlapped flag
     * so that we can file async requests in the worker thread.
     * all requests for which the OVERLAPPED field is NULL will
     * still be synchronous though
     */

    hDriver = CreateFile(
            FullName,            
            GENERIC_READ,             
            0,
            NULL,
            OPEN_EXISTING,
            0,
            NULL);

    if (hDriver == INVALID_HANDLE_VALUE) {
        return VT_ERR_BOARD_NOT_FOUND; //(VT_ERR_BOARD_NOT_FOUND);
    }

	pDevInfo=new DEVICE_INFO;
	ZeroMemory (pDevInfo, sizeof (DEVICE_INFO));

	/* 
	 * in NT only one board, in Win95 if a DeviceIndex is supplied, open a
	 * specific device.
	 */
    BOOL bFoundBoard = FALSE;
	if (VerInfo.dwPlatformId == VER_PLATFORM_WIN32_WINDOWS) 
	{
				/* query the vxd for all available devices in the system */
		DEVICE_IDS DeviceIDs;
		if (!DeviceIoControl (
				hDriver, 
				IOCTL_VIDTOOL_GET_DEVICE_IDS,
				NULL,
				0,
				&DeviceIDs,
				sizeof(DEVICE_IDS),
				&dwCount,
				NULL)) { 

			CloseHandle (hDriver);
			return VT_ERR_BOARD_NOT_FOUND;
		}
#ifdef _DEBUG
		if (DeviceIDs.dwNumDevNodes>1)
		{
			dprintf("************* Found %d Broadway boards **********\n",DeviceIDs.dwNumDevNodes);
			__asm int 3
		}
#endif

		/* try to find a device thats not busy */

			DeviceIndex = DeviceIDs.DevNodes[DeviceIndex];
		}


			if (!DeviceIoControl (
					hDriver, 
					IOCTL_VIDTOOL_GET_DEVICEINFO,
					&DeviceIndex,
					sizeof(ULONG),
					pDevInfo,
					sizeof(DEVICE_INFO),
					&dwCount,
					NULL)) { 

				delete pDevInfo;
				return VT_ERR_BOARD_NOT_FOUND;
			}

			/* make sure we got our hardware info!!! */
			if (pDevInfo->dwStructSize < sizeof(DEVICE_INFO)) {
				delete pDevInfo;
				return VT_ERR_BOARD_NOT_FOUND;
			} 

			/* we got it!!! */
			bFoundBoard = TRUE;
	
    /* did we get a free device ??? */
    if (!bFoundBoard) {
		CloseHandle (hDriver);
		delete pDevInfo;
        return VT_ERR_BOARD_NOT_FOUND;
    }
	
	/* create the device structure */
	*dwDevice=(DEVID)new CDevice(hDriver,pDevInfo);
	if (*dwDevice==NULL) {
		delete pDevInfo;
		return VT_ERR_OTHER;
	}

	char *tstr=new char[strlen(szTablePath)+1];
	strcpy(tstr,szTablePath);
	(*dwDevice)->szTablePath=tstr;

	// if device is idle initialize it

    return VT_NO_ERROR;
}

ECODE BW_INTERFACE BroadwayCloseDevice(DEVID Device)
{
	ReturnIfError(CDevice::CheckDevice(*Device,FALSE));
	delete (void*)Device->szTablePath;
	delete Device;
	return 0;
}

ECODE BW_INTERFACE BroadwayGetHardwareVersion(DEVID Device,LPSTR StringVersion,int StrLen)
{
	int ver;
	ReturnIfError(CDevice::CheckDevice(*Device,FALSE));
	ver=Device->GetRevisionID();
	itoa(ver,StringVersion,10);
	return VT_NO_ERROR;
}

ECODE BW_INTERFACE BroadwayGetHardwareSerial(DEVID Device,LPSTR StringSerial,int StrLen)
{
	ReturnIfError(CDevice::CheckDevice(*Device,FALSE));
	if (Device->GetRevisionID()>=4 && StrLen>=10)
	{
		DWORD sn;
		Device->ReadFlash(&sn,EEPROM_OFFSET(SerialNumber),4);
		wsprintf(StringSerial,"%08d",sn);
	}
	else
		LoadString(hModule,IDS_UNKNOWN_SN,StringSerial,StrLen);
	return VT_NO_ERROR;
}

ECODE BW_INTERFACE BroadwayGetUpgradeSerial(DEVID dwDevice,LPSTR StringVersion,int StrLen)    // is this needed?
{
	ReturnIfError(CDevice::CheckDevice(*dwDevice,FALSE));
	return VT_NO_ERROR;
}

ECODE BW_INTERFACE BroadwayGetBoardName(DEVID Device,LPSTR StringVersion,int StrLen) // Broadway ,,, Broadway Beginner
{
	ReturnIfError(CDevice::CheckDevice(*Device,FALSE));
	LoadString(hModule,IDS_BROADWAY,StringVersion,StrLen);
	if (Device->GetRevisionID()>=4)
	{
		DWORD type=99;
		ECODE ec=Device->ReadFlash(&type,EEPROM_OFFSET(HardwareType),2);
		if (!ec)
		{
			if (type==0) 
				LoadString(hModule,IDS_BROADWAY,StringVersion,StrLen);
			else
				LoadString(hModule,IDS_BROADWAY_BEGINNER,StringVersion,StrLen);
		}
		else 
		{
			LoadString(hModule,IDS_UNKNOWN_SN,StringVersion,StrLen);
			dprintf("Ecode was %d\n",ec);
		}
	}
	else
		LoadString(hModule,IDS_BROADWAY,StringVersion,StrLen);
	
	return VT_NO_ERROR;
}

ECODE BW_INTERFACE BroadwayGetBoardType(DEVID Device,LPDWORD pType) // Broadway ,,, Broadway Beginner
{
	ReturnIfError(CDevice::CheckDevice(*Device,FALSE));
	if (Device->GetRevisionID()>=4)
	{
		DWORD type=0;
		Device->ReadFlash(&type,EEPROM_OFFSET(HardwareType),2);
		*pType=type;
	}
	else
		pType=HARDWARE_TYPE_BROADWAY;
	return VT_NO_ERROR;
}

ECODE BW_INTERFACE BroadwayGetBoardCaps(DEVID Device,LPDWORD Caps)
{
	*Caps=0;
	static DWORD CapsCache=0;
	ReturnIfError(CDevice::CheckDevice(*Device,FALSE));
	if (CapsCache==0)
	{
		*Caps=CapsCache=Device->GetBoardCaps();
	}
	else
	{
		if (Device->GetGlobalState()==INITIALIZED || Device->GetGlobalState()==CODEC_INITIALIZED)
			*Caps=Device->GetBoardCaps();
		else
			*Caps=CapsCache;
	}
	return VT_NO_ERROR;
}


