#define WIN32_LEAN_AND_MEAN
#include <windows.h>
//#include <vt_regs.h>
#include <bwhelper.h>
#include "..\bldhelper.h"
#include <stdio.h>
#include <stdlib.h>
#include <time.h>
HANDLE hModule=0;

inline ECODE WritePPR(DEVID Device,DWORD Value)
{
	return Device->WriteFlash(0x7ff,&Value,1);
}

#define HARDWARE_REV	4;
#define FLASH_REV		0;
#include "KeyFuncs.h"
#define HARDWARE_TYPE_BROADWAY 0
#define HARDWARE_TYPE_BEGINNER 1

void FillRandom(void *ptr,int size)
{
	srand( (unsigned)time( NULL ) );

	for (int i=0;i<size;i++)
		((BYTE*)ptr)[i]=(rand()>>1) & 0xff;
}
static int CalcChecksum(DWORD value)
{
	int sum=0;
	for (int i=0;i<32;i++,value>>=1)
		if (value & 1) sum++;
	return sum;
}

#define pat 0xfa83de45  // top bits must be f and 28 bits of caps only
extern "C" ECODE __stdcall  BroadwayMakeEprom(int type ,DWORD sn,int rev,void *Eprom,DWORD *Size);
extern "C" ECODE __stdcall BroadwayMakeEprom(int type ,DWORD sn,int rev,void *Eprom,DWORD *Size)
{
	EEPROM *ep=(EEPROM*)Eprom;
	*Size=sizeof(EEPROM);
	FillRandom(ep,sizeof(ep));
	ep->HardwareRev=rev;
	ep->FlashRev=FLASH_REV;
	ep->HardwareType=type; 
	ep->WriteCount=0;
	ep->caps=type== HARDWARE_TYPE_BROADWAY ? BROADWAY_DEFAULT_CAPS
											| BROADWAY_CANCAPTURE_MPEG 
											| BROADWAY_CANOUTPUT_IMPEG
											:BROADWAY_CANCAPTURE_IMPEG 
											| BROADWAY_CANOUTPUT_IMPEG;

	ep->SerialNumber=sn;
	ep->SnKey=MakeKey32(sn); 
	DWORD c1=(sn*rev);
	DWORD c2=ep->caps  | ((CalcChecksum(ep->caps)+CalcChecksum(c1)) << CHECKSUM_LOC);
	UINT64 code=(UINT64)(c1 & pat | c2 & ~pat ) | ((UINT64)(c1 & ~pat | c2 & pat) << 32);
	*(UINT64*)ep->Key=MakeKey(code);
	return 0;
}
#ifdef _CONSOLE

ECODE BW_INTERFACE BroadwayInitHardware0(DEVID Device,int type,DWORD sn)
{
	EEPROM ep;
	char st1[20],st2[20];
	FillRandom(&ep,sizeof(ep));
	ReturnIfError(CDevice::CheckDevice(*Device,FALSE));
	WritePPR(Device,0x2);
	ep.HardwareRev=HARDWARE_REV;
	ep.FlashRev=FLASH_REV;
	ep.HardwareType=type; 
	ep.WriteCount=0;
	ep.caps=type== HARDWARE_TYPE_BROADWAY ? BROADWAY_DEFAULT_CAPS
											| BROADWAY_CANCAPTURE_MPEG 
											| BROADWAY_CANOUTPUT_IMPEG
											:BROADWAY_CANCAPTURE_IMPEG 
											| BROADWAY_CANOUTPUT_IMPEG;

	ep.SerialNumber=sn;
	ep.SnKey=MakeKey32(sn); 
	DWORD c1=(sn*Device->GetRevisionID()) ^ ep.caps;
	DWORD c2=ep.caps;
	UINT64 code=(UINT64)(c1 & pat | c2 & ~pat ) | ((UINT64)(c1 & ~pat | c2 & pat) << 32);
	*(UINT64*)ep.Key=MakeKey(code);
	printf("The Key is %s %s\n",_ultoa(ep.Key[1] ,st1,25),_ultoa(ep.Key[0],st2,26));
	UINT64 rkey=((UINT64)strtoul(st1,NULL,25) << 32) ;
	rkey |= ((UINT64)strtoul(st2,NULL,26));
	if (rkey!=*(UINT64*)ep.Key) 
		printf("Key conversion error");
	Device->WriteFlash(0,(DWORD*)&ep , 32);  // only writes first 32 bytes so far
	DWORD value;
	while (Device->ReadFlash(&value,0X7FF,1)) 
		Sleep(5);
	Device->WriteFlash(32,(DWORD*)ep.Key , 32);  // only writes first 32 bytes so far
	WritePPR(Device,0x0);
	return VT_NO_ERROR;
}

ECODE BW_INTERFACE BroadwayInitHardware(DEVID Device,int type,DWORD sn)
{
	EEPROM ep;
	char st1[20],st2[20];
	DWORD size;
	ReturnIfError(CDevice::CheckDevice(*Device,FALSE));
	BroadwayMakeEprom(type ,sn,Device->GetRevisionID(),&ep,&size);
	WritePPR(Device,0x2);
	printf("The Key is %s %s\n",_ultoa(ep.Key[1] ,st1,25),_ultoa(ep.Key[0],st2,26));
	UINT64 rkey=((UINT64)strtoul(st1,NULL,25) << 32) ;
	rkey |= ((UINT64)strtoul(st2,NULL,26));
	if (rkey!=*(UINT64*)ep.Key) 
		printf("Key conversion error");
	Device->WriteFlash(0,(DWORD*)&ep , 32);  // only writes first 32 bytes so far
	DWORD value;
	while (Device->ReadFlash(&value,0X7FF,1)) 
		Sleep(5);
	Device->WriteFlash(32,(DWORD*)ep.Key , 32);  // only writes first 32 bytes so far
	WritePPR(Device,0x0);
	return VT_NO_ERROR;
}

int __cdecl  main(int argc, char *argv[])
{
	BWDEV Dev;
	char string[128]; 
	if (sizeof(EEPROM)!=64)
		printf("Invalid Eprom size %d\n",sizeof(EEPROM));
	BroadwayOpenDevice(&Dev,0);
	if (!Dev) 
	{
		printf("Broadway board not found\n");
		exit(1);
	}
	if (argc > 1)
	{
		if (argc < 3) 
		{
			printf("This Program requires two paramiters type and sn\n");
			exit(1);
		}
		BroadwayInitHardware(Dev,atoi(argv[1]),atoi(argv[2]));
	}
	if (Dev->CheckValidBoard())
	{
		Dev->SetGlobalState(INITIALIZED);
		BroadwayGetBoardName(Dev,string,128);
		printf("Board is Valid, type is %s\n",string);
		DWORD count;
		Dev->ReadFlash(&count,EEPROM_OFFSET(WriteCount),4);
		printf("Board has been configured %d times\n",count);
	}
	else
		printf("Validation Failed");
	BroadwayCloseDevice(Dev);
	return 0;
}

#endif