// This file handle microcode related access to the C-Cube chip
#include "stdincs.h"
/*
#ifdef VTOOLSD
extern "C" {
#include <vtoolsc.h>
}
#else
#include <stdlib.h>
#endif
//#include "dtdefs.h"

#include "vt_acc.h"
#include "vt_regs.h"
#include "vt_vrp.h"
#include "mcode.h"
*/
#include <math.h>

#include "4100_api.h"
#undef max
__inline unsigned long  max(unsigned long a,unsigned long b)
{
	if (a>b) 
		return a;
	else return b;
}


//#define CONST_COMMAND (size,command,

ECODE CMicrocode::Start()
{	
//	VRP_COMMAND Cmd={EMI_Start,1};
	VRP_COMMAND Cmd={EMI_Start,2,TC_NOW};
	return SendCommand(&Cmd);
}

CMicrocode::~CMicrocode()
{	
	CrPoke(Dev->Regs,CTRL1,0X10); // Disable data
	if (Dev->Busy)
	{
		Sleep(44); // let transfer stop
 		Dev->Busy=FALSE;
	}
	Change32(&Dev->Regs->Csr,MSTR_MODE_MASK,0);
}

ECODE CMicrocode::Stop() 
{
//	VRP_COMMAND Cmd={EMI_Stop,1};
	VRP_COMMAND Cmd={EMI_Stop,2,TC_NOW};
	return SendCommand(&Cmd);
}
ECODE CMicrocode::Pause() 
{
	VRP_COMMAND Cmd={EMR_Pause,1};
	return VT_SendPriorityCommand(Dev,&Cmd);
}

ECODE CMicrocode::Continue() 
{
	VRP_COMMAND Cmd={EMR_Continue,1};
	return VT_SendPriorityCommand(Dev,&Cmd);
}


ECODE CMicrocode::LoadParams(int timecode, int paramID, int data_length, ulong *data)
{
  int   i;
  CVrpCommand cmd(data_length+3,EMR_LoadParams);

  cmd.param(0) = timecode;
  cmd[1] = paramID;

  for(i=0; i<data_length; ++i) 
    cmd[2+i] = data[i];

  return VT_SendCommand(Dev,cmd);
}

int CMicrocode::CalcQuant(int value) 
{
	value=(DWORD)(31*exp(value*-3.43398e-4)+0.5);
	if (value==0) value=1;
	if (value>31) value=31;
	return value;
}

ECODE CMicrocode::SetParam(DWORD param,long  value)
{
	PREGS bd_addr=Dev->Regs;
	DWORD uvalue=(DWORD)value;
	switch (Dev->CurrentMode)
	{
	case VT_IBPENCODE:
	case VT_IENCODE:
	    switch (param)
	    {
		
	    case PASSMODE:   
			if (Dev->Busy) return VT_ERR_HW_BUSY;

	    case DATARATE:	 
		case QUALITY: 	 
		case PADDING: 	 
			break;
 		case HUE:
 			Dev->VipRegs.VipHueCtrl=value;
			if (Dev->VipRegs.VipVer>0)
 				Dev->WriteVIP(VIP_HUE_CTRL, &uvalue, 1);	     
			else
				Dev->WriteVIP(0, &uvalue, 1);// Bug in version 1	     
			break;
 		case SATURATION: 
 			Dev->VipRegs.VipSatCtrl=value;
 			Dev->WriteVIP(VIP_SAT_CTRL, &uvalue, 1);	     
			break;
 		case INTENSITY:	 
 			Dev->VipRegs.VipBrightCtrl=value;
 			Dev->WriteVIP(VIP_BRIGHT_CTRL, &uvalue, 1);	     
			break;
 		case CONTRAST:	 
 			Dev->VipRegs.VipContCtrl=value;
 			Dev->WriteVIP(VIP_CONT_CTRL, &uvalue, 1);	     
			break;
	    case SOURCEISVCR:
		case SHIFTLEFT: 
		case SHIFTUP: 	 
		case BLANKLINES: 
		case FILTERVALUE:
		    break;
		default: 
			return VT_ERR_HW_BUSY;
		}
		break;
	case VT_CODEC:
	    switch (param)
	    {
	    case FRAMERATE:	Dev->CodecInfo.FrameRate=value; break;
	    case DATARATE:	Dev->CodecInfo.DataRate=value; break;
	    case FRAMEWIDTH: Dev->CodecInfo.FrameWidth=(short)value; break;
	    case FRAMEHEIGHT: Dev->CodecInfo.FrameHeight=(short)value; break;
	    case ASPECT:	Dev->CodecInfo.Aspect=value; break;
		case QUALITY: 	Dev->CodecInfo.Quality=value; break;
		case FRAMETYPE:  Dev->CodecInfo.FrameType=value; break;
		case PADDING: 	Dev->CodecInfo.Padding=value; break;
		default: 
			return VT_ERR_HW_BUSY;
		}
		break;
#if 0
	case VT_IDECOMPRESS:
	    switch (param)
	    {
	    case FRAMERATE:	Dev->EditDecompressInfo.FrameRate=value; break;
	    case DATARATE:	Dev->EditDecompressInfo.DataRate=value; break;
	    case FRAMEWIDTH: Dev->EditDecompressInfo.FrameWidth=(short)value; break;
	    case FRAMEHEIGHT: Dev->EditDecompressInfo.FrameHeight=(short)value; break;
	    case ASPECT:	Dev->EditDecompressInfo.Aspect=value; break;
		case QUALITY: 	Dev->EditDecompressInfo.Quality=value; break;
		case FRAMETYPE:  Dev->EditDecompressInfo.FrameType=value; break;
		case PADDING: 	Dev->EditDecompressInfo.Padding=value; break;
		default: 
			return VT_ERR_HW_BUSY;
		}
		break;
#endif
	case VT_IDECODE:
		return VT_ERR_UNSUPPORTED;			 
		break;
	case VT_RECODE:
	    switch (param)
	    {
	    case FRAMERATE:	Dev->RecodeInfo.FrameRate=value; break;
	    case DATARATE:	Dev->RecodeInfo.DataRate=value; break;
	    case FRAMEWIDTH: Dev->RecodeInfo.FrameWidth=(short)value; break;
	    case FRAMEHEIGHT: Dev->RecodeInfo.FrameHeight=(short)value; break;
	    case ASPECT:	Dev->RecodeInfo.Aspect=value; break;
		case QUALITY: 	Dev->RecodeInfo.Quality=value; break;
		case SHIFTUP: 	Dev->RecodeInfo.ShiftUp=value; break;
		case SHIFTLEFT: 	Dev->RecodeInfo.ShiftLeft=value; break;
		case PADDING: 	Dev->RecodeInfo.Padding=(short)value; break;
		case FILTERVALUE: 	Dev->RecodeInfo.FilterValue=value; break;

		default: 
			return VT_ERR_HW_BUSY;
		}
		break;
	default:
		return VT_ERR_HW_BUSY;			 
	}
	return VT_NO_ERROR;
}
/***********************************************************************
 *
 * Load Clipping Tables Command.
 *
 ***********************************************************************/

#define	BLOCK_H	16
#define	BLOCK_V	16
#define ON	1
#define OFF	0
#define XSKIP	8
#define YSKIP	4

	int		vertical_mb_resolution = 15;
	int		horizontal_mb_resolution = 22;
	int		clip_x[720];
	int		clip_y[288];
	int		data[24][64];
#ifdef NOFLIP
typedef struct
	{
	  unsigned long rfxmin	: 3,
			mxmin	: 7,
			mymin	: 6,
			rfxmax	: 3,
			mxmax	: 7,
			mymax	: 6;
	}  meminmax;

#else
typedef struct
	{
	  unsigned long
			mymax	: 6,
			mxmax	: 7,
			rfxmax	: 3,
			mymin	: 6,
			mxmin	: 7,
			rfxmin	: 3;
	}  meminmax;
#endif

meminmax r_minmax;

	int		pixnumx, pixnumy, pixreftop, pixrefbot;
	int		xnum, ynum, bksizex, bksizey;
	int		minx, maxx, miny, maxy, extime;
	int		curnum, refnum;

/*-----------------------------------------------------------------------*/
static void mktablep(int y, int x);
static void mktableb(int y, int x);
void CMicrocode::loadClippingTables(int horizontal, int vertical)
{
  int    horizontal_resolution;
  int    vertical_resolution;
  int	 index;
  ulong	 addr;
  int	 mvx, mvy;

  vertical_mb_resolution = vertical;
  horizontal_mb_resolution = horizontal;

  vertical_resolution = vertical_mb_resolution * 16;
  horizontal_resolution = horizontal_mb_resolution * 16;
  
  for(index = 0; index < horizontal_resolution; index++)  {
    clip_x[index] = (0 << 22) + (7 << 6) + (7 << 13);
  }
  for(index = 0; index < vertical_resolution; index++)  {
    clip_y[index] = (0 << 16) + 7;
  }
  clip_x[0] = (4 << 22) + (7 << 6) + (7 << 13);
  clip_x[horizontal_resolution - BLOCK_H] = (0 << 22) + (4 << 6) + (7 << 13);
  clip_y[0] = (4 << 16) + 7;
  clip_y[vertical_resolution - BLOCK_V] = (0 << 16) + 4;
  VT_Write(Dev,VR_DRAM, m_DA_CLIPX, (DWORD*)clip_x,horizontal_resolution);
  VT_Write(Dev,VR_DRAM, m_DA_CLIPY, (DWORD*)clip_y,vertical_resolution);

  
  for(mvy = 0; mvy < 24; mvy++)  {
    for(mvx = 0; mvx < 48; mvx++)  {
      mktablep(mvy, mvx);
    }
  }
  
  addr = m_DA_PTB;
  for(mvy = 0; mvy < 24; mvy++)  {
    for(mvx = 0; mvx < 64; mvx++)  {
      dramWrite(Dev->Regs, addr++, data[mvy][mvx]);
    }
  }
  
  for(mvy = 0; mvy < 16; mvy++)  {
    for(mvx = 0; mvx < 32; mvx++)  {
      mktableb(mvy, mvx);
    }
  }
  
  addr = m_DA_BTB;
  for(mvy = 0; mvy < 16; mvy++)  {
    for(mvx = 0; mvx < 32; mvx++)  {
      dramWrite(Dev->Regs, addr++, data[mvy][mvx]);
    }
  }
  
  pixnumx = horizontal_resolution / 2;
  pixnumy = vertical_resolution / 2;
  pixreftop = 0;
  pixrefbot = 0;
  bksizex = 8;
  bksizey = 8;
  curnum = 6;
  refnum = 4;
  minx = -24;
  maxx = 23;
  miny = -12;
  maxy = 11;
  extime = 2;
  
  clipme1(m_ME1_CLIP_TABLE_P);
  
  pixnumx = horizontal_resolution / 2;
  pixnumy = vertical_resolution / 2;
  pixreftop = 0;
  pixrefbot = 0;
  bksizex = 8;
  bksizey = 8;
  curnum = 4;
  refnum = 3;
  minx = -16;
  maxx = 15;
  miny = -8;
  maxy = 7;
  extime = 1;
  
  clipme1(m_ME1_CLIP_TABLE_B);
}

/*-----------------------------------------------------------------------*/
static void mktablep(int y, int x)
{
  int	address;
  int	p, xxx, yyy, zzz;
  int	new_y, new_x;
  
  new_y = y + 3;
  new_x = x + 7;
  
  p = new_y / 8;
  yyy = new_y - p * 8;
  
  xxx = new_x / 2;
  zzz = new_x % 2;
  
  address = p * (horizontal_mb_resolution) * 256 
            + xxx * 64 + yyy * 8 + zzz * 4;
  address -= (256 + 64);
  
  data[y][x] = address;
}

/*-----------------------------------------------------------------------*/
static void mktableb(int y, int x)
{
  int	address;
  int	p, xxx, yyy, zzz;
  int	new_y, new_x;
  
  new_y = y + 7;
  new_x = x + 7;
  
  p = new_y / 8;
  yyy = new_y - p * 8;
  
  xxx = new_x / 2;
  zzz = new_x % 2;
  
  address = p * (horizontal_mb_resolution) * 256 
              + xxx * 64 + yyy * 8 + zzz * 4;
  address -= (horizontal_mb_resolution + 1) * 256;
  
  data[y][x] = address;
}
#pragma warning (disable: 4018)
 
/*-----------------------------------------------------------------------*/
void CMicrocode::clipme1(ulong addr)
{
  int x, y;
  int MINX, MAXX, MINY, MAXY ;
  int ovrtopy, ovrboty, right, left, mid, temp, temp2, prev ;
  int clipminx, clipmaxx, clipminy, clipmaxy ;
  int curmax, firstcurnum ;
  ulong dmem;
  PREGS Regs=GET_STRUCT_PTR(Dev);
 
  MINY = (YSKIP+(miny%YSKIP))%YSKIP*2  ;
  MAXY = (MINY + (maxy - miny)*2) + 1 ;
  ovrtopy = ((0 - miny + YSKIP - 1)/YSKIP)*YSKIP ;
  ovrboty = ((0 + maxy + YSKIP - 1)/YSKIP)*YSKIP ;
  
  if(curnum<=4) {
    curmax = 4;
    firstcurnum = 2;
  }
  else if(curnum<=8) {	
    curmax = 8;
    if(extime == 2) 
      firstcurnum = 3;
    if(extime == 3)
      if(curnum != 6) 
	printf(" ERROR !!! ") ;
  }
  else {
    printf("Error:  Over Horizontal Search Range!") ;
    exit(0) ;
  }
  
  MINX = ((curmax-curnum)*XSKIP + (curnum/2)*bksizex + minx)*2 ;
  temp = (curnum/2)*bksizex + maxx ;
  MAXX = ((curmax-curnum+(temp/bksizex))*XSKIP + (temp%bksizex))*2 + 1  ;
  right = ((curmax-curnum+((curnum-2)/2))*XSKIP)*2 ;
  left = ((curmax-((curnum-2)/2))*XSKIP)*2 - 1 ;
  mid = (right+left+1) ;

  xnum = (pixnumx+bksizex-1)/bksizex;;
  ynum = (pixnumy+bksizey-1)/bksizey;;
  
  prev = 0 ;
  for(y=0; y<ynum; y++) {   
    clipminy = (ovrtopy - (y*bksizey) - pixreftop)*2 ;
    clipmaxy = (ovrtopy + pixnumy - ((y+1)*bksizey) + pixrefbot)*2 ;
    for(x=0; x<xnum; x++) {
      clipmaxx = left + (x*XSKIP)*2 ;
      clipminx = right + (((x+2)*bksizex - pixnumx)/bksizex)*XSKIP*2 ;
      r_minmax.rfxmin = 0 ;
      r_minmax.rfxmax = clipminx<=right?(bksizex-1):
	(clipminx-right)>(bksizex-1)?0:((bksizex-1)-(clipminx-right)) ;
      if((curnum<=4) && (refnum<=4) && (extime == 1)) {
	temp = MINY>clipminy?MINY:clipminy ;
	r_minmax.mymax = MAXY<clipmaxy?MAXY:clipmaxy ;
	r_minmax.mymin = r_minmax.mymax<temp?r_minmax.mymax:temp ;
	temp = MINX>clipminx?MINX:mid<clipminx?mid:clipminx ;
	r_minmax.mxmax = MAXX<clipmaxx?MAXX:clipmaxx ;
	r_minmax.mxmin = r_minmax.mxmax<temp?r_minmax.mxmax:temp ;
	
	dmem = *((ulong *)&r_minmax);
	dramWrite(Regs, addr++, dmem);
	
      }
      else if((curnum<=8) && (refnum<=4) && (extime == 2))
	{	temp = MINY>clipminy?MINY:clipminy ;
		r_minmax.mymax = MAXY<clipmaxy?MAXY:clipmaxy ;
		r_minmax.mymin = r_minmax.mymax<temp?r_minmax.mymax:temp ;
		temp = (curmax-firstcurnum)*XSKIP*2 ;
		r_minmax.mxmax = clipmaxx<MAXX?clipmaxx:MAXX ;
		r_minmax.mxmin = r_minmax.mxmax<temp?r_minmax.mxmax:temp ;

		dmem = *((ulong *)&r_minmax);
		dramWrite(Regs, addr++, dmem);

		temp = clipminx>MINX?clipminx:MINX ;
		r_minmax.mxmax = ((curmax-firstcurnum)*XSKIP)*2-1 ;
		r_minmax.mxmin = r_minmax.mxmax<temp?r_minmax.mxmax:temp ;

		dmem = *((ulong *)&r_minmax);
		dramWrite(Regs, addr++, dmem);

	    }
      else if((curnum==6) && (refnum<=4) && (extime == 3))
	{	temp = MINY>clipminy?MINY:clipminy ;
		r_minmax.mymax = MAXY<clipmaxy?MAXY:clipmaxy ;
		r_minmax.mymin = r_minmax.mymax<temp?r_minmax.mymax:temp ;
		temp = (curmax-2)*XSKIP*2 ;
		r_minmax.mxmax = clipmaxx<MAXX?clipmaxx:MAXX ;
		r_minmax.mxmin = r_minmax.mxmax<temp?r_minmax.mxmax:temp ;

		dmem = *((ulong *)&r_minmax);
		dramWrite(Regs, addr++, dmem);

		temp = clipmaxx<MAXX?clipmaxx:MAXX ;
		temp2 = ((curmax-2)*XSKIP)*2-1 ;
		r_minmax.mxmax = temp>temp2?temp2:temp ;
		temp = clipminx>MINX?clipminx:MINX ;
		temp2 = (curmax-(2*2))*XSKIP ;
		r_minmax.mxmin = temp<temp2?temp2:temp ;

		dmem = *((ulong *)&r_minmax);
		dramWrite(Regs, addr++, dmem);

		temp = clipminx>MINX?clipminx:MINX ;
		r_minmax.mxmax = ((curmax-(2*2))*bksizex)*2 - 1 ;
		r_minmax.mxmin = r_minmax.mxmax<temp?r_minmax.mxmax:temp ;

		dmem = *((ulong *)&r_minmax);
		dramWrite(Regs, addr++, dmem);

	      }
      else if((curnum<=4) && (refnum==5))
	{	temp = MINX>clipminx?MINX:mid<clipminx?mid:clipminx ;
	 	r_minmax.mxmax = MAXX<clipmaxx?MAXX:clipmaxx ;
		r_minmax.mxmin = r_minmax.mxmax<temp?r_minmax.mxmax:temp ;
		temp = MINY>clipminy?MINY:clipminy ;
		r_minmax.mymax = (5-3)*bksizey*2 - 1 ;
		r_minmax.mymin = r_minmax.mymax<temp?r_minmax.mymax:temp ;

		dmem = *((ulong *)&r_minmax);
		dramWrite(Regs, addr++, dmem);

		temp = MAXY<clipmaxy?MAXY:clipmaxy ;
		r_minmax.mymin = (5-3)*bksizey*2 ;
		r_minmax.mymax = temp>r_minmax.mymin?temp:r_minmax.mymin ;

		dmem = *((ulong *)&r_minmax);
		dramWrite(Regs, addr++, dmem);

		extime = 2;
	      }
    }
  }
  r_minmax.mxmin = 1;
  r_minmax.mxmax = 1;
  r_minmax.mymin = 1;
  r_minmax.mymax = 1;
  for(x=0; x<extime*curnum/2; x++) {
    dmem = *((ulong *)&r_minmax);
    dramWrite(Regs, addr++, dmem);
  }
}

ECODE CMicrocode::LoadMicrocode(char *filename)
{
	char dataFileName[128];
	HANDLE hFile = NULL;
	HANDLE hMapFile = NULL;
	DWORD *pData;
	DWORD *ptr;
	ECODE ecode=VT_NO_ERROR;


	strcpy(dataFileName,Dev->szTablePath);
	strcat(dataFileName,filename);
	strcpy(Dev->ErrorMessage,dataFileName);

	/* open first file and memory map it */
    hFile = CreateFile(dataFileName, GENERIC_READ, 0, NULL, OPEN_EXISTING, FILE_FLAG_RANDOM_ACCESS, NULL);                              
    if (hFile== INVALID_HANDLE_VALUE) 	{ecode=VT_ERR_FILE_NOT_FOUND; goto error;}

    hMapFile = CreateFileMapping(hFile, NULL, PAGE_READONLY, 0, 0, NULL);
    if (hMapFile== INVALID_HANDLE_VALUE) {ecode=VT_ERR_FILE_NOT_FOUND; goto error;}

    ptr= pData = (DWORD*)MapViewOfFile(hMapFile, FILE_MAP_READ, 0,	0, 0);    	                
    if (pData== NULL) {ecode=VT_ERR_FILE_NOT_FOUND; goto error;}

	// FIRST LOAD IMEM
	ecode=VT_Write(Dev,VR_IMEM,*pData,pData+2,pData[1] & 0x3ff);

	// then load dmem
	while (!(*ptr==0 && ptr[1]==0))
	{
		ecode=VT_Write(Dev,VR_DRAM,*ptr,ptr+2,ptr[1]);
		ptr+=ptr[1]+2;
	}

	*Dev->ErrorMessage=0;
error:
    if (pData != NULL && pData!=INVALID_HANDLE_VALUE) {
        UnmapViewOfFile (pData);
    }        
    if (hMapFile != NULL&& hMapFile!=INVALID_HANDLE_VALUE) {
        CloseHandle (hMapFile);
    }        
    if (hFile != NULL && hFile!=INVALID_HANDLE_VALUE) {
        CloseHandle (hFile);
    }
	if (ecode)
	{
		wsprintf(Dev->ErrorMessage,"Could Not open File %s\n",dataFileName);
		Dev->FatalError(ecode,Dev->ErrorMessage);
	}
	return ecode;
}

CVrpCommand::CVrpCommand(unsigned short size,unsigned short command)
{
	 if (size<=MaxSimpleCommandSize)
	 {
	 	m_pCommand=(VRP_COMMAND*)&m_SimpleCommand;
	 	m_MustFree=FALSE;
	 }
	 else
	 {
	 	m_pCommand=(VRP_COMMAND*) malloc(size*sizeof(long));
	 	m_MustFree=TRUE;
	 }
	 m_pCommand->size=size;
	 m_pCommand->command=command;
}

CVrpCommand::CVrpCommand(unsigned short size,unsigned short command,CTimeCode tc)
{
	 CVrpCommand(size,command);
	 param(0)=tc;
}



ECODE VT_StopTC1(DEVID dwDevice,CTimeCode tc)
{
	ALLOC_COMMAND(Stop,2,EMI_Stop);
	Stop->params[0]=tc.Dword();
	return VT_SendCommand(dwDevice,Stop); 
}






ECODE VT_StopTC(DEVID dwDevice,TC tc)
{
	ALLOC_COMMAND(Stop,2,EMI_Stop);
	Stop->params[0]=*(DWORD*)&tc;
	return VT_SendCommand(dwDevice,Stop); 
}


ECODE VRP_Stop(DEVID dwDevice)
{
	DWORD StopD;
	VRP_COMMAND *Stop=(VRP_COMMAND*) &StopD;
	Stop->size=1;
	Stop->command=EMI_Stop;
	return VT_SendPriorityCommand(dwDevice,Stop); 
}

ECODE VRP_Start(DEVID dwDevice)
{
	DWORD StartD;
	VRP_COMMAND *Start=(VRP_COMMAND*) &StartD;
	Start->size=1;
	Start->command=EMI_Start;
	return VT_SendCommand(dwDevice,Start); 
}

ECODE VRP_StartP(DEVID dwDevice)
{
	 return VT_SendPriorityCommand(dwDevice,CVrpCommand(1,EMI_Start));
	//return CMicrocode(dwDevice).Start();
}

ECODE VRP_SetDefaultValues(DEVID dwDevice,DWORD mode)
{
	 CVrpCommand cmd(3,EMR_SetDefaultValues);
	 cmd.params()[0]=0xFFFF;
	 cmd.param(1)=mode;
	 return VT_SendCommand(dwDevice,cmd);
}

ECODE VRP_Pause(DEVID dwDevice)
{
	 CVrpCommand cmd(1,EMR_Pause);
	 return VT_SendPriorityCommand(dwDevice,cmd);
}

ECODE VRP_PauseTc(DEVID dwDevice,TC tc)
{
	 return VT_SendCommand(dwDevice,CVrpCommand(2,EMR_Pause,tc));
}

ECODE VRP_Continue(DEVID dwDevice)
{
	 return VT_SendPriorityCommand(dwDevice,CVrpCommand(1,EMR_Continue));
}


ECODE VRP_LoadVrpParams(DEVID dwDevice)
{
	 return VT_SendCommand(dwDevice,CVrpCommand(1,EMR_LoadParams));
}

ECODE VRP_SetRefTimecode(DEVID dwDevice,TC RefTC)
{
	 return VT_SendCommand(dwDevice,CVrpCommand(2,EMR_SetRefTimeCode,RefTC));
}


ECODE VT_SendCommand(DEVID dwDevice, const VRP_COMMAND *command)
{
	    PREGS Regs=GET_STRUCT_PTR(dwDevice);
		DWORD offsets[2];  
		DWORD cpsize;
		VT_Read(dwDevice,VR_DRAM,offsets,HOST_TO_CHIP_WRITE, 2);
		if (offsets[0]!=offsets[1])
		  if (command->size>((offsets[1]-offsets[0]) & 0xff))
			return VT_ERR_SIMB_FULL;
#ifdef DEBUG_UCODE
		if (offsets[0]!=offsets[1]) 
			dprintf("***Previous command not processed\n");
		dprintf("Sending Command %d size %d data:",command->command,command->size);
		{
			int i;
			for (i=0;i < min(command->size-1,10);i++)
				dprintf(" %x",command->params[i]);
			dprintf("\n");
		}
#endif
		cpsize=min(command->size,256-offsets[0]);
		VT_Write(dwDevice,VR_DRAM,HOST_TO_CHIP_BUFFER+offsets[0],(DWORD *)command,cpsize);
		if (cpsize<command->size)
		{
			DWORD *tptr=(DWORD *)command+cpsize;
			offsets[0]=0;
			cpsize=command->size-cpsize;
			VT_Write(dwDevice,VR_DRAM,HOST_TO_CHIP_BUFFER,tptr,cpsize);
		} 
		return dramWrite(Regs,HOST_TO_CHIP_WRITE,(offsets[0]+cpsize) & 0xff);
}

ECODE VT_SendPriorityCommand(DEVID dwDevice,const VRP_COMMAND *command)
{
	ReturnIfError(CDevice::CheckDevice(*dwDevice));
	return dwDevice->MicroCode->SendPriorityCommand(command);
}

ECODE CMicrocode::SendCommand(const VRP_COMMAND *command)
{
	return VT_SendCommand(Dev,command);
}


ECODE CMicrocode::SendPriorityCommand(const VRP_COMMAND *command)
{

	DWORD data,status;
	if (command->size>2)
		return VT_ERR_INVALID_COMMAND;

	VT_AquireMutex();
	dramRead(Regs,PRIORITY_BUFFER,&data);
	if (data & 0xffff0000)
	{
		VT_ReleaseMutex();
		return VT_ERR_PIMB_FULL;
	}
#ifdef DEBUG_UCODE
	if (command->size==2)
		dprintf("Sending Priority command %d size %d data:%x\n",command->command,command->size,command->params[0]);
	else
		dprintf("Sending Priority command %d size %d \n",command->command,command->size);
#endif

	if (command->size==2)
		dramWrite(Regs,PRIORITY_BUFFER+1,command->params[0]);
	status=dramWrite(Regs,PRIORITY_BUFFER,*(DWORD*)command);
	VT_ReleaseMutex();
	return status;
}



/***********************************************************************
 *
 * SetDefaultValues (2)
 *
 ***********************************************************************/
//			 FT_NTSC_QSIF,FT_NTSC_FILM_QSIF,FT_PAL_QSIF,FT_SQPIXEL_QSIF
ECODE CMicrocode::SetDefaultValues(ulong timecode, int defaultSetID)
{
  int status;
//  ulong data[2];
  VRP_COMMAND Cmd={2,3,timecode,defaultSetID};
//  data[0] = timecode;
//  data[1] = defaultSetID;

  switch (defaultSetID)
  {
	case FT_PAL:
		loadClippingTables(22, 18);    /* load PAL clipping tables */
		break;
	case FT_SQPIXEL:
		loadClippingTables(20, 15);    /* load SQ PIX clipping tables */
		break;
	case FT_NTSC_QSIF:
		loadClippingTables(11,7);
        break;
	case FT_PAL_QSIF:
		loadClippingTables(11,9);
        break;
	case FT_SQPIXEL_QSIF:
		loadClippingTables(10, 7);    /* load SQ PIX clipping tables */
		break;
	case FT_NTSC_FILM:
	case FT_NTSC:
	default:
		loadClippingTables(22, 15);    /* load NTSC clipping tables */
		break;
 }
  status=VT_SendCommand(Dev,&Cmd);
//  status = SendMsg(2, 3, &data[0]);
  return(status);
}


#if 0  // now using vt_send_command

ECODE CMicrocode::SendMsg(int type, int length, DWORD *buf)
{
	ulong	write_ptr, read_ptr;
	ulong	temp; 
	int	i;
	dramRead(Dev->Regs, HOST_TO_CHIP_READ, &read_ptr);
	dramRead(Dev->Regs, HOST_TO_CHIP_WRITE, &write_ptr);
	if (write_ptr >= read_ptr)  {
		if ((256 - (write_ptr - read_ptr)) <= ((ulong)length))  {
			return (FALSE);
		}
	}
	else  {
		if ((read_ptr - write_ptr) <= ((ulong)length))  {
			return (FALSE);
		}
	}
	temp = (length << 16) | type;
	dramWrite(Dev->Regs, HOST_TO_CHIP_BUFFER + write_ptr, temp);
	write_ptr = (write_ptr + 1) & 255;
	for (i = 1; i < length; i++)  {
		dramWrite(Dev->Regs, HOST_TO_CHIP_BUFFER + write_ptr, *buf++);
		write_ptr = (write_ptr + 1) & 255;
	}
	dramWrite(Dev->Regs, HOST_TO_CHIP_WRITE, write_ptr);
	return (VT_NO_ERROR);

}

#endif