// vt_use.c user routines
#include "stdincs.h"

#define WIN32_LEAN_AND_MEAN
#include <windows.h>
#include <stdlib.h>
#include "dtdefs.h"
#include "4000_reg.h"

#include "vt_acc.h"
#include "vt_regs.h"
#include "vt_vrp.h"
#include "mcode.h"
#include "capture.h"
#include "ibpcapture.h"
#include "lbrcapture.h"
#include "capframe.h"
#include "recode.h"
#include "codec.h"
#include "byteswap.h"
#include <mmsystem.h>
#include <stdio.h>


char VT_ErrorMessage[128];
#include "playback.h"

extern "C" volatile int HaveMutex;
extern "C" HANDLE Mutex;

#if 0
#define GetMutex() if (!HaveMutex) {int status=WaitForSingleObject(Mutex,2000);} HaveMutex++
#define ReleaseMutex() if (!--HaveMutex) {int status=ReleaseMutex(Mutex);} 
#endif

// ERROR MACROS

#ifdef _DEBUG
#include <assert.h>
#else
#define assert(x) 
#endif

#ifdef _DEBUG
#define TRACEWND_CLASSNAME "MfxTraceWindow"
#define TRACEWND_MESSAGE   "*WM_TRACE_MSG"

extern "C" void cdecl dprintf(LPSTR szFormat, ...)
{
    char ach[128];

    vsprintf(ach,szFormat,(LPSTR)(&szFormat+1));

	HWND TraceWnd = FindWindow(TRACEWND_CLASSNAME, NULL);
	if (TraceWnd) {
		static UINT WM_TRACE_MSG = RegisterWindowMessage(TRACEWND_MESSAGE);

		// Found Trace window: send message there as a global atom.
		// Delete atom after use.
		//
		ATOM atom = GlobalAddAtom((LPCSTR)ach);
		SendMessage(TraceWnd,WM_TRACE_MSG, (WPARAM)atom,NULL);
		GlobalDeleteAtom(atom);
	}
	else
		OutputDebugString(ach);
}

#endif
HANDLE hModule;

//BOOL DllEntryPoint (
BOOL DllMain (
    HANDLE    hDLL,
    DWORD     dwReason,
    LPVOID    lpReserved)
{
    
	dprintf("In Vtaccess DllMain.");
    switch (dwReason)
        {
        case DLL_PROCESS_ATTACH:
        //    if (!InitializeProcess (hDLL))
        //        return FALSE;
			dprintf("Process Attach\n");
   			Mutex=CreateMutex(NULL,FALSE,"BroadwayAccess");
			hModule=hDLL;
			if (Mutex==NULL)
				return FALSE;
            break;

        case DLL_PROCESS_DETACH:
			dprintf("Process Detach\n");
			if (Mutex!=NULL)
				CloseHandle(Mutex);
         //   CleanupProcess (hDLL);
            break;
        case DLL_THREAD_ATTACH:
			dprintf("Thread Attach\n");
            break;
        case DLL_THREAD_DETACH:
			dprintf("Thread Detach\n");
            break;
        default:
			dprintf("Unknown %d\n",dwReason);
            break;
        }
    return TRUE;
}


ECODE VT_INTERFACE VT_InitDevice(DWORD BoardPointer,DEVID *pdwDevice)
{
	return VT_InitDeviceEx(BoardPointer,"C:\\c-cube\\tables\\",FALSE,pdwDevice);
}


ECODE VT_INTERFACE VT_InitDeviceEx(DWORD BoardPointer, LPCSTR TablePath, BOOL ShortBurst, DEVID *pdwDevice)
{
	VT_REGS* regs=(VT_REGS*) BoardPointer;
	LPSTR tstr;
	*pdwDevice=(DEVID)new CDevice;
	if (*pdwDevice==NULL) return 1;

	// initialize structure
//	(*pdwDevice)->EditDecompressInfo.StructSize=sizeof(COMPRESSINFO);


	tstr=new char[strlen(TablePath)+1];
	strcpy(tstr,TablePath);
	(*pdwDevice)->szTablePath=tstr;

	(*pdwDevice)->ShortBurstsOnly=ShortBurst;
	// init VRP values
	
	(*pdwDevice)->Regs=regs;
	regs->Csr&=~SOFT_RESET;	// make shure board is enabled
	regs->Csr|= SCLK | SDO | VRP_SEL; // set Vip com bits to default state
	return 0; 
}




ECODE VT_INTERFACE VT_FlushCache(DEVID dwDevice)
{
	DWORD dummy;
#if 0
	int sum=0;
	int *ptr=(int*)dwDevice->FB_LinAddr;
	for (int tt=0;tt<70*1024;tt++)
	{
		sum+=*ptr++;
	}
#endif
	DWORD ec=DeviceIoControl(dwDevice->hDriver,IOCTL_VIDTOOL_FLUSHCACHE,NULL,NULL,NULL,NULL,&dummy,NULL);
	if (!ec)
	{
		dprintf("Flush Cash Failed\n");
		return(100);
	}
	return VT_NO_ERROR;
}
#define MAX_PATHNAME 256
 

ECODE VT_INTERFACE VT_OpenDevice(LPTSTR pDriverName, DWORD DeviceIndex, DEVID *dwDevice)
{
    DWORD i;
    
    PDEVICE_INFO pDevInfo;   
    DEVICE_IDS DeviceIDs;
    DWORD dwCount;
    TCHAR szTablePath[MAX_PATHNAME];
    HANDLE hDriver;
	TCHAR FullName[MAX_PATHNAME]=TEXT("\\\\.\\"); 
    OSVERSIONINFO VerInfo;
	//dprintf("Open device\n");

    
    /* 
     * We need to get the platform information to determine 
     * if we are running on WinNT or Win95
     */
    VerInfo.dwOSVersionInfoSize = sizeof(OSVERSIONINFO);
    if (!GetVersionEx(&VerInfo)) {
		return VT_ERR_OTHER;
    }

    /* 
     * NT doesn't recognize the 
     * Win95 Plug & Play device node.
     */
    if (VerInfo.dwPlatformId == VER_PLATFORM_WIN32_NT) {

	    TCHAR DeviceName[MAX_VIDCAP_NAME_LENGTH];

        if (pDriverName == NULL) {

            /*
             * no driver name given, so we should just open the device
             * numbered DeviceIndex
             */
            wsprintf(DeviceName, TEXT("%s%d"), DD_VIDCAP_DEVICE_NAME_U, DeviceIndex);
        } else {

            /*
             * open the Nth device created by driver lpDriverName.
             *
             * - currently we have no way of locating more than one device
             * per driver (to be fixed with the multimedia device map)
             */
            if (DeviceIndex != 0) {
		        return VT_ERR_BOARD_NOT_FOUND;
            }

            DeviceName[0] = TEXT('\0');
            if (!HW_ReadProfileString(pDriverName, REG_DEVNAME, DeviceName, sizeof(DeviceName))) {
		        return VT_ERR_BOARD_NOT_FOUND;
            }
        
            if (DeviceName[0] == TEXT('\0')) {
		        return VT_ERR_BOARD_NOT_FOUND;
            }

			/* create the full filename */
			lstrcat(FullName, DeviceName);
        }

    } else {
        /*
         * For Win95 the device is identified by the device node, not
         * by the name created by the driver for the device.
         *
         */
		lstrcat(FullName,pDriverName);

		/* vxd name must allways be devicename+d+.vxd */
    	lstrcat(FullName,TEXT("d.vxd"));
    }


	GetSystemDirectory (szTablePath, sizeof(szTablePath));
    lstrcat (szTablePath, "\\BWTABLES\\");

    /*
     * open the kernel driver - we need the overlapped flag
     * so that we can file async requests in the worker thread.
     * all requests for which the OVERLAPPED field is NULL will
     * still be synchronous though
     */

    hDriver = CreateFile(
            FullName,            
            GENERIC_READ,             
            0,
            NULL,
            OPEN_EXISTING,
            0,
            NULL);

    if (hDriver == INVALID_HANDLE_VALUE) {
        return VT_ERR_BOARD_NOT_FOUND; //(VT_ERR_BOARD_NOT_FOUND);
    }

	pDevInfo=new DEVICE_INFO;
	ZeroMemory (pDevInfo, sizeof (DEVICE_INFO));

	/* 
	 * in NT only one board, in Win95 if a DeviceIndex is supplied, open a
	 * specific device.
	 */
    BOOL bFoundBoard = FALSE;
	if (VerInfo.dwPlatformId == VER_PLATFORM_WIN32_NT || DeviceIndex != 0) {
		
			if (!DeviceIoControl (
					hDriver, 
					IOCTL_VIDTOOL_GET_DEVICEINFO,
					&DeviceIndex,
					sizeof(ULONG),
					pDevInfo,
					sizeof(DEVICE_INFO),
					&dwCount,
					NULL)) { 

				delete pDevInfo;
				return VT_ERR_BOARD_NOT_FOUND;
			}

			/* make sure we got our hardware info!!! */
			if (pDevInfo->dwStructSize < sizeof(DEVICE_INFO)) {
				delete pDevInfo;
				return VT_ERR_BOARD_NOT_FOUND;
			} 

			/* we got it!!! */
			bFoundBoard = TRUE;
	} else {
		
		/* query the vxd for all available devices in the system */
		if (!DeviceIoControl (
				hDriver, 
				IOCTL_VIDTOOL_GET_DEVICE_IDS,
				NULL,
				0,
				&DeviceIDs,
				sizeof(DEVICE_IDS),
				&dwCount,
				NULL)) { 

			CloseHandle (hDriver);
			return VT_ERR_BOARD_NOT_FOUND;
		}
#ifdef _DEBUG
		if (DeviceIDs.dwNumDevNodes>1)
		{
			dprintf("************* Found %d Broadway boards **********\n",DeviceIDs.dwNumDevNodes);
			__asm int 3
		}
#endif

		/* try to find a device thats not busy */
		for ( i=0; i<DeviceIDs.dwNumDevNodes; i++) {

			DWORD dwDeviceIndex = DeviceIDs.DevNodes[i];
			DEVICE_INFO DevInfo;

			ZeroMemory (&DevInfo, sizeof (DEVICE_INFO));
			
			/* 
			 * get the pointer to the device memory 
			 * and frame buffers 
			 */
			if (!DeviceIoControl (
					hDriver, 
					IOCTL_VIDTOOL_GET_DEVICEINFO,
					&dwDeviceIndex,
					sizeof(ULONG),
					&DevInfo,
					sizeof(DEVICE_INFO),
					&dwCount,
					NULL)) { 

				continue;
			}

			/* make sure we got our hardware info!!! */
			if (DevInfo.dwStructSize < sizeof(DEVICE_INFO)) {
				continue;
			}

			/* 
			 * first choice is initialized - if we got this
			 * we're done!!!
			 */
			if ((DevInfo.pUserInfo->State & 0xffff)== INITIALIZED) {
				CopyMemory (pDevInfo, &DevInfo, sizeof (DEVICE_INFO));
				bFoundBoard = TRUE;
				break;
			} else if ((DevInfo.pUserInfo->State & 0xffff) == CODEC_INITIALIZED) {
				/* 
				 * second choice is CODEC_INITIALIZED, but we'll wait
				 * until maybe something better comes along.
				 */
				CopyMemory (pDevInfo, &DevInfo, sizeof (DEVICE_INFO));
				bFoundBoard = TRUE;
			} else if (!bFoundBoard) {
				/*
				 * this is the last choice, but it's the one we'll end up
				 * using if nothing else shows up...
				 */
				CopyMemory (pDevInfo, &DevInfo, sizeof (DEVICE_INFO));
				bFoundBoard = TRUE;
			}
		}
	}
	
    /* did we get a free device ??? */
    if (!bFoundBoard) {
		CloseHandle (hDriver);
		delete pDevInfo;
        return VT_ERR_BOARD_NOT_FOUND;
    }
	
	/* create the device structure */
	*dwDevice=(DEVID)new CDevice(hDriver,pDevInfo);
	if (*dwDevice==NULL) {
		delete pDevInfo;
		return VT_ERR_OTHER;
	}
#if 0 // _DEBUG
	if ((*dwDevice)->GetRevisionID()>3) // this driver diliberatly breaks here...
	{
		(*dwDevice)->ShowMessage(VT_ERR_INCOMPATIBLE_HARDWARE);
		VT_DeleteDevice(*dwDevice);
		return VT_ERR_INCOMPATIBLE_HARDWARE;
	}
#endif

	char *tstr=new char[strlen(szTablePath)+1];
	strcpy(tstr,szTablePath);
	(*dwDevice)->szTablePath=tstr;

	// if device is idle initialize it

    return VT_NO_ERROR;
}


ECODE VT_INTERFACE VT_DeleteDevice(DEVID dwDevice)
{
//	if	(!(dwDevice->pDeviceInfo->State==CODEC_INITIALIZED || dwDevice->pDeviceInfo->State==CODEC_BUSY))
	//dprintf("Deleteing device\n");

	VT_AquireMutex();

	if (dwDevice->MicroCode!=NULL) {
	
		BOARD_STATE State = dwDevice->GetGlobalState();
		
		switch (dwDevice->CurrentMode) {
			case VT_IDLE:
				break;

			case VT_CODEC:
				if (State == CODEC_INITIALIZED || State == CODEC_BUSY || State == INITIALIZED) {
					VT_SetMode(dwDevice,VT_IDLE);  
				}
				break;
			
			default:
				VT_SetMode(dwDevice,VT_IDLE);  
				break;
		}
	}

	delete (void *)dwDevice->szTablePath;
	delete (CDevice*)dwDevice;

	VT_ReleaseMutex();
	return 0;
}

ECODE VT_Reset(DEVID dwDevice)
{
	dprintf("VT_Reset\n");
    PREGS bd_addr=GET_STRUCT_PTR(dwDevice);
	__try 
	{
		VT_SetMode(dwDevice,VT_IDLE); // Clean up Microcode if needed
		//reset(bd_addr);  // why bother????
		VT_AquireMutex();
		GBusWrite(bd_addr,GBUS_HCTL,0x50001000 + (CHIP_ID << 4));
		dwDevice->SetGlobalState(INITIALIZED); // Clear state for all drivers
		if (dwDevice->pGlobalInfo->RevisionID>2) 
			VidOutCtlPoke(bd_addr,0); //disable video out
		CrPoke(bd_addr,CTRL1,0x10);  // disable 
		CrPoke(bd_addr,CTRL2,FIFO_MODE_RECODE);  // disable 
	}
	__except ( _exception_code()==VT_ERR_IREG_TIMEOUT)
	{
		dprintf("Exception caught in reset\n");
	}
	Change32(&bd_addr->Csr,MSTR_MODE_MASK,0);	// clear master

	bd_addr->Csr=SOFT_RESET;
	Change32(&bd_addr->Csr,SOFT_RESET,0);	// make shure board is enabled
	if (bd_addr->Csr & SOFT_RESET) 
	{
#ifdef _DEBUG
		__asm int 3
#endif
		return VT_ERR_HARDWARE_FAILURE;
	}
	Change32(&bd_addr->Csr, SCLK | SDO | VRP_SEL, SCLK | SDO | VRP_SEL); // set Vip com bits to default state

	if (dwDevice->ShortBurstsOnly)
		Change32(&bd_addr->Csr,DISABLE_LARGEBURST,DISABLE_LARGEBURST);  // Needs to be registry setting
	// now reset FIFO
// run a dummy input transfer
#if 0
	if (dwDevice->FB_LinAddr!=0 && dwDevice->FB_PhysAddr!=0)
	{
	    // recode mode set above CrPoke(bd_addr,CTRL2,FIFO_MODE_RECODE);  // use recode mode 
		ZeroMemory(dwDevice->FB_LinAddr,1024*sizeof(DWORD));
		VT_LoadDma(dwDevice,MASTER1,1024,dwDevice->FB_PhysAddr);
		Change32(&bd_addr->Csr,MSTR_INPUT |STOP_1DONE_IN,MSTR_INPUT |STOP_1DONE_IN);
		GBusWrite(bd_addr,GBUS_HCTL,0x50000100); // clear local
		Sleep(1); 
		VT_ResetVRP(dwDevice);
		// close master
		Change32(&bd_addr->Csr,MSTR_INPUT ,0);
		Change32(&bd_addr->Csr,MSTR_INPUT|STOP_1DONE_IN,0);
	}
#endif
	GBusWrite(bd_addr,GBUS_HCTL,0x50001000 + (CHIP_ID << 4));
    GBusWrite(bd_addr, GBUS_DMAMODE, VIDTOOL_MEMCONFIG); // set memory mode
    WBusWrite(bd_addr, WBUS_INTCTL, 0x00010001); // enable refresh
	VT_SetupFifo(dwDevice,70,8);
  	VT_ReleaseMutex();
	return VT_NO_ERROR;
}

ECODE VT_INTERFACE VT_GetDeviceMemory(DEVID dwDevice, void **Regs,DWORD *Regs16)
{
	if (Regs16!=NULL && dwDevice->pDeviceInfo!=NULL)
		*Regs16=(DWORD)dwDevice->pDeviceInfo->lpRegs16;
	if (Regs!=NULL) *Regs=(void*)dwDevice->Regs;
	return VT_NO_ERROR;
}

ECODE VT_INTERFACE VT_SetupHardwareBuffer(DEVID dwDevice,DWORD PhysAddr,void *LinAddr,DWORD Size)
// set hardware buffer (size in dwords)
{
	dwDevice->FB_PhysAddr=PhysAddr;
	dwDevice->FB_LinAddr=(DWORD*)LinAddr;
#if 0
	dwDevice->FB_LinAddr=VirtualAlloc(LinAddr,Size*sizeof(DWORD),MEM_COMMIT,PAGE_READWRITE|PAGE_NOCACHE);
	if (dwDevice->FB_LinAddr==NULL)
	{
		dwDevice->FB_Size=0;
		return (GetLastError());
//		return (VT_ERR_OTHER);
	}
#endif
	//if (!VirtualProtect(dwDevice->FB_LinAddr,Size*sizeof(DWORD),PAGE_READWRITE|PAGE_NOCACHE,&OldProtect))
	//;//		return (GetLastError());


	dwDevice->FB_Size=Size;
	return (VT_NO_ERROR);
}

ECODE VT_INTERFACE VT_GetHardwareBuffer(DEVID dwDevice,DWORD *PhysAddr,
    void **LinAddr,DWORD *LinAddr16,DWORD *Size)
{
	if (PhysAddr!=NULL) *PhysAddr=dwDevice->FB_PhysAddr;
	if (LinAddr!=NULL) *LinAddr=dwDevice->FB_LinAddr;
	if (Size!=NULL) *Size=dwDevice->FB_Size;
	if (LinAddr16!=NULL&& dwDevice->pDeviceInfo!=NULL)
		*LinAddr16=(DWORD)dwDevice->pDeviceInfo->lpLinearAddr16;
	return (VT_NO_ERROR);
}


ECODE VT_INTERFACE VT_GetHardwareState(DEVID dwDevice, DWORD *pState) 
{
	// return the current device hardware state...
	VT_AquireMutex();
	// Hi bits of state are used to break with beginner
	*pState = (DWORD)dwDevice->GetGlobalState(); //DeviceInfo->pUserInfo->State;
	VT_ReleaseMutex();
	return 0;
}


ECODE VT_INTERFACE VT_GetRevisionID(DEVID dwDevice, UCHAR *pRevisionID) 
{
	// the kernel mode driver has to be installed to determine 
	// the hardware revision!!!
	if (dwDevice->hDriver == NULL) {
		return VT_ERR_OTHER;
	}
	*pRevisionID = dwDevice->GetRevisionID(); //pDeviceInfo->RevisionID;
	return 0;
}
#if 0
static void FatalError(ECODE ecode,char *String)
{
	if (String)
		MessageBox(NULL,String,NULL,MB_TASKMODAL | MB_OK);
	else
	{
		sprintf(DebugBuffer,__FILE__ ": Error %d ",ecode);
		MessageBox(NULL,String,NULL,MB_TASKMODAL | MB_OK);
	}
	
	RaiseException(ecode,0,0,NULL);
}
#endif


ECODE VT_GetCurrentTransferAddress(DEVID dwDevice,DWORD Master, DWORD *Current)
{
	PREGS bd_addr=GET_STRUCT_PTR(dwDevice);
	switch (Master)
	{
	case MASTER1:
		*Current=bd_addr->MasterStart1;
		break;
	case MASTER2:
		*Current=bd_addr->MasterStart2;
		break;
	default:
		return VT_ERR_PROGRAMED_RANGE;
	}
	return VT_NO_ERROR;
}


ECODE VT_GetMaster2PreviousTransferedSize(DEVID dwDevice,DWORD *TransferSizeDwords)
{
	PREGS bd_addr=GET_STRUCT_PTR(dwDevice);
	*TransferSizeDwords=bd_addr->Status & 0x3ffff;
	return VT_NO_ERROR;
}

ECODE VT_LoadDma(DEVID dwDevice,DWORD Mode,DWORD points,DWORD PhysicalAddress)
/*
DMA Modes:
1:  PORTA_TOPC Port A Full frame capture
2:  PORTA_FROMPC
3:  PORTB_TOPC
This routine will Enable a DMA transfer Initial setup will be done by the Load microcode command.
*/
{
	PREGS bd_addr=GET_STRUCT_PTR(dwDevice);
#ifdef _DEBUG
	if (PhysicalAddress<dwDevice->FB_PhysAddr ||
		PhysicalAddress+(points<<2)>dwDevice->FB_PhysAddr+(dwDevice->FB_Size<<2))
		dprintf("Invalid buffer for master\n");
#endif
	switch (Mode)
	{
	case MASTER1:
		// do in reverse order to avoid PCI burst write
		bd_addr->MasterSize1=points;
		bd_addr->MasterStart1=PhysicalAddress;
		break;
	case MASTER2:
		bd_addr->MasterSize2=points;
		bd_addr->MasterStart2=PhysicalAddress;
		break;
	default:
		return VT_ERR_PROGRAMED_RANGE;
	}
	return 0;
}


ECODE VT_INTERFACE VT_SetupVideoMode(DEVID dwDevice,DWORD VidMode,DWORD VidSource)
//VidMode is one of  NTSC=1 or PAL=2 or AutoDetect=3 
//VidSource is Composite=1 or SVideo=2 or AutoDetect=3
{
	dprintf("VT_SetupVideoMode %d %d\n",VidMode,VidSource);

	PREGS bd_addr=GET_STRUCT_PTR(dwDevice);
	switch (VidSource)
	{
	case 1: dwDevice->VipRegs.VipInputCtrl1 &= ~ 0x7;  // Set mode 0
		break;
	case 2: dwDevice->VipRegs.VipInputCtrl1 |=   0x7;  // Set mode 7  Mode 5 should also work
		break;
	default : return VT_ERR_PROGRAMED_RANGE; 
	}

	switch (VidMode)
	{
	case NTSC:	dwDevice->VipRegs.VipSyncCtrl = (dwDevice->VipRegs.VipSyncCtrl & 0x3F) | 0x40;
		dwDevice->VipRegs.VipHsb=0xDB;
		dwDevice->VipRegs.VipFmtDlyCtrl=0x50;
		
		break;
	case PAL:	dwDevice->VipRegs.VipSyncCtrl = (dwDevice->VipRegs.VipSyncCtrl & 0x3F);
		dwDevice->VipRegs.VipHsb=0xD9;
		dwDevice->CurrentVideoMode=PAL;
		dwDevice->VipRegs.VipFmtDlyCtrl=0x78;
		break;
	case AUTO_DETECT:	dwDevice->VipRegs.VipSyncCtrl = (dwDevice->VipRegs.VipSyncCtrl & 0x3F) | 0x80;
		break;

	default : return VT_ERR_PROGRAMED_RANGE; 
	}
	dwDevice->CurrentVideoMode=(short)VidMode;
	dwDevice->CurrentSource=(short)VidSource;
	dwDevice->SetVipFilterMode(FALSE);

    return dwDevice->WriteVIP(VIP_INPUT_CTRL1,(DWORD*)&(dwDevice->VipRegs.VipInputCtrl1),(sizeof(VIP_REGS)-2)/sizeof(DWORD));
}


ECODE VT_INTERFACE VT_GetVideoMode(DEVID dwDevice,DWORD* VidMode,DWORD* VidSource)
//VidMode is one of  NTSC=1 or PAL=2 or AutoDetect=3 
//VidSource is Composite=1 or SVideo=2 or AutoDetect=3
{
	PREGS bd_addr=GET_STRUCT_PTR(dwDevice);
	ECODE retval=VT_NO_ERROR;
	DWORD status;
	dwDevice->ReadVIP(&status,VIP_STATUS,1);

	// version 0 fixup
	int Loops=0;
	if (dwDevice->VipRegs.VipVer==0 && ( (status & 0x21) == 0x20) && Loops++<4)
	{
		DWORD tmp=dwDevice->VipRegs.VipChromaCtrl | 0x30;
		dwDevice->WriteVIP(VIP_CHROMA_CTRL,&tmp,1);
		Sleep(100);
		tmp &=~0x30;
		dwDevice->WriteVIP(VIP_CHROMA_CTRL,&tmp,1);
		Sleep(10);
		dwDevice->ReadVIP(&status,VIP_STATUS,1);
	}

	if (!(status & 1)) retval=VT_NOT_MATCHED;  // low priority error
	switch (status & 0x60)
	{
	case 0x60: status=VT_NOT_HSUNK;
		*VidMode= NTSC;
		break;
	case 0x20: *VidMode= NTSC;	
		 dwDevice->CurrentVideoMode=NTSC;
		 break;
	case 0x40: 
		status=VT_NOT_HSUNK;
		*VidMode= PAL;
		break;
	case 0x0: 
		*VidMode= PAL;
		dwDevice->CurrentVideoMode=PAL;
		break;
	default: 	  ;
	}


	switch (dwDevice->VipRegs.VipInputCtrl1 & 0x7)
	{
	case 0x0: *VidSource=1; break;
	case 0x5:
	case 0x7: *VidSource=2; break;
	default: retval=VT_VIP_UNDEFINED; break;
	}

	return retval;
}



long CalcVideoParam(DWORD param,long value)
{
	long tval;
	switch (param)
	{
		// signed input and output input +-1000;
 		case HUE: 
			return (long)(value*0.127) & 0xff;	     


		// signed input unsigned output
 		case INTENSITY:	 
			tval=(long)((value+1000)*0.128);
		 // scale hue to a byte
			if (tval>255)  
				return 255;
			else
				return tval;

		// contrast is funny
	// this formula was found from curve fitting on points (-1000,0)(0,135)(1000,255)
	// curve fit was done with DTVEE 
 		case SATURATION: 
			tval=(long)((value+1000)*0.064);
			if (tval>127)  
				return 127;
			else
				return tval;

 		case CONTRAST: 
 			tval=(long)(71+(value*(0.1275+value*-7.5e-6)/2));	 
			if (tval>127)  
				return 127;
			else
				return tval;

	    case DATARATE:	 
		case QUALITY:
		case PADDING: 	 
	    case SOURCEISVCR:
		case SHIFTLEFT: 
		case SHIFTUP: 	 
		case BLANKLINES: 
		case FILTERVALUE:
		default: return value;
	}
}


ECODE VT_SetParam(DEVID dwDevice,DWORD param,long  value)
{
	PREGS bd_addr=GET_STRUCT_PTR(dwDevice);
	CDevice *Dev=(CDevice*)dwDevice; 
    DWORD HwVal=CalcVideoParam(param,value);
	if (dwDevice->State>=VTS_READY && dwDevice->MicroCode!=NULL)  // use Microcode function
	{	
		return (dwDevice->MicroCode->SetParam(param,HwVal));  
	}
	else
		dwDevice->State=VTS_CONFIGNEEDED;
	switch (dwDevice->CurrentMode)
	{
	case VT_IENCODE:
	    switch (param)
	    {
	    case FRAMERATE:	dwDevice->CaptureInfo.FrameRate=value; break;
	    case DATARATE:	dwDevice->CaptureInfo.DataRate=value; break;
		case QUALITY: 	dwDevice->CaptureInfo.Quality=value; break;
		case PADDING: 	dwDevice->CaptureInfo.Padding=value; break;
	    case FRAMEWIDTH: dwDevice->CaptureInfo.FrameWidth=(short)value; break;
	    case FRAMEHEIGHT: dwDevice->CaptureInfo.FrameHeight=(short)value; break;
	    case PASSMODE: dwDevice->CaptureInfo.PassMode=(short)value; break;
	    case SOURCEISVCR: dwDevice->CaptureInfo.SourceIsVcr=(short)value; break;
	    case ASPECT:	dwDevice->CaptureInfo.Aspect=(short)value; break;
 		case HUE:	   dwDevice->CaptureInfo.Hue=(short)value; break;
 		case SATURATION:  dwDevice->CaptureInfo.Saturation=(short)value; break;
 		case INTENSITY:	 dwDevice->CaptureInfo.Intensity=(short)value; break;
 		case CONTRAST:	dwDevice->CaptureInfo.Contrast=(short)value; break;
		case SHIFTLEFT: 	dwDevice->CaptureInfo.ShiftLeft=(short)value; break;
		case SHIFTUP: 	dwDevice->CaptureInfo.ShiftUp=(short)value; break;
		case BLANKLINES: 	dwDevice->CaptureInfo.Blanklines=(short)value; break;
		case FILTERVALUE: 	
			Dev->CaptureInfo.FilterValue=(short)value;
			Dev->SetVipFilterMode(TRUE);
			break;

		default: 
			return VT_ERR_UNSUPPORTED;
		}
		break;
	case VT_CODEC:
	    switch (param)
	    {
	    case FRAMERATE:	dwDevice->CodecInfo.FrameRate=value; break;
	    case DATARATE:	dwDevice->CodecInfo.DataRate=value; break;
	    case FRAMEWIDTH: dwDevice->CodecInfo.FrameWidth=(short)value; break;
	    case FRAMEHEIGHT: dwDevice->CodecInfo.FrameHeight=(short)value; break;
	    case ASPECT:	dwDevice->CodecInfo.Aspect=value; break;
		case QUALITY: 	dwDevice->CodecInfo.Quality=value; break;
		case FRAMETYPE:  dwDevice->CodecInfo.FrameType=value; break;
		case PADDING: 	dwDevice->CodecInfo.Padding=value; break;
		case CODECMODE: break;
		default: 
			return VT_ERR_UNSUPPORTED;
		}
		break;
#if 0
	case VT_IDECOMPRESS:
	    switch (param)
	    {
	    case FRAMERATE:	dwDevice->EditDecompressInfo.FrameRate=value; break;
	    case DATARATE:	dwDevice->EditDecompressInfo.DataRate=value; break;
	    case FRAMEWIDTH: dwDevice->EditDecompressInfo.FrameWidth=(short)value; break;
	    case FRAMEHEIGHT: dwDevice->EditDecompressInfo.FrameHeight=(short)value; break;
	    case ASPECT:	dwDevice->EditDecompressInfo.Aspect=value; break;
		case QUALITY: 	dwDevice->EditDecompressInfo.Quality=value; break;
		case FRAMETYPE:  dwDevice->EditDecompressInfo.FrameType=value; break;
		case PADDING: 	dwDevice->EditDecompressInfo.Padding=value; break;
		default: 
			return VT_ERR_UNSUPPORTED;
		}
		break;
#endif 
	case VT_IDECODE:
		return VT_ERR_UNSUPPORTED;			 
		break;
	case VT_RECODE:
	    switch (param)
	    {
	    case FRAMERATE:	dwDevice->RecodeInfo.FrameRate=value; break;
	    case DATARATE:	dwDevice->RecodeInfo.DataRate=value; break;
	    case FRAMEWIDTH: dwDevice->RecodeInfo.FrameWidth=(short)value; break;
	    case FRAMEHEIGHT: dwDevice->RecodeInfo.FrameHeight=(short)value; break;
	    case ASPECT:	dwDevice->RecodeInfo.Aspect=value; break;
		case QUALITY: 	dwDevice->RecodeInfo.Quality=value; break;
		case SHIFTUP: 	dwDevice->RecodeInfo.ShiftUp=value; break;
		case SHIFTLEFT: 	dwDevice->RecodeInfo.ShiftLeft=value; break;
		case PADDING: 	dwDevice->RecodeInfo.Padding=(short)value; break;
		case FILTERVALUE: 	dwDevice->RecodeInfo.FilterValue=value; break;

		default: 
			return VT_ERR_UNSUPPORTED;
		}
		break;
	default:
		return VT_ERR_UNSUPPORTED;			 
	}
	return VT_NO_ERROR;
}

ECODE VT_GetParam(DEVID dwDevice,DWORD param,long *value)
{
	PREGS bd_addr=GET_STRUCT_PTR(dwDevice);
	switch (dwDevice->CurrentMode)
			
	{
	case VT_IENCODE:
	case VT_IBPENCODE:
	case VT_DIAG1:
	    switch (param)
	    {
	    case FRAMERATE:	*value=dwDevice->CaptureInfo.FrameRate;	break;
	    case DATARATE:	*value=dwDevice->CaptureInfo.DataRate;	break;
	    case FRAMEWIDTH: *value=dwDevice->CaptureInfo.FrameWidth;	break;
	    case FRAMEHEIGHT: *value=dwDevice->CaptureInfo.FrameHeight;	break;
	    case ASPECT:	*value=dwDevice->CaptureInfo.Aspect;	break;
 		case HUE:	   *value=dwDevice->CaptureInfo.Hue;	break;
 		case SATURATION:  *value=dwDevice->CaptureInfo.Saturation;	break;
 		case INTENSITY:	 *value=dwDevice->CaptureInfo.Intensity;	break;
 		case CONTRAST:	*value=dwDevice->CaptureInfo.Contrast;	break;
		case QUALITY: 	*value=dwDevice->CaptureInfo.Quality;break;
		case SHIFTUP: 	*value=dwDevice->CaptureInfo.ShiftUp;break;
		case SHIFTLEFT: 	*value=dwDevice->CaptureInfo.ShiftLeft;break;
		case BLANKLINES: 	*value=dwDevice->CaptureInfo.Blanklines;break;
		case PADDING: 	*value=dwDevice->CaptureInfo.Padding;break;
		case FILTERVALUE: 	*value=dwDevice->CaptureInfo.FilterValue;break;

		default: 
			*value=0;
			return VT_ERR_UNSUPPORTED;
		}
		break;
	case VT_CODEC:
	    switch (param)
	    {
	    case FRAMERATE:	*value=dwDevice->CodecInfo.FrameRate;	break;
	    case DATARATE:	*value=dwDevice->CodecInfo.DataRate;	break;
	    case FRAMEWIDTH: *value=dwDevice->CodecInfo.FrameWidth;	break;
	    case FRAMEHEIGHT: *value=dwDevice->CodecInfo.FrameHeight;	break;
	    case ASPECT:	*value=dwDevice->CodecInfo.Aspect;	break;
		case QUALITY: 	*value=dwDevice->CodecInfo.Quality;break;
		case FRAMETYPE: *value=(DWORD) dwDevice->CodecInfo.FrameType; break;
		case PADDING: 	*value=dwDevice->CodecInfo.Padding;break;
		default: 
			*value=0;
			return VT_ERR_UNSUPPORTED;
		}
		break;
#if 0
	case VT_IDECOMPRESS:
	    switch (param)
	    {
	    case FRAMERATE:	*value=dwDevice->EditDecompressInfo.FrameRate;	break;
	    case DATARATE:	*value=dwDevice->EditDecompressInfo.DataRate;	break;
	    case FRAMEWIDTH: *value=dwDevice->EditDecompressInfo.FrameWidth;	break;
	    case FRAMEHEIGHT: *value=dwDevice->EditDecompressInfo.FrameHeight;	break;
	    case ASPECT:	*value=dwDevice->EditDecompressInfo.Aspect;	break;
		case QUALITY: 	*value=dwDevice->EditDecompressInfo.Quality;break;
		case FRAMETYPE: *value=(DWORD) dwDevice->EditDecompressInfo.FrameType; break;
		case PADDING: 	*value=dwDevice->EditDecompressInfo.Padding;break;
		default: 
			*value=0;
			return VT_ERR_UNSUPPORTED;
		}
		break;
#endif
	case VT_IDECODE:
		return VT_ERR_UNSUPPORTED;			 
		break;
	case VT_RECODE:
	    switch (param)
	    {
	    case FRAMERATE:	*value=dwDevice->RecodeInfo.FrameRate;	break;
	    case DATARATE:	*value=dwDevice->RecodeInfo.DataRate;	break;
	    case FRAMEWIDTH: *value=dwDevice->RecodeInfo.FrameWidth;	break;
	    case FRAMEHEIGHT: *value=dwDevice->RecodeInfo.FrameHeight;	break;
	    case ASPECT:	*value=dwDevice->RecodeInfo.Aspect;	break;
		case QUALITY: 	*value=dwDevice->RecodeInfo.Quality;break;
		case SHIFTUP: 	*value=dwDevice->RecodeInfo.ShiftUp;break;
		case SHIFTLEFT: 	*value=dwDevice->RecodeInfo.ShiftLeft;break;
		case PADDING: 	*value=dwDevice->RecodeInfo.Padding;break;
		case FILTERVALUE: 	*value=dwDevice->RecodeInfo.FilterValue;break;

		default: 
			*value=0;
			return VT_ERR_UNSUPPORTED;
		}
		break;
	default:
		*value=0;
		return VT_ERR_UNSUPPORTED;			 
	}
	return VT_NO_ERROR;
}

ECODE VT_GetCaptureInfo(DEVID dwDevice, CAPTUREINFO *ConfigInfo)
{
	PREGS bd_addr=GET_STRUCT_PTR(dwDevice);
	if (ConfigInfo->StructSize>sizeof(CAPTUREINFO))
		return VT_ERR_BAD_STRUCT_SIZE;
	memcpy(ConfigInfo,&dwDevice->CaptureInfo,ConfigInfo->StructSize);
	return VT_NO_ERROR;

}

ECODE VT_SetCaptureInfo(DEVID dwDevice, CAPTUREINFO *ConfigInfo)
{
	PREGS bd_addr=GET_STRUCT_PTR(dwDevice);
	if (dwDevice->State>=VTS_READY) return VT_ERR_HW_BUSY;
    if (dwDevice->Busy) return VT_ERR_HW_BUSY;
	if (ConfigInfo->StructSize>sizeof(CAPTUREINFO))
		return VT_ERR_BAD_STRUCT_SIZE;

	memcpy(&dwDevice->CaptureInfo,ConfigInfo,ConfigInfo->StructSize);

	dwDevice->VipRegs.VipHueCtrl=CalcVideoParam(HUE,dwDevice->CaptureInfo.Hue); // scale hue to a byte
	dwDevice->VipRegs.VipSatCtrl=CalcVideoParam(SATURATION,dwDevice->CaptureInfo.Saturation);
 	dwDevice->VipRegs.VipContCtrl=CalcVideoParam(CONTRAST,dwDevice->CaptureInfo.Contrast);
	dwDevice->VipRegs.VipBrightCtrl=CalcVideoParam(INTENSITY,dwDevice->CaptureInfo.Intensity); 
	((CDevice*)dwDevice)->SetVipFilterMode(FALSE); 
    dwDevice->WriteVIP(0,(DWORD*)&dwDevice->VipRegs,(sizeof(VIP_REGS))/sizeof(DWORD));
	dwDevice->ReadVIP(&dwDevice->VipRegs.VipVer,VIP_VER,1); // read back version

	return VT_NO_ERROR;
}

ECODE VT_GetCodecInfo(DEVID dwDevice, COMPRESSINFO *CompressInfo)
{
	PREGS bd_addr=GET_STRUCT_PTR(dwDevice);
	if (CompressInfo->StructSize>sizeof(COMPRESSINFO))
		return VT_ERR_BAD_STRUCT_SIZE;
	memcpy(CompressInfo,&dwDevice->CodecInfo,CompressInfo->StructSize);
	return VT_NO_ERROR;
}

ECODE VT_SetCodecInfo(DEVID dwDevice, COMPRESSINFO *CompressInfo)
{
	PREGS bd_addr=GET_STRUCT_PTR(dwDevice);
 //	if (dwDevice->State>=VTS_READY) return VT_ERR_HW_BUSY;
 //  if (dwDevice->Busy) return VT_ERR_HW_BUSY;
	if (CompressInfo->StructSize>sizeof(COMPRESSINFO))
		return VT_ERR_BAD_STRUCT_SIZE;
	memcpy(&dwDevice->CodecInfo,CompressInfo,CompressInfo->StructSize);
	return VT_NO_ERROR;
}

#if 0
ECODE VT_GetEditDecompressInfo(DEVID dwDevice, COMPRESSINFO *CompressInfo)
{
	PREGS bd_addr=GET_STRUCT_PTR(dwDevice);
	if (CompressInfo->StructSize>sizeof(COMPRESSINFO))
		return VT_ERR_BAD_STRUCT_SIZE;
	memcpy(CompressInfo,&dwDevice->EditDecompressInfo,CompressInfo->StructSize);
	return VT_NO_ERROR;
}

ECODE VT_SetEditDecompressInfo(DEVID dwDevice, COMPRESSINFO *CompressInfo)
{
	PREGS bd_addr=GET_STRUCT_PTR(dwDevice);
	if (dwDevice->State>=VTS_READY) return VT_ERR_HW_BUSY;
    if (dwDevice->Busy) return VT_ERR_HW_BUSY;
	if (CompressInfo->StructSize>sizeof(COMPRESSINFO))
		return VT_ERR_BAD_STRUCT_SIZE;
	memcpy(&dwDevice->EditDecompressInfo,CompressInfo,CompressInfo->StructSize);
	return VT_NO_ERROR;
}

#endif
ECODE VT_GetRecodeInfo(DEVID dwDevice, RECODEINFO *RecodeInfo)
{
	PREGS bd_addr=GET_STRUCT_PTR(dwDevice);
	if (RecodeInfo->StructSize>sizeof(RECODEINFO))
		return VT_ERR_BAD_STRUCT_SIZE;
	memcpy(RecodeInfo,&dwDevice->RecodeInfo,RecodeInfo->StructSize);
	return VT_NO_ERROR;
}

ECODE VT_SetRecodeInfo(DEVID dwDevice, RECODEINFO *RecodeInfo)
{
	PREGS bd_addr=GET_STRUCT_PTR(dwDevice);
	if (dwDevice->State>=VTS_READY) return VT_ERR_HW_BUSY;
    if (dwDevice->Busy) return VT_ERR_HW_BUSY;
	if (RecodeInfo->StructSize>sizeof(RECODEINFO))
		return VT_ERR_BAD_STRUCT_SIZE;
	memcpy(&dwDevice->RecodeInfo,RecodeInfo,RecodeInfo->StructSize);
	return VT_NO_ERROR;
}

ECODE VT_ModifyRecodeParam(DEVID dwDevice, TC tc,DWORD param, DWORD value)
/*
Recode params are:
1:  Quality
2:  DataRate
3:  FilterValue
*/
{
	PREGS bd_addr=GET_STRUCT_PTR(dwDevice);
	return VT_NO_ERROR;
}


ECODE VT_GetMode(DEVID dwDevice,DWORD *Mode)
{
	PREGS bd_addr=GET_STRUCT_PTR(dwDevice);
	*Mode=dwDevice->CurrentMode;
	return VT_NO_ERROR;
}

#ifndef USE_LIB
ECODE VT_SetMode(DEVID dwDevice,DWORD Mode)
/*
Microcode is
1:  I frame Ecoder
2:  I frame Editor
3:  I frame Decoder
4:  Recoder
10:  Diagnostic mode 1
11:  Diagnostic mode 2	 ...

All microcode files will be stored in a directory sepecified in the system registry
*/
{
	PREGS bd_addr=GET_STRUCT_PTR(dwDevice);
	
	VT_AquireMutex();
	
	if (Mode != VT_IDLE && !(dwDevice->GetGlobalState()==INITIALIZED 
		|| dwDevice->GetGlobalState()==CODEC_INITIALIZED))
	{
		VT_ReleaseMutex();
		dprintf("Set Mode %d failed .... Busy\n",Mode); 
		return VT_ERR_HW_BUSY;
	}

	if (dwDevice->MicroCode!=NULL)
	{
		 delete (CMicrocode*)dwDevice->MicroCode;
		 dwDevice->MicroCode=NULL;
	}
	dprintf("Setting Mode %d\n",Mode);
	switch (Mode)
	{
	case VT_IDLE:
		dwDevice->CurrentMode=VT_IDLE;
		dwDevice->Busy=FALSE;
		dwDevice->State=VTS_CONFIGNEEDED;
		if (!(dwDevice->GetGlobalState()==CODEC_INITIALIZED 
			  || dwDevice->GetGlobalState()==CODEC_BUSY))
			dwDevice->SetGlobalState(INITIALIZED);

		VT_ReleaseMutex();
		return VT_NO_ERROR;
	case VT_IENCODE:
		dwDevice->CurrentMode=VT_IENCODE;
		if (dwDevice->CaptureInfo.IBP)
			dwDevice->MicroCode=new CIBPCapture(dwDevice);
		else
			dwDevice->MicroCode=new CCapture(dwDevice);
		dwDevice->SetGlobalState(CAPTURE);
		break; 
	case VT_RECODE:
		dwDevice->CurrentMode=VT_RECODE;
		dwDevice->MicroCode=new CRecode(dwDevice);
		dwDevice->SetGlobalState(RECODE);
		break;
	case VT_IBPENCODE:
		dwDevice->CurrentMode=VT_IBPENCODE;
		dwDevice->MicroCode=new CIBPCapture(dwDevice);
		dwDevice->SetGlobalState(CAPTURE);
		break; 

	case VT_LBRENCODE:
		dwDevice->CurrentMode=VT_LBRENCODE;
		dwDevice->MicroCode=new CLbrCapture(dwDevice);
		dwDevice->SetGlobalState(CAPTURE);
		break; 

	case VT_DIAG1:
		dwDevice->CurrentMode=VT_DIAG1;
		dwDevice->MicroCode=new CCapFrame(dwDevice);
		break; 
	case VT_IDECODE:
		dwDevice->CurrentMode=VT_IDECODE;
		dwDevice->MicroCode=new CPlayback(dwDevice);
		dwDevice->SetGlobalState(RECODE); // fix this?
		break; 
//	case VT_ICOMPRESS:
	case VT_IDECOMPRESS:
	case VT_CODEC:
		dwDevice->CurrentMode=VT_CODEC;
		dwDevice->MicroCode=new CCodec(dwDevice);
		break;

	case VT_DIAG2:
		dwDevice->CurrentMode=VT_DIAG2;
		dwDevice->MicroCode=new CDiag2(dwDevice);
		dwDevice->SetGlobalState(CAPTURE);

		break;
	default: 
		VT_ReleaseMutex();
		return VT_ERR_UNSUPPORTED;			 
	}
#ifdef _DEBUG
	VT_ReleaseMutex(); // allow debugging of load
	ECODE ecode=dwDevice->MicroCode->Load();
#else
	ECODE ecode=dwDevice->MicroCode->Load();
	VT_ReleaseMutex();
#endif
	if (ecode) return ecode;
	dwDevice->State=VTS_READY;
	return VT_NO_ERROR;
}

ECODE VT_WriteData(DEVID dwDevice, const DWORD *SrcPtr, DWORD Words, DWORD *WordsWritten)
{
	if (dwDevice->MicroCode==NULL) return VT_ERR_NO_MODE;
	return (dwDevice->MicroCode->WriteData(SrcPtr, Words, WordsWritten));
}

ECODE VT_ReadData(DEVID dwDevice,  void **DestPtr, DWORD *WordsRead)
{
	if (dwDevice->MicroCode==NULL) return VT_ERR_NO_MODE;
	return (dwDevice->MicroCode->ReadData(DestPtr, WordsRead));
}

ECODE VT_GetNextBuffer(DEVID dwDevice, VTBUFFERTYPE Type, void **BuffPtr, DWORD *SizeWords)
{
	ReturnIfError(CDevice::CheckDevice(*dwDevice));
	return (dwDevice->MicroCode->GetNextBuffer( Type, BuffPtr, SizeWords));
}

ECODE VT_GetStatus(DEVID dwDevice,DWORD param,long *value)
{
	ReturnIfError(CDevice::CheckDevice(*dwDevice));
	return (dwDevice->MicroCode->GetStatus(param,value));
}

#endif

ECODE VT_Start(DEVID dwDevice)
{
	PREGS bd_addr=GET_STRUCT_PTR(dwDevice);
	if (dwDevice->MicroCode==NULL) return VT_ERR_NO_MODE;
	//dprintf("Starting\n");
	ReturnIfError(((CMicrocode*)dwDevice->MicroCode)->Start());
	dwDevice->State=VTS_RUNNING;
	return VT_NO_ERROR;
	//return VRP_StartP(dwDevice);
}

ECODE VT_Stop(DEVID dwDevice)
{
	PREGS bd_addr=GET_STRUCT_PTR(dwDevice);
	if (dwDevice->MicroCode==NULL) return VT_ERR_NO_MODE;
	return dwDevice->MicroCode->Stop();
}

ECODE VT_Pause(DEVID dwDevice)
{
	PREGS bd_addr=GET_STRUCT_PTR(dwDevice);
	if (dwDevice->MicroCode==NULL) return VT_ERR_NO_MODE;
	return dwDevice->MicroCode->Pause();
}

ECODE VT_Continue(DEVID dwDevice)
{
	PREGS bd_addr=GET_STRUCT_PTR(dwDevice);
	if (dwDevice->MicroCode==NULL) return VT_ERR_NO_MODE;
	return dwDevice->MicroCode->Continue();
}

#define Timeout(From,ms) ((timeGetTime()-From) > ms)

ECODE VT_INTERFACE VT_GetHW(DEVID dwDevice,DWORD TimeOutms)
{
	DWORD status=WaitForSingleObject(Mutex,TimeOutms);
	if (status==WAIT_TIMEOUT) 
		return  VT_ERR_HW_BUSY;

	BOARD_STATE state=dwDevice->GetGlobalState();
	
	if (!(state==CODEC_INITIALIZED ||
		  state==INITIALIZED ||
		  state==CODEC_BUSY) )
	{
		VT_ReleaseMutex();
		return VT_ERR_HW_BUSY;

	}

	if (state==INITIALIZED)
	{
		dprintf("%x:GetHW reloading\n",dwDevice);
		VT_SetMode(dwDevice,VT_CODEC);
	}

	dwDevice->SetGlobalState(CODEC_BUSY);
	return VT_NO_ERROR;
}


ECODE VT_INTERFACE VT_ReleaseHW(DEVID dwDevice)  // release the HW
{
	
	dwDevice->SetGlobalState(CODEC_INITIALIZED);
	
	VT_ReleaseMutex();

	return VT_NO_ERROR;
}


ECODE VT_EnablePassthrough(DEVID dwDevice,int Enable)
{
	PREGS bd_addr=GET_STRUCT_PTR(dwDevice);
	if (Enable)
	{
		dwDevice->Busy=TRUE;
		CrChange(bd_addr,CTRL1,0xff,0x10);
		// dma init ??
		Change32(&bd_addr->Csr,MSTR_MODE_MASK, MSTR_TOGGLE); // No passthru
//		bd_addr->Csr=bd_addr->Csr & ~MSTR_MODE_MASK | MSTR_OUTPUT; // No passthru
		CrChange(bd_addr,CTRL2,FIFO_MODE_MASK,FIFO_MODE_ICAPTURE);
	//	CrChange(bd_addr,CTRL2,FIFO_MODE_MASK,FIFO_MODE_PASSTHROUGH);
		CrChange(bd_addr,CTRL1,0xff,0x11);
	}
	else
	{
		CrPoke(bd_addr,CTRL1,0x10);
		dwDevice->Busy=FALSE;
	}
	return VT_NO_ERROR;
}

#if 0
ECODE VT_SetIntCallback(DEVID dwDevice,void (*func)(),DWORD Imask)
{
	PREGS bd_addr=GET_STRUCT_PTR(dwDevice);
	if (Imask & VRP_INT)
		dwDevice->pVrpIntProc=(DWORD) func;
	if (Imask & B_DONE_INT)
		dwDevice->pBDoneIntProc=(DWORD) func;
	if (Imask & A_DONE_INT)
		dwDevice->pADoneIntProc=(DWORD) func;				
	if (Imask & VSYNC_INT)
		dwDevice->pVsyncIntProc=(DWORD) func;				
	return VT_NO_ERROR;
}					
					

ECODE VT_ModifyInts(DEVID dwDevice,DWORD Imask,DWORD values)
{
	PREGS bd_addr=GET_STRUCT_PTR(dwDevice);
	Imask=(Imask & 0xf)<<4;
//	bd_addr->IntCtrl = (bd_addr->IntCtrl & ~Imask) | ( (values<<4) & Imask);
	return VT_NO_ERROR;
}
#endif
/* read from a binary file with the format 
DWORD	Dest
DWORD	Points
DWORD	Data[Points]
Next block
The last block will have a Dest and Points of 0
*/

ECODE VT_INTERFACE VT_LoadFile(DEVID dwDevice,VT_MEMTYPE Type,char *filename)
{
	char dataFileName[128];
	HANDLE hFile = NULL;
	HANDLE hMapFile = NULL;
	DWORD *pData;
	DWORD *ptr;
	ECODE ecode=VT_NO_ERROR;


	strcpy(dataFileName,dwDevice->szTablePath);
	strcat(dataFileName,filename);
	strcpy(VT_ErrorMessage,dataFileName);

	/* open first file and memory map it */
    hFile = CreateFile(dataFileName, GENERIC_READ, 0, NULL, OPEN_EXISTING, FILE_FLAG_RANDOM_ACCESS, NULL);                              
    if (hFile== INVALID_HANDLE_VALUE) 	{ecode=VT_ERR_FILE_NOT_FOUND; goto error;}

    hMapFile = CreateFileMapping(hFile, NULL, PAGE_READONLY, 0, 0, NULL);
    if (hMapFile== INVALID_HANDLE_VALUE) {ecode=VT_ERR_FILE_NOT_FOUND; goto error;}

    ptr= pData = (DWORD*)MapViewOfFile(hMapFile, FILE_MAP_READ, 0,	0, 0);    	                
    if (pData== NULL) {ecode=VT_ERR_FILE_NOT_FOUND; goto error;}

	while (!(*ptr==0 && ptr[1]==0))
	{
		ecode=VT_Write(dwDevice,Type,*ptr,ptr+2,ptr[1]);
		ptr+=ptr[1]+2;
	}
	*VT_ErrorMessage=0;
error:
    if (pData != NULL && pData!=INVALID_HANDLE_VALUE) {
        UnmapViewOfFile (pData);
    }        
    if (hMapFile != NULL&& hMapFile!=INVALID_HANDLE_VALUE) {
        CloseHandle (hMapFile);
    }        
    if (hFile != NULL && hFile!=INVALID_HANDLE_VALUE) {
        CloseHandle (hFile);
    }
	if (ecode)
	{
		wsprintf(dwDevice->ErrorMessage,"Could Not open File %s\n",dataFileName);
		dwDevice->FatalError(ecode,dwDevice->ErrorMessage);
	}
	return ecode;
}

ECODE VT_INTERFACE VT_LoadFileBin(DEVID dwDevice,VT_MEMTYPE Type,char *filename,DWORD Addr,DWORD LastAddr,DWORD FileOffset)
{

	char dataFileName[128];
	HANDLE hFile = NULL;
	HANDLE hMapFile = NULL;
	DWORD *pData;
	ECODE ecode=VT_NO_ERROR;
	DWORD FileSize;

	strcpy(dataFileName,dwDevice->szTablePath);
	strcat(dataFileName,filename);
	strcpy(VT_ErrorMessage,dataFileName);

	/* open first file and memory map it */
    hFile = CreateFile(dataFileName, GENERIC_READ, 0, NULL, OPEN_EXISTING, FILE_FLAG_RANDOM_ACCESS, NULL);                              
    if (hFile== INVALID_HANDLE_VALUE) 
		{ecode=VT_ERR_FILE_NOT_FOUND; goto error;}
	
	FileSize=GetFileSize(hFile,NULL);
	if (FileSize<(FileOffset+LastAddr-Addr+1)*sizeof(DWORD))
	{
		LastAddr=Addr+(FileSize/sizeof(DWORD)-FileOffset)-1;
	}
	if (Addr > LastAddr)
		{ecode=VT_ERR_PROGRAMED_RANGE; goto error;}

    hMapFile = CreateFileMapping(hFile, NULL, PAGE_READONLY, 0, 0, NULL);
    if (hMapFile== INVALID_HANDLE_VALUE) {ecode=VT_ERR_FILE_NOT_FOUND; goto error;}

    pData = (DWORD*)MapViewOfFile(hMapFile, FILE_MAP_READ, 0,	0, 0);    	                
    if (pData== NULL) {ecode=VT_ERR_FILE_NOT_FOUND; goto error;}

	ecode=VT_Write(dwDevice,Type,Addr,pData+FileOffset,LastAddr-Addr+1);
	*VT_ErrorMessage=0;
error:
    if (pData != NULL && pData!=INVALID_HANDLE_VALUE) {
        UnmapViewOfFile (pData);
    }        
    if (hMapFile != NULL&& hMapFile!=INVALID_HANDLE_VALUE) {
        CloseHandle (hMapFile);
    }        
    if (hFile != NULL && hFile!=INVALID_HANDLE_VALUE) {
        CloseHandle (hFile);
    }
	if (ecode)
	{
		wsprintf(dwDevice->ErrorMessage,"Could Not open File %s\n",dataFileName);
		dwDevice->FatalError(ecode,dwDevice->ErrorMessage);
	}
	
	return ecode;
}

ECODE VT_INTERFACE VT_Read(DEVID dwDevice, VT_MEMTYPE Type, DWORD *DestPtr, DWORD Source, DWORD Words)
{
    PREGS bd_addr=GET_STRUCT_PTR(dwDevice);
	switch (Type)
	{
	case VR_DRAM:
		Source = ((Source<<2) & 0x007ffffc) | VRP_CHIP;
		break;

	case VR_GBUS:
		Source = ((Source<<2) & 0x000000fc) | 0x00800000 | VRP_CHIP;
		break;

	case VR_WBUS:
		Source = ((Source<<2) & 0x0000007c) | 0x00a00000 | VRP_CHIP;
		break;
	case VR_IMEM:
		return ImemRead(bd_addr,DestPtr, Source, Words);
	case VR_SMEM:
		return SmemRead(bd_addr,DestPtr, Source, Words);
	case VR_RMEM: 
		return RmemRead(bd_addr,DestPtr, Source, Words);
	case VR_AFIFO: 
		return FifoARead(bd_addr,DestPtr, Source, Words);
	case VR_BFIFO: 
		return FifoBRead(bd_addr,DestPtr, Source, Words);
	case CR_REGS:
	    return CrRead(bd_addr,DestPtr, Source, Words);
	case SR_REGS:
		return dwDevice->ReadVIP(DestPtr, Source, Words);
	case SR_FLASH:
		return VT_ERR_UNSUPPORTED; //dwDevice->ReadFlash(DestPtr, Source, Words);
	}
	return VrpRead(bd_addr,DestPtr, Source, Words);
}


ECODE VT_Write(DEVID dwDevice, VT_MEMTYPE Type, DWORD Dest, const DWORD *SrcPtr, DWORD Words)

{
    PREGS bd_addr=GET_STRUCT_PTR(dwDevice);
	switch (Type)
	{
	case VR_DRAM:
		Dest = ((Dest<<2) & 0x007ffffc) | VRP_CHIP;
		break;

	case VR_GBUS:
		Dest = ((Dest<<2) & 0x000000fc) | 0x00800000 | VRP_CHIP;
		break;

	case VR_WBUS:
		Dest = ((Dest<<2) & 0x0000007c) | 0x00a00000 | VRP_CHIP;
		break;
	case VR_IMEM:
		return ImemWrite(bd_addr, Dest, SrcPtr, Words);
	case VR_SMEM:
		return SmemWrite(bd_addr, Dest, SrcPtr, Words);
	case VR_RMEM: 
		return RmemWrite(bd_addr, Dest, SrcPtr, Words);
	case VR_AFIFO: 
		return FifoAWrite(bd_addr, Dest, SrcPtr, Words);
	case VR_BFIFO: 
		return FifoBWrite(bd_addr, Dest, SrcPtr, Words);
	case CR_REGS:
	    return CrWrite(bd_addr, Dest, SrcPtr, Words);
	case SR_REGS:
		return dwDevice->WriteVIP(Dest, SrcPtr, Words);
	case SR_FLASH:
		return 	VT_ERR_UNSUPPORTED; //dwDevice->WriteFlash(Dest, SrcPtr, Words);
	}
	return VrpWrite(bd_addr, Dest, SrcPtr, Words);
}

ECODE VT_ReadModifyWrite(DEVID dwDevice, VT_MEMTYPE Type, DWORD Dest, DWORD Mask, DWORD Data)

{
    PREGS bd_addr=GET_STRUCT_PTR(dwDevice);
    DWORD prevData;

	switch (Type)
	{
	case VR_DRAM:
		Dest = ((Dest<<2) & 0x007ffffc) | VRP_CHIP;
		break;

	case VR_GBUS:
		Dest = ((Dest<<2) & 0x000000fc) | 0x00800000 | VRP_CHIP;
		break;

	case VR_WBUS:
		Dest = ((Dest<<2) & 0x0000007c) | 0x00a00000 | VRP_CHIP;
		break;
	case VR_IMEM:
	case VR_SMEM:
	case VR_RMEM: 
		return VT_ERR_UNSUPPORTED;
	case CR_REGS:
	    return CrChange(bd_addr, Dest, Mask, Data);
	case SR_REGS:
			   dwDevice->ReadVIP(&prevData, Dest, 1);
			   prevData=(prevData & ~Mask) | (Data & Mask);
		return dwDevice->WriteVIP( Dest, &prevData,1);
	case SR_FLASH:
			   dwDevice->ReadFlash( &prevData, Dest, 1);
			   prevData=(prevData & ~Mask) | (Data & Mask);
		return 	dwDevice->WriteFlash( Dest, &prevData,1);
	}
	return VrpChange(bd_addr, Dest, Mask, Data);
}
