// AudioLevel.h : header file
//
// WHAT IS IT?
// HOW IS IT INSTALLED?
// HOW IS IT USED?
// HOW DOES IT WORK? Please refer to the code file portion of the
//   module.
#ifndef AUDIOLEVEL_H
#define AUDIOLEVEL_H

#include <tlhelp32.h> // needed for tool help declarations  

/////////////////////////////////////////////////////////////////////////////
// CAudioLevel

// Status is supplied by a code which is stored in a publically
// available member item.
typedef enum
{
	CAudioLevelSTATUSCONTROLS,											//mixerGetLineControls err
	CAudioLevelSTATUSDETAILS,											//mixerGetControlDetails err
	CAudioLevelSTATUSLINE,												//mixerGetLineInfo err
	CAudioLevelSTATUSMEMORY,											//out of mem
	CAudioLevelSTATUSMIXERS,											//no mxrs
	CAudioLevelSTATUSNAMEEMPTY,											//Name mixer name is empty
	CAudioLevelSTATUSOK,												//OK
	CAudioLevelSTATUSRUN,												//Run failed: utility not there?
	CAudioLevelSTATUSRUNNING,											//Run: utility already running
	CAudioLevelSTATUSSET,												//mixerSetControlDetails err
	CAudioLevelSTATUSSYSMIX												//can't find sys mxr
} CAudioLevelSTATUS;


// Type definitions for pointers to call tool help functions. 
// We currently are only using a subset of the functions,
// but they're all included for future reference.
typedef BOOL (WINAPI *MODULEWALK)(HANDLE hSnapshot, 
    LPMODULEENTRY32 lpme); 
typedef BOOL (WINAPI *THREADWALK)(HANDLE hSnapshot, 
    LPTHREADENTRY32 lpte); 
typedef BOOL (WINAPI *PROCESSWALK)(HANDLE hSnapshot, 
    LPPROCESSENTRY32 lppe); 
typedef HANDLE (WINAPI *CREATESNAPSHOT)(DWORD dwFlags, 
    DWORD th32ProcessID); 
 
class CToolHelp
{
public:
	// -- Instantiation ---
	CToolHelp();

	// --- ToolHelp ---
	// File scope globals. These pointers are declared because of the need 
	// to dynamically link to the functions.  They are exported only by 
	// the Windows 95 kernel. Explicitly linking to them will make this 
	// application unloadable in Microsoft(R) Windows NT(TM) and will 
	// produce an ugly system dialog box. 
	CREATESNAPSHOT m_pCreateToolhelp32Snapshot; 
	MODULEWALK  m_pModule32First; 
	MODULEWALK  m_pModule32Next; 
	PROCESSWALK m_pProcess32First; 
	PROCESSWALK m_pProcess32Next; 
	THREADWALK  m_pThread32First; 
	THREADWALK  m_pThread32Next; 
	
	// TRUE only if toolhelp initialization succeded.
	BOOL m_bHaveToolHelp;

	// This method initializes the toolhelp pointers;
	BOOL InitToolhelp32 ();

	// Returns TRUE if there is information about the specified module or  
	//   FALSE if it could not enumerate the modules in the process or 
	//   the module is not found in the process. 
	// dwPID - identifier of the process that owns the module to 
	//   retrieve information about. 
	// dwModuleID - tool help identifier of the module within the 
	//   process 
	// lpMe32 - structure to return data about the module 
	// cbMe32 - size of the buffer pointed to by lpMe32 (to ensure 
	//   the buffer is not over filled) 
	BOOL GetProcessModule (DWORD dwPID, DWORD dwModuleID, 
		    LPMODULEENTRY32 lpMe32, DWORD cbMe32);

	// Walks the system application list and finds the application specified
	// by p_szAppName.  Either the module name or the full path name may be
	// specified.  If found, the main window for the process is returned.
	HWND GetApplicationWindow (LPCTSTR p_szAppName);

	// Retrieves the window handle associated with the given process ID.
	HWND GetProcessWindow (DWORD dwProcessId);

	// This method is a callback for EnumerateWindows. It checks the process
	// ID of each top level window of currently running apps and compares it 
	// to the member variable m_dwAppProcessID. If a match is found, it sets
	// the member variable m_hWndTargetApp to the hWnd associated with the
	// process.
	static BOOL CALLBACK EnumWindowFunc (HWND hWnd, LPARAM lParam);

	// Windows is just way too goofy! After going throw the hastle of walking  
	// through all the running processes to find the app we want, and then 
	// walking through all the topmost window until we get the one associated
	// with that process, we still may not have the main window for the app in
	// question. The top level window may be the app button on the taskbar if
	// the window is minimized.  IsIconic() and OpenIcon() just doesn't work
	// on this window.  So, we enumerate all the windows in the app thread and
	// do an IsIconic() on them. If we hit one that is iconic (minimized) we
	// know (or hope!!!) that this is the handle to the minimized app and set 
	// the member variable m_hWndTargetApp accordingly.
	static BOOL CALLBACK EnumThreadWindowsFunc (HWND hWnd, LPARAM lParam);

	// required by EnumWindowFunc
	HWND m_hWndTargetApp;
	DWORD m_dwAppProcessID;	

};


class CAudioLevel
{
public:
	// --- Constant(s) ---
	// The maximum volume value contains the maximum value which may be
	// requested.
	static const unsigned int	m_kMaximumVOLUME;

	// The utility name constant contains the default name to be used
	// for the mixer utility. The value is "SndVol32.EXE".
	static const CString	m_kUtilityNAME;

	// The utility path constant contains the default path to be used
	// for the mixer utility. The value is the CString Empty value.
	static const CString	m_kUtilityPATH;

	// --- Status ---
	// The status member item contains a code which identifies the
	// success or anomaly state of the most recent method invocation.
	// Typically, "get" methods do not set the status. See the
	// CAudioLevelSTATUS declaration. The default value is
	// CAudioLevelSTATUSOK.
	static CAudioLevelSTATUS	m_Status;

	// --- Utility ---
	// The utility name methods retrieve and set the name to be used
	// for the mixer utility. The default name is contained in the
	// publically available constant m_kUtilityNAME. Together with the
	// mixer utility path (see /UtilityPath), the mixer name supplies
	// the location of the system's mixer utility.
	static CString /*name*/	Name(){return m_UtilityName;}
	static CAudioLevelSTATUS /*OK?*/	Name(CString p_Name = m_kUtilityNAME);	//new name(opt)

	// The utility path methods retrieve and set the path to be used
	// for the mixer utility. The default path is contained in the
	// publically available constant m_kUtilityPATH. Together with the
	// utility name, the utility path supplies the location of the
	// system's mixer utility. If the utility path is empty, the
	// software searches for the utility in the MS-Windows system
	// directory. When specified, a utility path need not have the
	// trailing backward slash present: if missing, one is
	// automatically appended.
	static CAudioLevelSTATUS /*OK?*/	Path(CString p_Path = m_kUtilityPATH);	//new path(opt)
	static CString /*path*/	Path(){return m_UtilityPath;}

	// The run utility method invokes the system mixer utility. The
	// ready method merely returns CAudioLevelSTATUSOK if the utility
	// is not running (use this method to prevent multiple instances
	// of the utility).
	static CAudioLevelSTATUS /*OK?*/	Ready();
	static CAudioLevelSTATUS /*OK?*/	Run();
	static HWND CAudioLevel::GetUtilityWindow();



	// --- Volume ---
	// The playing volume methods retrieve and set the playing
	// volume level.
	static unsigned int /*level*/	PlayingVolume();
	static CAudioLevelSTATUS /*OK?*/	PlayingVolume(unsigned int p_Volume);	//new vol

	// The recording volume methods retrieve and set the recording
	// volume level.
	static unsigned int /*level*/	RecordingVolume();
	static CAudioLevelSTATUS /*OK?*/	RecordingVolume(unsigned int p_Volume);	//new vol

	// The recording peak method retrieves the recording
	// peak recording level.
	static unsigned int /*level*/	RecordingPeak();

protected:
	static PROCESS_INFORMATION	m_ProcessInformation;					//proc info
	static CString	m_UtilityName;										//utl name
	static CString	m_UtilityPath;										//utl path

	static CAudioLevelSTATUS /*OK?*/	RunUtility();
	static CAudioLevelSTATUS /*new status*/	Status(
		CAudioLevelSTATUS p_Status);									//new sts
};
#endif
