// bwdialog.cpp : Defines the initialization routines for the DLL.
//

#include "stdafx.h"
#include "bwdialog.h"
#include "limits.h"
#include "videobuf.h"
#include "voverlay.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CBwdialogApp

BEGIN_MESSAGE_MAP(CBwdialogApp, CWinApp)
	//{{AFX_MSG_MAP(CBwdialogApp)
		// NOTE - the ClassWizard will add and remove mapping macros here.
		//    DO NOT EDIT what you see in these blocks of generated code!
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CBwdialogApp construction

CBwdialogApp::CBwdialogApp()
{  
	// TODO: add construction code here,
	// Place all significant initialization in InitInstance
}

CBwdialogApp::~CBwdialogApp()
{  
	m_pszHelpFilePath = m_pszOldHelpFilePath;
}

/////////////////////////////////////////////////////////////////////////////
// CBwdialogApp initialization

BOOL CBwdialogApp::InitInstance()
{
	HKEY KeyHandle;

	m_StartupDirectory.Empty();

    /* try to get the path to the broadway installation directory */
    if (RegOpenKeyEx(HKEY_LOCAL_MACHINE, 
            TEXT("SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\App Paths\\BROADWAY.exe"), 
            0L,
            KEY_ALL_ACCESS,
            &KeyHandle) == ERROR_SUCCESS) {

	    /* try to get the path to the broadway app */    
		DWORD dwType, cbData = 256;
		long lResult = RegQueryValueEx(
			    KeyHandle,              
				TEXT("Path"),
				NULL,                  
				&dwType,              
				(unsigned char*)m_StartupDirectory.GetBufferSetLength(256),          
				&cbData);

		
	    m_StartupDirectory.ReleaseBuffer();
        RegCloseKey(KeyHandle);
		
		if (lResult == ERROR_SUCCESS) {
			m_StartupDirectory = m_StartupDirectory.Left(m_StartupDirectory.Find ("PROGRAM"));
		    m_StartupDirectory += TEXT("Broadway.hlp");
		}
    }

	/* if that didn't work, try an alternate method - use the startup directory */
	if (m_StartupDirectory.IsEmpty()) {
		GetCurrentDirectory (256, m_StartupDirectory.GetBufferSetLength(256));
		m_StartupDirectory.ReleaseBuffer();
	    m_StartupDirectory += "Broadway.hlp";
	}

	/* set the app help file path and registry key */
	m_pszOldHelpFilePath = m_pszHelpFilePath;
	m_pszHelpFilePath = (LPCSTR)m_StartupDirectory;          
	SetRegistryKey( TEXT("Data Translation"));
	return TRUE;
}              

/////////////////////////////////////////////////////////////////////////////
// The one and only CBwdialogApp object

CBwdialogApp theApp;


/////////////////////////////////////////////////////////////////////////////
// CRegistrySettings

CRegistrySettings::CRegistrySettings()
{
	m_bChangedAVI = FALSE;
	m_bChangedMPG = FALSE;
	m_EnableHW = FALSE;
	m_State = Uninitialized;

	// create the path to  the ini file
	BW_GetMpegIniFileName (m_iniFile.GetBufferSetLength(MAX_PATH),MAX_PATH-1);

	m_iniFile.ReleaseBuffer();
}


VOID CRegistrySettings::ValidateCapturePreferences (BW_CAPTURE_PREFERENCES *pOptions)
{
	/* validate color settings */
	if (pOptions->nHue < 0 || pOptions->nHue > 100) {
		pOptions->nHue = DefaultHue();
	}
	if (pOptions->nSaturation < 0 || pOptions->nSaturation > 100) {
		pOptions->nSaturation = DefaultSaturation();
	}
	if (pOptions->nIntensity < 0 || pOptions->nIntensity > 100) {
		pOptions->nHue = DefaultIntensity();
	}
	if (pOptions->nContrast < 0 || pOptions->nContrast > 100) {
		pOptions->nContrast = DefaultContrast();
	}

	/* validate video bit rate */
	if (pOptions->nBitRate < 35 || pOptions->nBitRate > 70) {
		pOptions->nBitRate = DefaultBitRate();
	}

	/* validate audio capture rate */
	switch (pOptions->nAudioRate) {
		case AUDIO_RATE_11KHZ:
		case AUDIO_RATE_22KHZ:
		case AUDIO_RATE_44KHZ:
			break;

		default:
			pOptions->nAudioRate = DefaultAudioRate();
	}

	/* validate audio mode */
	switch (pOptions->nAudioMode) {

		case AUDIO_MONO:
		case AUDIO_STEREO:
			break;

		default:
			pOptions->nAudioMode = DefaultAudioMode();
	}

	/* validate blank lines */
	if (pOptions->nBlankLines > 10) {
		pOptions->nBlankLines = 10;
	}
	if (pOptions->nBlankLines < 0) {
		pOptions->nBlankLines = 0;
	}

	/* validate capture mode */
	switch (pOptions->nCaptureMode) {
		
		case CAPMODE_IFRAME:
		case CAPMODE_RTMPEG:
			break;

		default:
			pOptions->nCaptureMode = DefaultCaptureMode();
	}
}



const static int MonoRates[]  ={32000,48000,56000,64000,80000,96000,112000,128000,160000,192000};
#define NumMonoRates (sizeof(MonoRates)/sizeof(int))

const static int StereoRates[]={64000,96000,112000,128000,160000,192000,224000,256000,320000,384000};
#define NumStereoRates (sizeof(StereoRates)/sizeof(int))

const static int AudioRates[] = {32000,48000,56000,64000,80000,96000,112000,128000,160000,192000,224000,256000,320000,384000};
#define NumAudioRates (sizeof(AudioRates)/sizeof(int))


BOOL CRegistrySettings::ValidatePlayBackTarget (LPCTSTR Target)
{
	BOOL bFoundTarget = FALSE;
	CString Search;
	int loop=1;
	
	if (Target == NULL || lstrlen(Target) == 0) {
		return FALSE;
	}

	Search.Empty();

	/* verify the playback target exists */
	do {
		char t1[10]="Target",t2[10];
		strcat(t1,itoa(loop++,t2,10));

		GetPrivateProfileString(m_EncodeType,t1,TEXT(""),Search.GetBufferSetLength(80),79,m_iniFile);
		Search.ReleaseBuffer();

		if (Search == Target) {
			bFoundTarget = TRUE;
			break;
		}
	} while (Search != TEXT(""));


	return bFoundTarget;
}



BOOL CRegistrySettings::ValidatePlayBackTarget (LPCTSTR Target, LPCTSTR EncodeType)
{
	BOOL bFoundTarget = FALSE;
	CString Search;
	int loop=1;
	
	if (Target == NULL || lstrlen(Target) == 0) {
		return FALSE;
	}
	if (EncodeType == NULL || lstrlen(EncodeType) == 0) {
		return FALSE;
	}

	Search.Empty();

	/* verify the playback target exists */
	do {
		char t1[10]="Target",t2[10];
		strcat(t1,itoa(loop++,t2,10));

		GetPrivateProfileString(EncodeType,t1,TEXT(""),Search.GetBufferSetLength(80),79,m_iniFile);
		Search.ReleaseBuffer();

		if (Search == Target) {
			bFoundTarget = TRUE;
			break;
		}
	} while (Search != TEXT(""));


	return bFoundTarget;
}


BOOL CRegistrySettings::UpdateEncodeType (LPCTSTR Target)
{
	BOOL bFoundTarget = FALSE;
	CString Search;
	int loop=1;

	if (Target == NULL || lstrlen(Target) == 0) {
		return FALSE;
	}

	Search.Empty();

	/* Get the encode type associated with the given Target */
	do {
		char t1[10]="Type",t2[10];
		strcat(t1,itoa(loop++,t2,10));

		GetPrivateProfileString(TEXT("RecodeTypes"),t1,TEXT(""),Search.GetBufferSetLength(80),79,m_iniFile);
		Search.ReleaseBuffer();

		if (ValidatePlayBackTarget (Target, Search)) {
			m_EncodeType = Search;
			bFoundTarget = TRUE;
			break;
		}
	} while (Search != TEXT(""));

	if (!bFoundTarget) {
		m_EncodeType = DefaultEncodeType();		
	}

	return bFoundTarget;
}


BOOL CRegistrySettings::ValidateEncodeType (LPCTSTR Target)
{
	BOOL bFoundTarget = FALSE;
	CString Search;
	int loop=1;

	if (Target == NULL || lstrlen(Target) == 0) {
		return FALSE;
	}

	Search.Empty();

	/* verify the playback type exists */
	do {
		char t1[10]="Type",t2[10];
		strcat(t1,itoa(loop++,t2,10));

		GetPrivateProfileString(TEXT("RecodeTypes"),t1,TEXT(""),Search.GetBufferSetLength(80),79,m_iniFile);
		Search.ReleaseBuffer();

		if (Search == Target) {
			bFoundTarget = TRUE;
			break;
		}
	} while (Search != TEXT(""));


	return bFoundTarget;
}


VOID CRegistrySettings::ValidateCompressionPreferences(BW_RECODE_PREFERENCES *pOptions)
{

	// if it doesn't exist use the first type as the default
	if (!ValidatePlayBackTarget(pOptions->EncodePlaybackTarget)) {						
		GetPrivateProfileString(m_EncodeType,TEXT("Target1"),TEXT("Hard Disk"),pOptions->EncodePlaybackTarget,29,m_iniFile);
	}

	CString Target = pOptions->EncodePlaybackTarget;

	// get video settings
	int nMaxVideoRate = GetPrivateProfileInt(Target,"MaxVideoRate",3000000,m_iniFile);
	int nMinVideoRate = GetPrivateProfileInt(Target,"MinVideoRate",400000,m_iniFile);
	int nDefaultVideoRate = GetPrivateProfileInt(Target,"DefaultVideoRate",1008000,m_iniFile);
	int nVideoQsif = GetPrivateProfileInt(Target,"VideoQsif",FALSE,m_iniFile);
	int nStreamMixType = GetPrivateProfileInt(Target,"StreamMixType",0,m_iniFile);

	//verify the audio rate based on min / max
	if (nMaxVideoRate == nMinVideoRate) {
		pOptions->nEncodeVideoBitRate = nDefaultVideoRate;		
	} else if (pOptions->nEncodeVideoBitRate > nMaxVideoRate) {
		pOptions->nEncodeVideoBitRate = nMaxVideoRate;	
	} else if (pOptions->nEncodeVideoBitRate < nMinVideoRate) {
		pOptions->nEncodeVideoBitRate = nMinVideoRate;	
	}

	// verify video enable
	if (!pOptions->nEncodeVideoEnable) {
		if (nStreamMixType == 2) {
			pOptions->nEncodeVideoEnable = TRUE;
		}
	}

	//
	if (nVideoQsif) {
		pOptions->nLargeImage = 0;
	} else if (nVideoQsif) {
		pOptions->nLargeImage = 1;
	}

	// get audio settings from ini file
	int nMaxAudioRate = GetPrivateProfileInt(Target,"MaxAudioRate",256000,m_iniFile);
	int nMinAudioRate = GetPrivateProfileInt(Target,"MinAudioRate",48000,m_iniFile);
	int nDefaultAudioRate = GetPrivateProfileInt(Target,"DefaultAudioRate",192000,m_iniFile);
	int nAudioEnable = GetPrivateProfileInt(Target,"AudioEnable",1,m_iniFile);
	int nMonoAudioEnabled = GetPrivateProfileInt(Target,"MonoAudioEnable",TRUE,m_iniFile);
	int nStereoAudioEnabled = GetPrivateProfileInt(Target,"StereoAudioEnable",TRUE,m_iniFile);
	int nAudioType = GetPrivateProfileInt(Target,"AudioType",2,m_iniFile);

	// verify audio enable setting
	if (pOptions->nEncodeAudioEnable) {
		if (!nAudioEnable) {
			pOptions->nEncodeAudioEnable = FALSE;
		}
	} else if (nStreamMixType == 2) {
		pOptions->nEncodeAudioEnable = TRUE;
	}

	// verify audio mode
	switch (pOptions->nEncodeAudioMode) {

		case AUDIO_MONO:
			if (!nMonoAudioEnabled) {
				pOptions->nEncodeAudioMode = nAudioType;
			}
			break;

		case AUDIO_JOINTSTEREO:
		case AUDIO_DUALCHANNEL:
		case AUDIO_STEREO:
			if (!nStereoAudioEnabled) {
				pOptions->nEncodeAudioMode = nAudioType;
			}
			break;

		default:
			pOptions->nEncodeAudioMode = nAudioType;
	}

	// audio rates are specific, verify based on audio mode
	if (pOptions->nEncodeAudioMode == AUDIO_MONO) {

		int i, nAudioRate = 0, nDiff = INT_MAX;
		for (i=0;i<NumMonoRates;i++) {
		
			if (MonoRates[i] == pOptions->nEncodeAudioBitRate) {
				nAudioRate = pOptions->nEncodeAudioBitRate;
				break;
			}

			if (abs (MonoRates[i] - nAudioRate) < nDiff) {
				nAudioRate = MonoRates[i];
			}
		}
		 pOptions->nEncodeAudioBitRate = nAudioRate;
	} else {

		int i, nAudioRate = 0, nDiff = INT_MAX;
		for (i=0;i<NumStereoRates;i++) {
		
			if (StereoRates[i] == pOptions->nEncodeAudioBitRate) {
				nAudioRate = pOptions->nEncodeAudioBitRate;
				break;
			}

			if (abs (StereoRates[i] - nAudioRate) < nDiff) {
				nAudioRate = StereoRates[i];
			}
		}
		pOptions->nEncodeAudioBitRate = nAudioRate;
	}

	//verify the audio rate based on min / max
	if (nMaxAudioRate == nMinAudioRate) {
		pOptions->nEncodeAudioBitRate = nDefaultAudioRate;		
	} else if (pOptions->nEncodeAudioBitRate > nMaxAudioRate) {
		pOptions->nEncodeAudioBitRate = nMaxAudioRate;	
	} else if (pOptions->nEncodeAudioBitRate < nMinAudioRate) {
		pOptions->nEncodeAudioBitRate = nMinAudioRate;	
	}
}


UINT CRegistrySettings::CaptureInitialize(BW_BOARDINFO *pInfo, PVC_PROFILE_INFO pProf)
{
	BOOL bCreated;

    //make sure we're not already initialized
    if (IsInitialized()) {
        return FALSE;
    }
    
    //make a local copy of the board info
    m_pProf    = pProf;

    if (pInfo != NULL) {
		m_vh       = pInfo->vh;
		m_DevID    = pInfo->DevID;
		m_bCapture = pInfo->bCapture;
		m_bOverlay = pInfo->bOverlay;

		m_EnableHW = TRUE;
	} else {
		m_EnableHW = FALSE;
	}

    m_State = Capture;

	bCreated = CreateDefaultSettings();

	m_nAutoDetect 			= GetInitialSetting(IDS_AUTODETECT);
	m_nNTSC 				= GetInitialSetting(IDS_NTSC);
	m_nComposite 			= GetInitialSetting(IDS_COMPOSITE);

	m_nAudioEnable			= GetInitialSetting(IDS_AUDIOENABLE);
	m_nAudioMode			= GetInitialSetting(IDS_AUDIOMODE);
	m_nAudioRate			= GetInitialSetting(IDS_AUDIORATE);

	m_nHue 					= GetInitialSetting(IDS_HUE);
	m_nSaturation 			= GetInitialSetting(IDS_SATURATION);
	m_nIntensity 			= GetInitialSetting(IDS_INTENSITY);
	m_nContrast 			= GetInitialSetting(IDS_CONTRAST);
	m_nFilterValue 			= GetInitialSetting(IDS_FILTERVALUE);
	m_nBlankLines           = GetInitialSetting(IDS_BLANKLINES);

	m_nBitRate 				= GetInitialSetting(IDS_BITRATE);
	m_nCaptureMode 			= GetInitialSetting(IDS_CAPTUREMODE);
	m_nShowVolumeHelp		= GetInitialSetting(IDS_SHOWVOLUMEHELP);

	/* validate capture mode */
	switch (m_nCaptureMode) {
		
		case CAPMODE_IFRAME:
		case CAPMODE_RTMPEG:
			break;

		default:
			m_nCaptureMode = DefaultCaptureMode();
	}

	CString Target, Section;

	int loop=1;
	BOOL bBase = FALSE;
	Target.Empty();

	while (TRUE) {

		char t1[10]="Target",t2[10];
		strcat(t1,itoa(loop++,t2,10));

		GetPrivateProfileString("RecodeTargets",t1,"",Target.GetBufferSetLength(80),79,m_iniFile);
		Target.ReleaseBuffer();

		if (Target == "") {
			break;
		}

		CreateInitialTargetStringSetting(Target, IDS_ENCODEPLAYBACKTARGET);

		CreateInitialTargetSetting(Target, IDS_ENCODEVIDEOENABLE);
		CreateInitialTargetSetting(Target, IDS_ENCODEVIDEORATE);
		CreateInitialTargetSetting(Target, IDS_ENCODESOFTENINGLEVEL);
		CreateInitialTargetSetting(Target, IDS_ENCODESQUAREPIXELS);
		CreateInitialTargetSetting(Target, IDS_LARGEIMAGE);

		CreateInitialTargetSetting(Target, IDS_ENCODEAUDIOENABLE);
		CreateInitialTargetSetting(Target, IDS_ENCODEAUDIORATE);
		CreateInitialTargetSetting(Target, IDS_ENCODEAUDIOMODE);

		CreateInitialTargetSetting(Target, IDS_COMPRESSIONESTIMATE);

		CreateInitialTargetSetting(Target, IDS_ENCODEUSEOPTIMUMRATE);
		CreateInitialTargetSetting(Target, IDS_ENCODESTREAMDATARATE);
		CreateInitialTargetSetting(Target, IDS_ENCODEPACKSIZE);
		CreateInitialTargetSetting(Target, IDS_ENCODEPACKETSPERPACK);
		CreateInitialTargetSetting(Target, IDS_STREAMMIXTYPE);
	}

	CString Type;
	Type.Empty();

	loop=1;

	/* create initial playback targets for all types */
   	while (TRUE) {

		char t1[10]="Type",t2[10];
		strcat(t1,itoa(loop++,t2,10));

		Type.Empty();

		/* get the next recode type */
		GetPrivateProfileString(TEXT("RecodeTypes"),t1,TEXT(""),Type.GetBufferSetLength(80),79,m_iniFile);
		Type.ReleaseBuffer();

		if (Type == TEXT("")) {
			break;
		}

		Target.Empty();

		/* get the current target for this type */
		Target = GetStringSetting (IDS_SECTION, Type);
		if (!ValidatePlayBackTarget (Target, Type)) { 
			GetPrivateProfileString(Type,TEXT("Target1"),TEXT(""),Target.GetBufferSetLength(80),79,m_iniFile);
			Target.ReleaseBuffer();
		}
		 
		if (Target != "") {
			SetStringSetting( IDS_SECTION, Type, Target);
		}
	} 

	m_EncodeType = GetInitialStringSetting(IDS_ENCODETYPE);
	m_EncodePlayBackTarget = GetStringSetting(IDS_SECTION, m_EncodeType);		

	if (!ValidateEncodeType(m_EncodeType)) {
		m_EncodeType = DefaultEncodeType();		
	}

	if (!ValidatePlayBackTarget(m_EncodePlayBackTarget)) {
		m_EncodePlayBackTarget = GetStringSetting(IDS_DEFAULTS, m_EncodeType);		
	}

	Target.LoadString(IDS_SECTION);
	Target += TEXT("\\");
	Target += m_EncodePlayBackTarget;

	m_nEncodeVideoEnable 	= GetSetting(Target, IDS_ENCODEVIDEOENABLE, (UINT)~0);
	m_nEncodeVideoBitRate   = GetSetting(Target, IDS_ENCODEVIDEORATE, (UINT)~0);
	m_nEncodeSofteningLevel = GetSetting(Target, IDS_ENCODESOFTENINGLEVEL, (UINT)~0);
	m_nEncodeSquarePixels	= GetSetting(Target, IDS_ENCODESQUAREPIXELS, (UINT)~0);
	m_nLargeImage 			= GetSetting(Target, IDS_LARGEIMAGE, (UINT)~0);

	m_nEncodeAudioEnable 	= GetSetting(Target, IDS_ENCODEAUDIOENABLE, (UINT)~0);
	m_nEncodeAudioBitRate 	= GetSetting(Target, IDS_ENCODEAUDIORATE, (UINT)~0);
	m_nEncodeAudioMode 		= GetSetting(Target, IDS_ENCODEAUDIOMODE, (UINT)~0);

    m_nCompressionEstimate  = GetSetting(Target, IDS_COMPRESSIONESTIMATE, (UINT)~0);

	m_nEncodeUseOptimumRate	= GetSetting(Target, IDS_ENCODEUSEOPTIMUMRATE, (UINT)~0);
	m_nEncodeStreamDataRate = GetSetting(Target, IDS_ENCODESTREAMDATARATE, (UINT)~0);
	m_nEncodePackSize       = GetSetting(Target, IDS_ENCODEPACKSIZE, (UINT)~0);
	m_nEncodePacketsPerPack = GetSetting(Target, IDS_ENCODEPACKETSPERPACK, (UINT)~0);
    m_nStreamMixType		= GetSetting(Target, IDS_STREAMMIXTYPE, (UINT)~0);

    return TRUE;
}


BOOL CRegistrySettings::ReadEncodeType(LPCTSTR lpszEncodeType)
{
	if (!ValidateEncodeType(lpszEncodeType)) {
		return FALSE;
	}

	CString Target = GetStringSetting(IDS_SECTION, lpszEncodeType);		

	if (!ValidatePlayBackTarget(Target, lpszEncodeType)) {
		return FALSE;
	}

	CString Section;
	Section.LoadString(IDS_SECTION);
	Section += TEXT("\\") + Target;
	
	ReadTargetSettings(Section);
	m_EncodeType = lpszEncodeType;
	m_EncodePlayBackTarget = Target;

	return TRUE;
}

UINT CRegistrySettings::RecodeInitialize()
{
	BOOL bCreated;

    //make sure we're not already initialized
    if (IsInitialized()) {
        return FALSE;
    }
    
    m_pProf = NULL;
    m_State = Encode;

	bCreated = CreateDefaultSettings();

	CString Target, Section;

	int loop=1;
	BOOL bBase = FALSE;
	Target.Empty();

	while (TRUE) {

		char t1[10]="Target",t2[10];
		strcat(t1,itoa(loop++,t2,10));

		GetPrivateProfileString("RecodeTargets",t1,"",Target.GetBufferSetLength(80),79,m_iniFile);
		Target.ReleaseBuffer();

		if (Target == "") {
			break;
		}

		CreateInitialTargetStringSetting(Target, IDS_ENCODEPLAYBACKTARGET);

		CreateInitialTargetSetting(Target, IDS_ENCODEVIDEOENABLE);
		CreateInitialTargetSetting(Target, IDS_ENCODEVIDEORATE);
		CreateInitialTargetSetting(Target, IDS_ENCODESOFTENINGLEVEL);
		CreateInitialTargetSetting(Target, IDS_ENCODESQUAREPIXELS);
		CreateInitialTargetSetting(Target, IDS_LARGEIMAGE);

		CreateInitialTargetSetting(Target, IDS_ENCODEAUDIOENABLE);
		CreateInitialTargetSetting(Target, IDS_ENCODEAUDIORATE);
		CreateInitialTargetSetting(Target, IDS_ENCODEAUDIOMODE);

		CreateInitialTargetSetting(Target, IDS_COMPRESSIONESTIMATE);

		CreateInitialTargetSetting(Target, IDS_ENCODEUSEOPTIMUMRATE);
		CreateInitialTargetSetting(Target, IDS_ENCODESTREAMDATARATE);
		CreateInitialTargetSetting(Target, IDS_ENCODEPACKSIZE);
		CreateInitialTargetSetting(Target, IDS_ENCODEPACKETSPERPACK);
		CreateInitialTargetSetting(Target, IDS_STREAMMIXTYPE);
	}

	CString Type;
	Type.Empty();

	loop=1;

	/* create initial playback targets for all types */
   	while (TRUE) {

		char t1[10]="Type",t2[10];
		strcat(t1,itoa(loop++,t2,10));

		Type.Empty();

		/* get the next recode type */
		GetPrivateProfileString(TEXT("RecodeTypes"),t1,TEXT(""),Type.GetBufferSetLength(80),79,m_iniFile);
		Type.ReleaseBuffer();

		if (Type == TEXT("")) {
			break;
		}

		Target.Empty();

		/* get the current target for this type */
		Target = GetStringSetting (IDS_SECTION, Type);
		if (!ValidatePlayBackTarget (Target, Type)) { 
			GetPrivateProfileString(Type,TEXT("Target1"),TEXT(""),Target.GetBufferSetLength(80),79,m_iniFile);
			Target.ReleaseBuffer();
		}
		 
		if (Target != "") {
			SetStringSetting( IDS_SECTION, Type, Target);
		}
	} 

	m_EncodeType = GetInitialStringSetting(IDS_ENCODETYPE);
	m_EncodePlayBackTarget = GetStringSetting(IDS_SECTION, m_EncodeType);		

	if (!ValidateEncodeType(m_EncodeType)) {
		m_EncodeType = DefaultEncodeType();		
	}

	if (!ValidatePlayBackTarget(m_EncodePlayBackTarget)) {
		m_EncodePlayBackTarget = GetStringSetting(IDS_DEFAULTS, m_EncodeType);		
	}

	Target.LoadString(IDS_SECTION);
	Target += TEXT("\\");
	Target += m_EncodePlayBackTarget;

	m_nEncodeVideoEnable 	= GetSetting(Target, IDS_ENCODEVIDEOENABLE, (UINT)~0);
	m_nEncodeVideoBitRate   = GetSetting(Target, IDS_ENCODEVIDEORATE, (UINT)~0);
	m_nEncodeSofteningLevel = GetSetting(Target, IDS_ENCODESOFTENINGLEVEL, (UINT)~0);
	m_nEncodeSquarePixels	= GetSetting(Target, IDS_ENCODESQUAREPIXELS, (UINT)~0);
	m_nLargeImage 			= GetSetting(Target, IDS_LARGEIMAGE, (UINT)~0);

	m_nEncodeAudioEnable 	= GetSetting(Target, IDS_ENCODEAUDIOENABLE, (UINT)~0);
	m_nEncodeAudioBitRate 	= GetSetting(Target, IDS_ENCODEAUDIORATE, (UINT)~0);
	m_nEncodeAudioMode 		= GetSetting(Target, IDS_ENCODEAUDIOMODE, (UINT)~0);

    m_nCompressionEstimate  = GetSetting(Target, IDS_COMPRESSIONESTIMATE, (UINT)~0);

	m_nEncodeUseOptimumRate	= GetSetting(Target, IDS_ENCODEUSEOPTIMUMRATE, (UINT)~0);
	m_nEncodeStreamDataRate = GetSetting(Target, IDS_ENCODESTREAMDATARATE, (UINT)~0);
	m_nEncodePackSize       = GetSetting(Target, IDS_ENCODEPACKSIZE, (UINT)~0);
	m_nEncodePacketsPerPack = GetSetting(Target, IDS_ENCODEPACKETSPERPACK, (UINT)~0);
    m_nStreamMixType		= GetSetting(Target, IDS_STREAMMIXTYPE, (UINT)~0);

    return TRUE;
}

void CRegistrySettings::WriteTargetSettings(CString sTarget) {

	SetStringSetting(sTarget, IDS_ENCODEPLAYBACKTARGET, m_EncodePlayBackTarget);

	SetSetting(sTarget,IDS_ENCODEVIDEOENABLE,m_nEncodeVideoEnable);
	SetSetting(sTarget,IDS_ENCODEVIDEORATE,m_nEncodeVideoBitRate);
	SetSetting(sTarget,IDS_ENCODESOFTENINGLEVEL,m_nEncodeSofteningLevel);
	SetSetting(sTarget,IDS_ENCODESQUAREPIXELS,m_nEncodeSquarePixels);
	SetSetting(sTarget,IDS_LARGEIMAGE,m_nLargeImage);

	SetSetting(sTarget,IDS_ENCODEAUDIOENABLE,m_nEncodeAudioEnable);
	SetSetting(sTarget,IDS_ENCODEAUDIORATE,m_nEncodeAudioBitRate);
	SetSetting(sTarget,IDS_ENCODEAUDIOMODE,m_nEncodeAudioMode);

    SetSetting(sTarget,IDS_COMPRESSIONESTIMATE,m_nCompressionEstimate);

	SetSetting(sTarget,IDS_ENCODEUSEOPTIMUMRATE,m_nEncodeUseOptimumRate);
	SetSetting(sTarget,IDS_ENCODESTREAMDATARATE,m_nEncodeStreamDataRate);
	SetSetting(sTarget,IDS_ENCODEPACKSIZE,m_nEncodePackSize);
	SetSetting(sTarget,IDS_ENCODEPACKETSPERPACK,m_nEncodePacketsPerPack);
    SetSetting(sTarget,IDS_STREAMMIXTYPE,m_nStreamMixType);	

}

// Read the registry setting for the specified capture or recode target
void CRegistrySettings::ReadTargetSettings(CString sTarget) {
	
	m_EncodePlayBackTarget  = GetStringSetting(sTarget,IDS_ENCODEPLAYBACKTARGET);

	m_nEncodeVideoEnable 	= GetSetting(sTarget,IDS_ENCODEVIDEOENABLE,m_nEncodeVideoEnable);
	m_nEncodeVideoBitRate   = GetSetting(sTarget,IDS_ENCODEVIDEORATE,m_nEncodeVideoBitRate);
	m_nEncodeSofteningLevel = GetSetting(sTarget,IDS_ENCODESOFTENINGLEVEL,m_nEncodeSofteningLevel);
	m_nEncodeSquarePixels	= GetSetting(sTarget,IDS_ENCODESQUAREPIXELS,m_nEncodeSquarePixels);
	m_nLargeImage 			= GetSetting(sTarget,IDS_LARGEIMAGE,m_nLargeImage);

	m_nEncodeAudioEnable 	= GetSetting(sTarget,IDS_ENCODEAUDIOENABLE,m_nEncodeAudioEnable);
	m_nEncodeAudioBitRate 	= GetSetting(sTarget,IDS_ENCODEAUDIORATE,m_nEncodeAudioBitRate);
	m_nEncodeAudioMode 		= GetSetting(sTarget,IDS_ENCODEAUDIOMODE,m_nEncodeAudioMode);

    m_nCompressionEstimate  = GetSetting(sTarget,IDS_COMPRESSIONESTIMATE,m_nCompressionEstimate);

	m_nEncodeUseOptimumRate	= GetSetting(sTarget,IDS_ENCODEUSEOPTIMUMRATE,m_nEncodeUseOptimumRate);
	m_nEncodeStreamDataRate = GetSetting(sTarget,IDS_ENCODESTREAMDATARATE,m_nEncodeStreamDataRate);
	m_nEncodePackSize       = GetSetting(sTarget,IDS_ENCODEPACKSIZE,m_nEncodePackSize);
	m_nEncodePacketsPerPack = GetSetting(sTarget,IDS_ENCODEPACKETSPERPACK,m_nEncodePacketsPerPack);
     m_nStreamMixType		= GetSetting(sTarget,IDS_STREAMMIXTYPE,m_nStreamMixType);

}

UINT CRegistrySettings::Terminate()
{
    //make sure we're initialized
    if (!IsInitialized()) {
        return FALSE;
    }

	//update the registry
    if (IsCapture()) {

	    SetSetting( IDS_SECTION, IDS_AUTODETECT, m_nAutoDetect);
	    SetSetting( IDS_SECTION, IDS_NTSC, m_nNTSC);
	    SetSetting( IDS_SECTION, IDS_COMPOSITE, m_nComposite);

		SetSetting( IDS_SECTION, IDS_AUDIOENABLE, m_nAudioEnable);
	    SetSetting( IDS_SECTION, IDS_AUDIOMODE, m_nAudioMode);
	    SetSetting( IDS_SECTION, IDS_AUDIORATE, m_nAudioRate);

		SetSetting( IDS_SECTION, IDS_HUE, m_nHue);
	    SetSetting( IDS_SECTION, IDS_SATURATION, m_nSaturation);
	    SetSetting( IDS_SECTION, IDS_INTENSITY, m_nIntensity);
	    SetSetting( IDS_SECTION, IDS_CONTRAST, m_nContrast);
	    SetSetting( IDS_SECTION, IDS_FILTERVALUE, m_nFilterValue);

		SetSetting( IDS_SECTION, IDS_BITRATE, m_nBitRate);
		SetSetting( IDS_SECTION, IDS_CAPTUREMODE, m_nCaptureMode);
		SetSetting( IDS_SECTION, IDS_SHOWVOLUMEHELP, m_nShowVolumeHelp);
												
		SetSetting( IDS_SECTION, IDS_BLANKLINES, m_nBlankLines);
	} 
	  
	SetStringSetting( IDS_SECTION, IDS_ENCODEPLAYBACKTARGET, m_EncodePlayBackTarget);
	SetStringSetting( IDS_DEFAULTS, m_EncodeType, m_EncodePlayBackTarget);
	SetStringSetting( IDS_SECTION, IDS_ENCODETYPE, m_EncodeType);

	SetStringSetting(IDS_SECTION, m_EncodeType, m_EncodePlayBackTarget);		

	SetSetting( IDS_SECTION, IDS_ENCODEVIDEOENABLE, m_nEncodeVideoEnable);
	SetSetting( IDS_SECTION, IDS_ENCODEVIDEORATE, m_nEncodeVideoBitRate);
	SetSetting( IDS_SECTION, IDS_ENCODESOFTENINGLEVEL, m_nEncodeSofteningLevel);
	SetSetting( IDS_SECTION, IDS_ENCODESQUAREPIXELS, m_nEncodeSquarePixels);
	SetSetting( IDS_SECTION, IDS_LARGEIMAGE, m_nLargeImage);
	
	SetSetting( IDS_SECTION, IDS_ENCODEAUDIOENABLE, m_nEncodeAudioEnable);
	SetSetting( IDS_SECTION, IDS_ENCODEAUDIORATE, m_nEncodeAudioBitRate);
	SetSetting( IDS_SECTION, IDS_ENCODEAUDIOMODE, m_nEncodeAudioMode);
	
	SetSetting( IDS_SECTION, IDS_ENCODEUSEOPTIMUMRATE, m_nEncodeUseOptimumRate);
	SetSetting( IDS_SECTION, IDS_ENCODESTREAMDATARATE, m_nEncodeStreamDataRate);
	SetSetting( IDS_SECTION, IDS_ENCODEPACKSIZE, m_nEncodePackSize);
	SetSetting( IDS_SECTION, IDS_ENCODEPACKETSPERPACK, m_nEncodePacketsPerPack);
	SetSetting( IDS_SECTION, IDS_COMPRESSIONESTIMATE, m_nCompressionEstimate);
	SetSetting( IDS_SECTION, IDS_STREAMMIXTYPE, m_nStreamMixType);

	CString Target;
		
	Target.LoadString(IDS_SECTION);
	Target += TEXT("\\");
	Target += m_EncodePlayBackTarget;

	WriteTargetSettings(Target);

	m_State = Uninitialized;
    return TRUE;
}

CRegistrySettings::~CRegistrySettings()
{
    //make sure we're terminated
    if (IsInitialized()) {
        Terminate();
    }
}

 
UINT CRegistrySettings::CreateDefaultSettings()
{
    //make sure we're initialized
    if (!IsInitialized()) {
        return FALSE;
    }

    if (IsCapture()) {
        CreateDefaultSetting(IDS_AUTODETECT, 1);
	    CreateDefaultSetting(IDS_NTSC, 1);
	    CreateDefaultSetting(IDS_COMPOSITE, 1);

		CreateDefaultSetting(IDS_AUDIOENABLE, 1);
	    CreateDefaultSetting(IDS_AUDIORATE, AUDIO_RATE_11KHZ);
	    CreateDefaultSetting(IDS_AUDIOMODE, AUDIO_MONO);
	    
		CreateDefaultSetting(IDS_HUE, 50);
	    CreateDefaultSetting(IDS_SATURATION, 50);
	    CreateDefaultSetting(IDS_INTENSITY, 50);
	    CreateDefaultSetting(IDS_CONTRAST, 50);
	    CreateDefaultSetting(IDS_FILTERVALUE, 0);
		CreateDefaultSetting(IDS_BLANKLINES, 0);

		CreateDefaultSetting(IDS_BITRATE, 55);
	    CreateDefaultSetting(IDS_CAPTUREMODE, CAPMODE_IFRAME);
	    CreateDefaultSetting(IDS_SHOWVOLUMEHELP, 1);
    } 

	CString Target, Type;
	Target.Empty();
	Type.Empty();


	GetPrivateProfileString(TEXT("RecodeTypes"),TEXT("Type1"),TEXT(""),Type.GetBufferSetLength(80),79,m_iniFile);
	Type.ReleaseBuffer();

	if (Type == "") {
		return FALSE;
	}

	CreateDefaultStringSetting (IDS_ENCODETYPE, Type);

	GetPrivateProfileString(Type,TEXT("Target1"),TEXT(""),Target.GetBufferSetLength(80),79,m_iniFile);
	Target.ReleaseBuffer();

	if (Target == "") {
		return FALSE;
	}

	CreateDefaultStringSetting (IDS_ENCODEPLAYBACKTARGET, Target);

	CString  Section;
	int loop=1;

	/* create default playback targets for all types */
   	while (TRUE) {

		char t1[10]="Type",t2[10];
		strcat(t1,itoa(loop++,t2,10));

		Type.Empty();

		/* get the next recode type */
		GetPrivateProfileString(TEXT("RecodeTypes"),t1,TEXT(""),Type.GetBufferSetLength(80),79,m_iniFile);
		Type.ReleaseBuffer();

		if (Type == TEXT("")) {
			break;
		}

		Target.Empty();

		/* get the default target for this type */
		GetPrivateProfileString(Type,TEXT("Target1"),TEXT(""),Target.GetBufferSetLength(80),79,m_iniFile);
		Target.ReleaseBuffer();

		if (Target == "") {
			continue;
		}
		
		SetStringSetting( IDS_DEFAULTS, Type, Target);
	} 


	int nVideoRate = GetPrivateProfileInt(Target,"DefaultVideoRate",1150000,m_iniFile);	
	int DisableQsif = GetPrivateProfileInt(Target,"DisableQsif",TRUE,m_iniFile);
	int VideoQsif = DisableQsif ?GetPrivateProfileInt(Target,"VideoQsif",FALSE,m_iniFile):FALSE;
		
	CreateDefaultSetting (IDS_ENCODEVIDEOENABLE, 1);
	CreateDefaultSetting (IDS_ENCODEVIDEORATE, nVideoRate);

	int DisableSquare = GetPrivateProfileInt(Target,"DisableSquarePixels",TRUE,m_iniFile);
	int VideoSquare = !DisableSquare ? GetPrivateProfileInt(Target,"VideoSquarePixels",FALSE,m_iniFile):FALSE;
	CreateDefaultSetting (IDS_ENCODESQUAREPIXELS, VideoSquare ? 1:0);

	CreateDefaultSetting (IDS_ENCODESOFTENINGLEVEL, 0);
	CreateDefaultSetting (IDS_LARGEIMAGE, VideoQsif ?0:1);
	
	int nAudioEnable = GetPrivateProfileInt(Target,"AudioEnable",1,m_iniFile);
	int nAudioRate = GetPrivateProfileInt(Target,"DefaultAudioRate",192000,m_iniFile);	
	int nAudioType = GetPrivateProfileInt(Target,"AudioType",AUDIO_JOINTSTEREO,m_iniFile);

	CreateDefaultSetting (IDS_ENCODEAUDIOENABLE, nAudioEnable ?1:0);
	CreateDefaultSetting (IDS_ENCODEAUDIORATE, nAudioRate);
	CreateDefaultSetting (IDS_ENCODEAUDIOMODE, nAudioType);

	int nCompressionEstimate = nVideoRate;
	if (nAudioEnable) {
		nCompressionEstimate += nAudioRate;
	}
	
	CreateDefaultSetting (IDS_COMPRESSIONESTIMATE, nCompressionEstimate);

	int nUseOptimumRate = GetPrivateProfileInt(Target,"OptimumStreamRate",1,m_iniFile);
	int nStreamDataRate = GetPrivateProfileInt(Target,"StreamDataRate",5600000,m_iniFile);
	int nPackSize = GetPrivateProfileInt(Target,"StreamPackSize",2048,m_iniFile);
	int nPacketsPerPack = GetPrivateProfileInt(Target,"PacketsPerPack",0,m_iniFile);
	int nStreamMixType = GetPrivateProfileInt(Target,"StreamMixType",0,m_iniFile);

	CreateDefaultSetting (IDS_ENCODEUSEOPTIMUMRATE, nUseOptimumRate);
	CreateDefaultSetting (IDS_ENCODESTREAMDATARATE, nStreamDataRate);
	CreateDefaultSetting (IDS_ENCODEPACKSIZE, nPackSize);
	CreateDefaultSetting (IDS_ENCODEPACKETSPERPACK, nPacketsPerPack);
	CreateDefaultSetting (IDS_STREAMMIXTYPE, nStreamMixType);

	/* create defaults for all playback targets */
	loop=1;
	while (TRUE) {

		char t1[10]="Target",t2[10];
		strcat(t1,itoa(loop++,t2,10));
	
		Target.Empty();

		GetPrivateProfileString(TEXT("RecodeTargets"),t1,TEXT(""),Target.GetBufferSetLength(80),79,m_iniFile);
		Target.ReleaseBuffer();

		if (Target == TEXT("")) {
			break;
		}

		Section.LoadString(IDS_DEFAULTS);
		Section += TEXT("\\");
		Section += Target;
	
		nVideoRate = GetPrivateProfileInt(Target,"DefaultVideoRate",1150000,m_iniFile);	
		DisableQsif = GetPrivateProfileInt(Target,"DisableQsif",TRUE,m_iniFile);
		VideoQsif = DisableQsif ?GetPrivateProfileInt(Target,"VideoQsif",FALSE,m_iniFile):FALSE;

		SetStringSetting( Section, IDS_ENCODEPLAYBACKTARGET, Target);
		
		SetSetting( Section, IDS_ENCODEVIDEOENABLE, 1);
		SetSetting( Section, IDS_ENCODEVIDEORATE, nVideoRate);
		DisableSquare = GetPrivateProfileInt(Target,"DisableSquarePixels",TRUE,m_iniFile);
		VideoSquare = !DisableSquare ? GetPrivateProfileInt(Target,"VideoSquarePixels",FALSE,m_iniFile):FALSE;
		SetSetting( Section, IDS_ENCODESQUAREPIXELS, VideoSquare ? 1:0);

		SetSetting( Section, IDS_ENCODESOFTENINGLEVEL, 0);
		SetSetting( Section, IDS_LARGEIMAGE, VideoQsif ?0:1);
	
		nAudioEnable = GetPrivateProfileInt(Target,"AudioEnable",1,m_iniFile);
		nAudioRate = GetPrivateProfileInt(Target,"DefaultAudioRate",192000,m_iniFile);	
		nAudioType = GetPrivateProfileInt(Target,"AudioType",AUDIO_JOINTSTEREO,m_iniFile);

		SetSetting( Section, IDS_ENCODEAUDIOENABLE, nAudioEnable ?1:0);
		SetSetting( Section, IDS_ENCODEAUDIORATE, nAudioRate);
		SetSetting( Section, IDS_ENCODEAUDIOMODE, nAudioType);

		nCompressionEstimate = nVideoRate;
		if (nAudioEnable) {
			nCompressionEstimate += nAudioRate;
		}
	
		SetSetting( Section, IDS_COMPRESSIONESTIMATE, nCompressionEstimate);

		nUseOptimumRate = GetPrivateProfileInt(Target,"OptimumStreamRate",1,m_iniFile);
		nStreamDataRate = GetPrivateProfileInt(Target,"StreamDataRate",5600000,m_iniFile);
		nPackSize = GetPrivateProfileInt(Target,"StreamPackSize",2048,m_iniFile);
		nPacketsPerPack = GetPrivateProfileInt(Target,"PacketsPerPack",0,m_iniFile);
		nStreamMixType = GetPrivateProfileInt(Target,"StreamMixType",0,m_iniFile);

		SetSetting( Section, IDS_ENCODEUSEOPTIMUMRATE, nUseOptimumRate);
		SetSetting( Section, IDS_ENCODESTREAMDATARATE, nStreamDataRate);
		SetSetting( Section, IDS_ENCODEPACKSIZE, nPackSize);
		SetSetting( Section, IDS_ENCODEPACKETSPERPACK, nPacketsPerPack);
		SetSetting( Section, IDS_STREAMMIXTYPE, nStreamMixType);
	}

	return( TRUE );
}

UINT CRegistrySettings::CreateDefaultSetting(UINT nSetting, UINT nValue)
{
    //make sure we're initialized
    if (!IsInitialized()) {
        return FALSE;
    }

    UINT nDefault = GetSetting(IDS_DEFAULTS, nSetting);
    
	if (nDefault != nValue) {
  		SetSetting( IDS_DEFAULTS, nSetting, nValue);
        return TRUE;
    }

    return FALSE;
}


UINT CRegistrySettings::CreateDefaultStringSetting(UINT nSetting, CString sValue)
{
    //make sure we're initialized
    if (!IsInitialized()) {
        return FALSE;
    }

	CString sDefault = GetStringSetting(IDS_DEFAULTS, nSetting);

	if (sDefault != sValue) {
		SetStringSetting( IDS_DEFAULTS, nSetting, sValue);
		return TRUE;
    }

    return FALSE;
}

UINT CRegistrySettings::CreateInitialTargetSetting(CString Target, UINT nSetting) 
{
	//make sure we're initialized
    if (!IsInitialized()) {
        return FALSE;
    }
	
	CString sSection;

	sSection.LoadString(IDS_SECTION);
	sSection += TEXT("\\");
	sSection += Target;

    UINT nValue = GetSetting(sSection, nSetting, (UINT)~0);

    if (nValue == (UINT)~0) {
		CString sDefault;
   	
		sDefault.LoadString(IDS_DEFAULTS);
		sDefault += "\\";
		sDefault += Target;

		SetSetting( sSection, nSetting, GetSetting(sDefault, nSetting, (UINT)~0));
	} 

	return TRUE;
}
 
UINT CRegistrySettings::GetInitialSetting(UINT nSetting) 
{
	//make sure we're initialized
    if (!IsInitialized()) {
        return FALSE;
    }
	
	UINT nSection;
    UINT nValue = GetSetting(IDS_SECTION, nSetting);

    if (nValue == (UINT)~0) {
	    nSection = IDS_DEFAULTS; 
	} else {
	    nSection = IDS_SECTION;
	}

	return GetSetting( nSection, nSetting);
}
 
UINT CRegistrySettings::CreateInitialTargetStringSetting(CString Target, UINT nSetting) 
{
	//make sure we're initialized
    if (!IsInitialized()) {
        return FALSE;
    }
	
	CString sSection;

	sSection.LoadString(IDS_SECTION);
	sSection += TEXT("\\");
	sSection += Target;

	CString sValue = GetStringSetting(sSection, nSetting);

    if (sValue == TEXT("")) {    
		CString sDefault;
   	
		sDefault.LoadString(IDS_DEFAULTS);
		sDefault += TEXT("\\");
		sDefault += Target;

		SetStringSetting( sSection, nSetting, GetStringSetting(sDefault, nSetting));
	} 

	return TRUE;
}
 
CString CRegistrySettings::GetInitialStringSetting(UINT nSetting) 
{
 //make sure we're initialized
    if (!IsInitialized()) {
        m_CString = TEXT("");
        return m_CString;
    }
	
    UINT nSection;
	CString sValue = GetStringSetting(IDS_SECTION, nSetting);

    if (sValue == TEXT("")) {
	    nSection = IDS_DEFAULTS; 
	} else {
	    nSection = IDS_SECTION;
	}

	return GetStringSetting( nSection, nSetting);
	
}

UINT CRegistrySettings::GetSetting( CString sSection, UINT nSetting,
								   UINT nDefault)
{
	CString  sSetting;

    //make sure we're initialized
    if (!IsInitialized()) {
        return (UINT)~0;
    }

	sSetting.LoadString(nSetting);

    UINT nValue = nDefault;

#ifdef _SEPERATE_SETTINGS    
    if (!IsEncode()) 
	{
        nValue = VC_ReadProfileUser (m_pProf
			                         ,(LPTSTR)(LPCTSTR)sSection
									 ,(LPTSTR)(LPCTSTR)sSetting
									 ,nDefault
									 );
	} 
else  
#endif
	{    
       	CBwdialogApp *pApp = (CBwdialogApp *)AfxGetApp();
        nValue = pApp->GetProfileInt(sSection, sSetting, nDefault);
    }

	return( nValue );
}


UINT CRegistrySettings::GetSetting( UINT nSection, UINT nSetting )
{
	CString sSection, sSetting;

    //make sure we're initialized
    if (!IsInitialized()) {
        return (UINT)~0;
    }

	sSetting.LoadString(nSetting);
	sSection.LoadString(nSection);

    UINT nValue;
    
    if (!IsEncode()) {
        nValue = VC_ReadProfileUser (m_pProf
			                         ,(LPTSTR)(LPCTSTR)sSection
									 ,(LPTSTR)(LPCTSTR)sSetting
									 ,(UINT)~0
									 );
	} else  {    
       	CBwdialogApp *pApp = (CBwdialogApp *)AfxGetApp();
        nValue = pApp->GetProfileInt(sSection, sSetting, (UINT)~0);
    }

	return( nValue );
}


CString CRegistrySettings::GetStringSetting( UINT nSection, UINT nSetting )
{
	CString sSection, sSetting;

	m_CString.Empty();

    //make sure we're initialized
    if (!IsInitialized()) {
        return m_CString;
    }

	sSetting.LoadString(nSetting);
	sSection.LoadString(nSection);

    if (!IsEncode()) {
		VC_ReadProfileUserString(m_pProf
		                         ,(LPTSTR)(LPCTSTR)sSection
								 ,(LPTSTR)(LPCTSTR)sSetting
								 ,m_CString.GetBufferSetLength(MAX_PATH)
								 ,MAX_PATH
								 );
		
		m_CString.ReleaseBuffer();

    } else {
       	CBwdialogApp *pApp = (CBwdialogApp *)AfxGetApp();
        m_CString = pApp->GetProfileString(sSection, sSetting, "");
    }

	return( m_CString );
}


CString CRegistrySettings::GetStringSetting( UINT nSection, CString sSetting )
{
	CString sSection;

	m_CString.Empty();

    //make sure we're initialized
    if (!IsInitialized()) {
        return m_CString;
    }

	sSection.LoadString(nSection);

    if (!IsEncode()) {
		VC_ReadProfileUserString(m_pProf
		                         ,(LPTSTR)(LPCTSTR)sSection
								 ,(LPTSTR)(LPCTSTR)sSetting
								 ,m_CString.GetBufferSetLength(MAX_PATH)
								 ,MAX_PATH
								 );
		
		m_CString.ReleaseBuffer();

    } else {
       	CBwdialogApp *pApp = (CBwdialogApp *)AfxGetApp();
        m_CString = pApp->GetProfileString(sSection, sSetting, "");
    }

	return( m_CString );
}


CString CRegistrySettings::GetStringSetting( CString sSection, UINT nSetting )
{
	CString sSetting;

	m_CString.Empty();

    //make sure we're initialized
    if (!IsInitialized()) {
        return m_CString;
    }

	sSetting.LoadString(nSetting);

#ifdef _SEPERATE_SETTINGS    
    if (!IsEncode()) 
	{
		VC_ReadProfileUserString(m_pProf
		                         ,(LPTSTR)(LPCTSTR)sSection
								 ,(LPTSTR)(LPCTSTR)sSetting
								 ,m_CString.GetBufferSetLength(MAX_PATH)
								 ,MAX_PATH
								 );
		
		m_CString.ReleaseBuffer();

    } 
	else 
#endif
	{
       	CBwdialogApp *pApp = (CBwdialogApp *)AfxGetApp();
        m_CString = pApp->GetProfileString(sSection, sSetting, "");
    }

	return( m_CString );
}


UINT CRegistrySettings::SetSetting( UINT nSection, UINT nSetting, int nValue )
{
	CString sSection, sSetting;

    //make sure we're initialized
    if (!IsInitialized()) {
        return FALSE;
    }

	sSetting.LoadString(nSetting);
	sSection.LoadString(nSection);
    
    BOOL bWorked;
    
    if (!IsEncode()) {
        bWorked = VC_WriteProfileUser (m_pProf, (LPTSTR)(LPCTSTR)sSection, (LPTSTR)(LPCTSTR)sSetting, nValue);
    } else {
       	CBwdialogApp *pApp = (CBwdialogApp *)AfxGetApp();
        bWorked = pApp->WriteProfileInt(sSection, sSetting, nValue);
    }

	return( bWorked );
}

UINT CRegistrySettings::SetSetting( CString sSection, UINT nSetting, int nValue )
{
	CString sSetting;

    //make sure we're initialized
    if (!IsInitialized()) {
        return FALSE;
    }

	sSetting.LoadString(nSetting);
    
    BOOL bWorked;
    
#ifdef _SEPERATE_SETTINGS    
    if (!IsEncode()) 
	{
        bWorked = VC_WriteProfileUser (m_pProf, (LPTSTR)(LPCTSTR)sSection, (LPTSTR)(LPCTSTR)sSetting, nValue);
    } 
	else 
#endif
	{
       	CBwdialogApp *pApp = (CBwdialogApp *)AfxGetApp();
        bWorked = pApp->WriteProfileInt(sSection, sSetting, nValue);
    }

	return( bWorked );
}
                   
UINT CRegistrySettings::SetStringSetting( UINT nSection, UINT nSetting, CString sValue )
{
	CString sSection, sSetting;

    //make sure we're initialized
    if (!IsInitialized()) {
        return FALSE;
    }

	sSetting.LoadString(nSetting);
	sSection.LoadString(nSection);
    
    BOOL bWorked;
    
    if (!IsEncode()) {
		bWorked = VC_WriteProfileUserString(m_pProf
						                   ,(LPTSTR)(LPCTSTR)sSection
						            	   ,(LPTSTR)(LPCTSTR)sSetting
						            	   ,(LPTSTR)(LPCTSTR)sValue
								           //,sValue.GetBufferSetLength(MAX_PATH)
								           ,sValue.GetLength() + 1
								           );
		
		sValue.ReleaseBuffer();

    } else {
       	CBwdialogApp *pApp = (CBwdialogApp *)AfxGetApp();
        bWorked = pApp->WriteProfileString(sSection, sSetting, sValue);
    }

	return( bWorked );
}
                   
UINT CRegistrySettings::SetStringSetting( UINT nSection, CString sSetting, CString sValue )
{
	CString sSection;

    //make sure we're initialized
    if (!IsInitialized()) {
        return FALSE;
    }

	sSection.LoadString(nSection);
    
    BOOL bWorked;
    
    if (!IsEncode()) {
		bWorked = VC_WriteProfileUserString(m_pProf
						                   ,(LPTSTR)(LPCTSTR)sSection
						            	   ,(LPTSTR)(LPCTSTR)sSetting
						            	   ,(LPTSTR)(LPCTSTR)sValue
								           //,sValue.GetBufferSetLength(MAX_PATH)
								           ,sValue.GetLength() + 1
								           );
		
		sValue.ReleaseBuffer();

    } else {
       	CBwdialogApp *pApp = (CBwdialogApp *)AfxGetApp();
        bWorked = pApp->WriteProfileString(sSection, sSetting, sValue);
    }

	return( bWorked );
}
                   
UINT CRegistrySettings::SetStringSetting( CString sSection, UINT nSetting, CString sValue )
{
	CString sSetting;

    //make sure we're initialized
    if (!IsInitialized()) {
        return FALSE;
    }

	sSetting.LoadString(nSetting);
    
    BOOL bWorked;
    
#ifdef _SEPERATE_SETTINGS    
    if (!IsEncode()) 
	{
		bWorked = VC_WriteProfileUserString(m_pProf
						                   ,(LPTSTR)(LPCTSTR)sSection
						            	   ,(LPTSTR)(LPCTSTR)sSetting
						            	   ,(LPTSTR)(LPCTSTR)sValue
								           //,sValue.GetBufferSetLength(MAX_PATH)
								           ,sValue.GetLength() + 1
								           );
		
		sValue.ReleaseBuffer();

    } 
	else 
#endif
	{
       	CBwdialogApp *pApp = (CBwdialogApp *)AfxGetApp();
        bWorked = pApp->WriteProfileString(sSection, sSetting, sValue);
    }

	return( bWorked );
}
                   
BOOL CRegistrySettings::HW_SetVideoMode (UINT VidMode, UINT VidSource)
{
    //make sure we're initialized
    if (!IsCapture() || !m_EnableHW) {
        return FALSE;
    }

    //change the mode
    BOOL bReturn = (VT_SetupVideoMode(m_DevID, VidMode, VidSource) == VT_NO_ERROR);
        
    return bReturn;
}

BOOL CRegistrySettings::HW_GetVideoMode (UINT *pVidMode, UINT *pVidSource)
{
    ECODE ecode;

    //make sure we're initialized
    if (!IsCapture() || !m_EnableHW) {
        return FALSE;
    }

    DWORD Mode, Source;
    
    ecode = VT_GetVideoMode(m_DevID, &Mode, &Source);
    if (ecode == VT_NO_ERROR) { 
        *pVidMode = Mode;
        *pVidSource = Source;
        return TRUE;
    } 

    //sorry - no synch!!!
    return FALSE;
}



