// captvid.cpp : implementation file
//

#include "stdafx.h"
#include "bwdialog.h"
#include "captvid.h"

#include "afxpriv.h"

#ifdef _DEBUG      
#undef THIS_FILE
static char BASED_CODE THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CCaptureVideo property page

IMPLEMENT_DYNCREATE(CCaptureVideo, CPropertyPage)

void CCaptureVideo::SetModified(BOOL bChanged)
{
	m_bChanged = bChanged;
	CPropertyPage::SetModified( bChanged );
}

CCaptureVideo::CCaptureVideo() : CPropertyPage(CCaptureVideo::IDD)
{
	//{{AFX_DATA_INIT(CCaptureVideo)
	//}}AFX_DATA_INIT
	m_psp.dwFlags |= PSP_HASHELP;
	m_bChanged    = FALSE;
    m_TimerActive = FALSE;
    m_TimerID = 1;
    m_Timeout = 500;
}

CCaptureVideo::~CCaptureVideo()
{                               
}

void CCaptureVideo::DoDataExchange(CDataExchange* pDX)
{
	CPropertyPage::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CCaptureVideo)
	DDX_Control(pDX, IDC_VIDEO_DEFAULTS, m_Defaults);
	DDX_Control(pDX, IDC_AUTODETECT, m_AutoDetect);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CCaptureVideo, CPropertyPage)
	//{{AFX_MSG_MAP(CCaptureVideo)
	ON_BN_CLICKED(IDC_NTSC, OnNtsc)
	ON_BN_CLICKED(IDC_PAL, OnPal)
	ON_BN_CLICKED(IDC_SVIDEO, OnSvideo)
	ON_BN_CLICKED(IDC_COMPOSITE, OnComposite)
	ON_BN_CLICKED(IDC_AUTODETECT, OnAutodetect)
	ON_BN_CLICKED(IDC_VIDEO_DEFAULTS, OnDefaults)
	ON_WM_TIMER()
	//}}AFX_MSG_MAP
    ON_MESSAGE( WM_COMMANDHELP, OnCommandHelp )
END_MESSAGE_MAP()


/////////////////////////////////////////////////////////////////////////////
// CCaptureVideo message handlers

BOOL CCaptureVideo::OnInitDialog() 
{
    UINT nVidMode, nVidSource;

	CPropertyPage::OnInitDialog();

   	CBwdialogApp *pApp = (CBwdialogApp *)AfxGetApp();

	m_nAutoDetect = pApp->m_Settings.AutoDetect();
    m_nNtsc = pApp->m_Settings.Ntsc();
    m_nComposite = pApp->m_Settings.Composite();

    m_AutoDetect.SetCheck( m_nAutoDetect );

    if (m_nAutoDetect) {
        nVidMode = AUTO_DETECT;
        nVidSource = m_nAutoComposite ?COMPOSITE:SVIDEO;
    } else {
        nVidMode = m_nNtsc ?NTSC:PAL;
        nVidSource = m_nComposite ?COMPOSITE:SVIDEO;
    }
    
    pApp->m_Settings.HW_SetVideoMode (nVidMode, nVidSource);
    
    if (m_nAutoDetect) {
        SetTimer (m_TimerID, m_Timeout, NULL);
        m_TimerActive = TRUE;
    } else {   
        SetItems ( TRUE );
    }

	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}

void CCaptureVideo::OnDestroy( )       
{
    if (m_TimerActive) {
        KillTimer (m_TimerID);
        m_TimerActive = FALSE;           
    }

	CPropertyPage::OnDestroy();
}

BOOL CCaptureVideo::OnKillActive( )
{
	UINT nNtsc;
	UINT nComposite;

	CPropertyPage::OnKillActive();

	m_nAutoDetect = m_AutoDetect.GetCheck();
    if (!m_nAutoDetect) {
    	nNtsc = GetCheckedRadioButton( IDC_NTSC, IDC_PAL );
	    m_nNtsc = (nNtsc == IDC_NTSC);
	    nComposite = GetCheckedRadioButton( IDC_COMPOSITE, IDC_SVIDEO );
	    m_nComposite = (nComposite == IDC_COMPOSITE);
    }

	return( TRUE );
}

void CCaptureVideo::OnOK( )
{
   	CBwdialogApp *pApp = (CBwdialogApp *)AfxGetApp();

    pApp->m_Settings.SetAutoDetect( m_nAutoDetect );
	
	if (m_bChanged)
	{
		pApp->m_Settings.SetNTSC( m_nNtsc );
		pApp->m_Settings.SetComposite( m_nAutoComposite );
		pApp->m_Settings.SetComposite( m_nComposite );

		SetModified( FALSE );
	}

	CPropertyPage::OnOK();
}

void CCaptureVideo::OnTimer(UINT nIdEvent) 
{
	// TODO: Add your message handler code here and/or call default
	
	CPropertyPage::OnTimer(nIdEvent);

    if (!IsWindowVisible()) {
        return;
    }

   	int nNtscID = GetCheckedRadioButton(IDC_NTSC, IDC_PAL);
   	int nCompositeID = GetCheckedRadioButton(IDC_COMPOSITE, IDC_SVIDEO);

    SetItems ( TRUE );
}

void CCaptureVideo::OnNtsc() 
{
    SetItems ( FALSE );
	SetModified( TRUE );
}

void CCaptureVideo::OnPal() 
{
    SetItems ( FALSE );
	SetModified( TRUE );
}

void CCaptureVideo::OnSvideo() 
{
    SetItems ( FALSE );
	SetModified( TRUE );
}

void CCaptureVideo::OnComposite() 
{
    SetItems ( FALSE );
	SetModified( TRUE );
}


void CCaptureVideo::OnAutodetect() 
{
    SetItems ( TRUE );
	SetModified( TRUE );
}

void CCaptureVideo::OnDefaults() 
{
	UINT nNtsc;
	UINT nComposite;

   	CBwdialogApp *pApp = (CBwdialogApp *)AfxGetApp();

	if (!DlgEqualsDefaults())
	{
		UINT nAutoDetect = pApp->m_Settings.DefaultAutoDetect();
       	m_AutoDetect.SetCheck( nAutoDetect );

        if (nAutoDetect) {
            SetItems ( TRUE );
        } else {

		    nNtsc = pApp->m_Settings.DefaultNTSC();
    		nComposite = pApp->m_Settings.DefaultComposite();
	
		    if (nNtsc)
			    CheckRadioButton( IDC_NTSC, IDC_PAL, IDC_NTSC);
		    else
			    CheckRadioButton( IDC_NTSC, IDC_PAL, IDC_PAL);
    
	    	if (nComposite)
		    	CheckRadioButton( IDC_COMPOSITE, IDC_SVIDEO, IDC_COMPOSITE);
		    else
			    CheckRadioButton( IDC_COMPOSITE, IDC_SVIDEO, IDC_SVIDEO);

            SetItems ( FALSE );
        }

		SetModified( TRUE );
	}
}

void CCaptureVideo::SetItems (BOOL bAutoUpdate)
{
   	CBwdialogApp *pApp = (CBwdialogApp *)AfxGetApp();

	UINT nDlgAutoDetect = m_AutoDetect.GetCheck();
    UINT nNtsc, nComposite, nIsEnabled, nCheck = TRUE;

    if (nDlgAutoDetect) {
        if (!pApp->m_Settings.HW_GetVideoMode (&nNtsc, &nComposite)) {
            m_nAutoComposite = m_nAutoComposite ?0:1;
            pApp->m_Settings.HW_SetVideoMode (AUTO_DETECT, m_nAutoComposite ?COMPOSITE:SVIDEO);
            nCheck = FALSE;
        } else {
            nNtsc = (nNtsc == NTSC) ?1:0;
            nComposite = (nComposite == COMPOSITE) ?1:0;
        }

        if (!m_TimerActive) {
            SetTimer (m_TimerID, m_Timeout, NULL);
            m_TimerActive = TRUE;
        }

    } else {
        if  (!bAutoUpdate) {
            nNtsc = (GetCheckedRadioButton(IDC_NTSC, IDC_PAL) == IDC_NTSC) ?NTSC:PAL;
	        nComposite = (GetCheckedRadioButton(IDC_COMPOSITE, IDC_SVIDEO) == IDC_COMPOSITE) ?COMPOSITE:SVIDEO;
            pApp->m_Settings.HW_SetVideoMode (nNtsc, nComposite);
            return;
        } else {
            nNtsc = m_nNtsc;
            nComposite = m_nComposite;
            pApp->m_Settings.HW_SetVideoMode (nNtsc, nComposite);
        }


        if (m_TimerActive) {
            KillTimer (m_TimerID);
            m_TimerActive = FALSE;
        }
    }

    if (nCheck) {
    	if (nNtsc) {
        	if (GetCheckedRadioButton(IDC_NTSC, IDC_PAL) != IDC_NTSC)
    	    	CheckRadioButton( IDC_NTSC, IDC_PAL, IDC_NTSC);
        } else {
        	if (GetCheckedRadioButton(IDC_NTSC, IDC_PAL) != IDC_PAL)
    		    CheckRadioButton( IDC_NTSC, IDC_PAL, IDC_PAL);
        }

    	if (nComposite) {
        	if (GetCheckedRadioButton(IDC_COMPOSITE, IDC_SVIDEO) != IDC_COMPOSITE)
    	    	CheckRadioButton( IDC_COMPOSITE, IDC_SVIDEO, IDC_COMPOSITE);
        } else {
        	if (GetCheckedRadioButton(IDC_COMPOSITE, IDC_SVIDEO) != IDC_SVIDEO)
    		    CheckRadioButton( IDC_COMPOSITE, IDC_SVIDEO, IDC_SVIDEO);
        }
    } else {
        UINT nId;
       	
       	if ((nId = GetCheckedRadioButton(IDC_NTSC, IDC_PAL)) != 0) {
            CheckDlgButton(nId, 0);
        }
        
        if ((nId = GetCheckedRadioButton(IDC_COMPOSITE, IDC_SVIDEO)) != 0) {
            CheckDlgButton(nId, 0);
        }
    }

    nIsEnabled = GetDlgItem(IDC_NTSC)->IsWindowEnabled();    
    if ((nDlgAutoDetect && nIsEnabled) || (!nDlgAutoDetect && !nIsEnabled)) {
        GetDlgItem(IDC_NTSC)->EnableWindow(nDlgAutoDetect ?FALSE:TRUE);
    }    

    nIsEnabled = GetDlgItem(IDC_PAL)->IsWindowEnabled();    
    if ((nDlgAutoDetect && nIsEnabled) || (!nDlgAutoDetect && !nIsEnabled)) {
        GetDlgItem(IDC_PAL)->EnableWindow(nDlgAutoDetect ?FALSE:TRUE);
    }    

    nIsEnabled = GetDlgItem(IDC_COMPOSITE)->IsWindowEnabled();    
    if ((nDlgAutoDetect && nIsEnabled) || (!nDlgAutoDetect && !nIsEnabled)) {
        GetDlgItem(IDC_COMPOSITE)->EnableWindow(nDlgAutoDetect ?FALSE:TRUE);
    }    
    
    nIsEnabled = GetDlgItem(IDC_SVIDEO)->IsWindowEnabled();    
    if ((nDlgAutoDetect && nIsEnabled) || (!nDlgAutoDetect && !nIsEnabled)) {
        GetDlgItem(IDC_SVIDEO)->EnableWindow(nDlgAutoDetect ?FALSE:TRUE);
    }    
}

BOOL CCaptureVideo::DlgEqualsDefaults()
{
	UINT nNtsc, nDlgNtsc;
	UINT nComposite, nDlgComposite;
	UINT nAutoDetect, nDlgAutoDetect;
	UINT nId;

   	CBwdialogApp *pApp = (CBwdialogApp *)AfxGetApp();

	nAutoDetect = pApp->m_Settings.DefaultAutoDetect();
	nNtsc = pApp->m_Settings.DefaultNTSC();
	nComposite = pApp->m_Settings.DefaultComposite();

    nDlgAutoDetect = m_AutoDetect.GetCheck();	
	if (nDlgAutoDetect != nAutoDetect) {
        return FALSE;
    }
	
	nId = GetCheckedRadioButton( IDC_NTSC, IDC_PAL );
	nDlgNtsc = (nId == IDC_NTSC) ? 1 : 0;

	nId = GetCheckedRadioButton( IDC_COMPOSITE, IDC_SVIDEO );
	nDlgComposite = (nId == IDC_COMPOSITE) ? 1 : 0;

	if ((nNtsc == nDlgNtsc) && (nComposite == nDlgComposite))
		return( TRUE );
	else
		return( FALSE );
}

//xxxxx this was added 
LRESULT CCaptureVideo::WindowProc(UINT message, WPARAM wParam, LPARAM lParam) 
{
	// TODO: Add your specialized code here and/or call the base class
	if (message == WM_HELP)	{
		HELPINFO* pHelpInfo = (HELPINFO*) lParam;
		OnHelpInfo(pHelpInfo);
		return TRUE;
	} else if (message == WM_COMMAND)	{
		if (wParam == ID_HELP) {
			HELPINFO HelpInfo;

			HelpInfo.iContextType = HELPINFO_WINDOW;
			HelpInfo.iCtrlId = IDC_CAPTURE_OPTIONS;
			OnHelpInfo(&HelpInfo);
			return TRUE;
		}
	}

	return CDialog::WindowProc(message, wParam, lParam);
}


BOOL CCaptureVideo::OnHelpInfo(HELPINFO* pHelpInfo) 
{
	if ( pHelpInfo->iContextType==HELPINFO_WINDOW )
	{
		long lParam = pHelpInfo->iCtrlId + HIDC_MYBASE;
    	CBwdialogApp *pApp = (CBwdialogApp *)AfxGetApp();
    	CWnd* pMainWnd = this;

	    // return global app help mode state to FALSE (backward compatibility)
	    pApp->m_bHelpMode = FALSE;
	    pMainWnd->PostMessage(WM_KICKIDLE); // trigger idle update

	    pMainWnd->WinHelp(lParam, HELP_CONTEXT);
	}
	return TRUE;
}

LRESULT CCaptureVideo::OnCommandHelp( WPARAM, LPARAM lParam )
{
    int nCtrlID = GetFocus()->GetDlgCtrlID(); // Get the ID of the control with the current focus

	lParam = nCtrlID + HIDC_MYBASE;
   	CBwdialogApp *pApp = (CBwdialogApp *)AfxGetApp();
   	CWnd* pMainWnd = this;

    // return global app help mode state to FALSE (backward compatibility)
    pApp->m_bHelpMode = FALSE;
    pMainWnd->PostMessage(WM_KICKIDLE); // trigger idle update

    pMainWnd->WinHelp(lParam, HELP_CONTEXT);

    return TRUE;
}
