/*  mmtimers.h
 *
 * Accurate timers using pentium cpu clock, QueryPerformanceCounter
 * or GetTickCount depending on what system the code is run on
 *
 * Copyright (C) 1995, Microsoft Corporation, all rights reserved
 *
 *-========================================================================*/
 
#if !defined _INC_MMTIMERS_
#define _INC_MMTIMERS_

	struct _pctimer_global 
	{
		
		DWORD dwRawHz;
		DWORD dwTimerKhz;
    
		unsigned __int64 base;
		
		DWORD (WINAPI * DifTicks     )(unsigned __int64  *);
		DWORD (WINAPI * DifMicrosec  )(unsigned __int64  *);
		DWORD (WINAPI * DifMillisec  )(unsigned __int64  *);
		DWORD (WINAPI * DeltaTicks   )(unsigned __int64  *);
		DWORD (WINAPI * DeltaMicrosec)(unsigned __int64  *);
		DWORD (WINAPI * DeltaMillisec)(unsigned __int64  *);
	};

extern struct _pctimer_global pc;
extern VOID WINAPI InitPerformanceCounters ();

#define pcBegin()          pc.DeltaTicks(&pc.base)
#define pcGetTime()        pc.DifMillisec(&pc.base)
#define pcGetTicks()       pc.DifMicrosec(&pc.base)
#define pcGetTickRate()    (pc.dwTimerKhz * 1000)
#define pcBeginTimer(ppt)  (pc.DeltaMicrosec(ppt), 0)
#define pcDeltaTicks(ppt)  pc.DeltaMicrosec(ppt)


#endif //_INC_MMTIMERS_



// =============================================================================

//
// include this in only one module in a DLL or APP
//   
#if (defined _INC_MMTIMERS_CODE_) && (_INC_MMTIMERS_CODE_ != FALSE)
#undef _INC_MMTIMERS_CODE_
#define _INC_MMTIMERS_CODE_ FALSE


static DWORD WINAPI tgtDeltaTime (unsigned __int64 *pbase)
{
	DWORD dwTime = timeGetTime();
    DWORD dwDelta = dwTime - (DWORD)(*pbase);
    *pbase = (unsigned __int64)dwTime;
    return dwDelta;
}


static DWORD WINAPI tgtDiffTime (unsigned __int64 *pbase)
{
	return timeGetTime() - (DWORD)(*pbase);
}


struct _pctimer_global pc = {1000,1,0,
							 (LPVOID)tgtDiffTime,
							 (LPVOID)tgtDiffTime,
							 (LPVOID)tgtDiffTime,
							 (LPVOID)tgtDeltaTime,
							 (LPVOID)tgtDeltaTime,
							 (LPVOID)tgtDeltaTime,
                            };


static VOID WINAPI qpcInitTimer (unsigned __int64 *pbase)
{
	QueryPerformanceCounter ((LARGE_INTEGER*)pbase);
}


static DWORD WINAPI qpcDiffTicks (unsigned __int64 *pbase)
{
    unsigned __int64 ticks;
    DWORD diffticks;

    QueryPerformanceCounter ((LARGE_INTEGER*)&ticks);
    diffticks = (DWORD)(ticks - *pbase);
    
	return diffticks;
}


static DWORD WINAPI qpcDiffMicrosec (unsigned __int64 *pbase)
{
    unsigned __int64 ticks;
	DWORD diffus;

	QueryPerformanceCounter((LARGE_INTEGER*)&ticks);
	diffus = (DWORD)((1000000*(ticks-*pbase))/pc.dwRawHz);
    
	return diffus;
}


static DWORD WINAPI qpcDiffMillisec (unsigned __int64 *pbase)
{
    unsigned __int64 ticks;
	DWORD diffms;

	QueryPerformanceCounter((LARGE_INTEGER*)&ticks);
	diffms = (DWORD)((1000*(ticks-*pbase))/pc.dwRawHz);
    
	return diffms;
}


static DWORD WINAPI qpcDeltaTicks (unsigned __int64 *pbase)
{
    unsigned __int64 ticks = *pbase;
	DWORD deltaticks;
	                                                   
   	QueryPerformanceCounter ((LARGE_INTEGER*)pbase);                  
   	deltaticks = (DWORD)(*pbase - ticks);

   	return deltaticks;
}


static DWORD WINAPI qpcDeltaMicrosec (unsigned __int64 *pbase)
{
    unsigned __int64 ticks = *pbase;
	DWORD deltaus;
	                                                   
   	QueryPerformanceCounter ((LARGE_INTEGER*)pbase);                  

   	deltaus = (DWORD)(1000000*(*pbase-ticks)/pc.dwRawHz);
   	return deltaus;
}


static DWORD WINAPI qpcDeltaMillisec (unsigned __int64 *pbase)
{
    unsigned __int64 ticks = *pbase;
	DWORD deltams;
	                                                   
   	QueryPerformanceCounter ((LARGE_INTEGER*)pbase);                  
   	deltams = (DWORD)(1000*(*pbase-ticks)/pc.dwRawHz);

	return deltams;
}


static DWORD WINAPI qpcTimerFreq ()
{
    unsigned __int64 freq;

   	if (QueryPerformanceFrequency ((LARGE_INTEGER*)&freq)) 
	{
		return (DWORD)freq;               
	}

   	return 0;                             
}



VOID WINAPI InitPerformanceCounters (void)
{
    DWORD dwFreq;

    if (dwFreq = qpcTimerFreq())
    {
        pc.dwRawHz		 = dwFreq;
        pc.dwTimerKhz    = 1000;

        pc.DifTicks      = qpcDiffTicks;
        pc.DifMicrosec   = qpcDiffMicrosec;
        pc.DifMillisec   = qpcDiffMillisec;
        pc.DeltaTicks    = qpcDeltaTicks;
        pc.DeltaMicrosec = qpcDeltaMicrosec;
        pc.DeltaMillisec = qpcDeltaMillisec;
    } 
}

#endif // _INC_MMTIMERS_CODE_

