/*
 * vidtool.h
 *
 * 16/32-bit Video Capture driver
 *
 * hardware-specific data structures for Video Tool board.
 *
 * This header describes the structures that are shared between
 * the video tool-specific user-mode code and the video tool-specific
 * kernel mode code.
 *
 * The vcuser library does not define the format of the
 * config structures passed to the config functions beyond the fact that
 * the first ULONG contains the length of the entire structure. Note that
 * any pointers within this struct will need special handling.
 *
 * Mark Vrabel, March 96
 */

#ifndef _VIDTOOL_H_
#define _VIDTOOL_H_
 
/*
 * the FOURCC code for the one and only format we support
 */
#ifndef FOURCC_BW10
#define FOURCC_BW10        mmioFOURCC('B', 'W', '1', '0')
#endif


/*
 * The mode defines whether a dialog to be displayed, the saved
 * settings fetched from the registry, or the current settings
 * modified but not written to the registry.
 * 
 */ 
#define MODE_MODIFY_CURRENT			0      
#define MODE_FETCH_SETTINGS			1      
#define MODE_DISPLAY_DIALOG			2
#define MODE_MODIFY_PLAYBACKTARGET	3

#pragma pack(1) 
                        

/* 
 * this structure is passed to the DriverProperties function. 
 */
typedef struct _BW_PROPERTYOPTIONS 
{
    unsigned long ulStructSize;

    TCHAR szFileVersion[256];
    TCHAR szFileDescription[256];
    TCHAR szHardwareRevision[256];
    TCHAR szSerialNumber[256];

}  BW_PROPERTYOPTIONS;


/* 
 * this structure is passed to the ConfigFormat ConfigSource and 
 * ConfigFormat functions. 
 */
typedef struct _BW_CAPTURE_CONFIG
{
    unsigned long ulStructSize;
    LPVC_PROFILE_INFO lpProf;

    long nMode;
    long nIsBroadway;
    long nWndParent;

    long nHeight;
    long nWidth;
	long nSizeImage;

} BW_CAPTURE_CONFIG;

#pragma pack()

#endif //_VIDTOOL_H_




