/*
 * vcdriver.h
 *
 *
 * 32-bit Video Capture driver
 *
 * definition of interface between hardware-specific portions of
 * video driver and the helper library vckernel.lib
 *
 *
 * All h/w specific functions are given a pointer to a VCUSER_HANDLE 
 * structure which they can pass to VC_ helper functions. VC_GetHWInfo() 
 * will return a pointer to the hardware-specific data structure requested 
 * from VC_Init().
 *
 * 
 * Mark Vrabel, October 95:
 */
                                           
#ifndef _VCDRIVER_
#define _VCDRIVER_

#ifdef __cplusplus
extern "C" {
#endif


/* include necessary headers so that hardware-specific callers do not
 * explicitly reference NT-specific headers.
 */
#include <windows.h>
#include <windowsx.h>
#include <windows.h>
#include <mmsystem.h>
#include <vfw.h>
//#include <msvideo.h>
#include <string.h>

#ifdef _WIN32

    typedef PCHAR   LPCHAR;
    typedef PUCHAR  LPUCHAR;

    typedef PBYTE   LPBYTE;

    typedef PSHORT  LPSHORT;
    typedef PUSHORT LPUSHORT;

    typedef PLONG   LPLONG;
    typedef PULONG  LPULONG;

    typedef PDWORD  LPDWORD;

#else
    typedef char            CHAR, _huge *LPCHAR;
    typedef unsigned char   UCHAR, _huge *LPUCHAR;

    typedef CHAR            TCHAR;   
    typedef LPSTR           LPTSTR;   
    
    typedef short           SHORT, _huge *LPSHORT;
    typedef unsigned short  USHORT, _huge *LPUSHORT;

    typedef unsigned long   ULONG, _huge *LPULONG;   
    
    #define TEXT(a)   (a)

#endif


#include <vcstruct.h>
#include <vcuser.h>
#include <vcerrors.h>


#ifdef _GLOBAL_INIT_
    HINSTANCE ghDLLInst;
#else
    extern HINSTANCE ghDLLInst;
#endif


#ifndef DriverCallback
    BOOL WINAPI DriverCallback(DWORD dwCallback, UINT uFlags,
        HANDLE hDevice, UINT uMessage, DWORD dwUser, DWORD dwParam1,
        DWORD dwParam2);
#endif

/*
 * The following functions MUST be provided by the hardware
 * specific code!!!
 */

/* device installtion / properties dialog functions */	
LRESULT DriverInstallDialog (HWND hWnd, LPCONFIG_INFO lpGeneric);
LRESULT DriverPropertiesDialog (HWND hWnd, LPCONFIG_INFO lpGeneric);

/* main driver entry point for hardware specific code */
VCUSER_HANDLE DeviceEntry(LPTSTR lpDriverName, DWORD DeviceIndex);


/* --- callbacks to h/w specific code --------------------------*/


/* these are the hardware-specific functions called from the dispatcher */

typedef struct _VC_CALLBACK {

   /* return TRUE for success */

   /* called on device open/close - optional routines */
    BOOL  (FAR *DeviceOpenFunc)(VCUSER_HANDLE);
    BOOL  (FAR *DeviceCloseFunc)(VCUSER_HANDLE);

    BOOL  (FAR *ConfigFormatFunc)(VCUSER_HANDLE, LPCONFIG_INFO);
    BOOL  (FAR *ConfigDisplayFunc)(VCUSER_HANDLE, LPCONFIG_INFO);
    BOOL  (FAR *ConfigSourceFunc)(VCUSER_HANDLE, LPCONFIG_INFO);

	/* overlay functions - all optional */

   /* return overlay mode DWORD */
    DWORD (FAR *GetOverlayModeFunc) (VCUSER_HANDLE);

   /* return TRUE for success */
    BOOL  (FAR *SetKeyRGBFunc)(VCUSER_HANDLE, LPRGBQUAD);
    BOOL  (FAR *SetKeyPalIdxFunc)(VCUSER_HANDLE, ULONG);
    BOOL  (FAR *SetOverlayWindowFunc)(VCUSER_HANDLE, HWND);
    BOOL  (FAR *SetOverlayRectsFunc)(VCUSER_HANDLE, LPOVERLAY_RECTS);
    BOOL  (FAR *SetOverlayOffsetFunc)(VCUSER_HANDLE, LPRECT);
    ULONG (FAR *GetKeyColourFunc)(VCUSER_HANDLE);

    BOOL  (FAR *CaptureFunc)(VCUSER_HANDLE, BOOL);

    BOOL  (FAR *OverlayFunc)(VCUSER_HANDLE, BOOL);

    /* capture single frame routine */
    BOOL  (FAR *FrameFunc)(VCUSER_HANDLE, LPVIDEOHDR);

   /* capture routines  - start/stop, interrupt and capture are
    * required routines
    */
    BOOL  (FAR *StreamInitFunc)(VCUSER_HANDLE, ULONG);
    BOOL  (FAR *StreamFiniFunc)(VCUSER_HANDLE);
    BOOL  (FAR *StreamStartFunc)(VCUSER_HANDLE);
    BOOL  (FAR *StreamStopFunc)(VCUSER_HANDLE);
    ULONG (FAR *StreamGetPositionFunc)(VCUSER_HANDLE);
	VOID  (FAR *StreamRemoveReadyHeadFunc)(VCUSER_HANDLE, LPVIDEOHDR);
	VOID  (FAR *StreamAddReadyTailFunc)(VCUSER_HANDLE, LPVIDEOHDR);
	VOID  (FAR *StreamReplaceReadyHeadFunc)(VCUSER_HANDLE, LPVIDEOHDR);
	VOID  (FAR *StreamReportSkipFunc)(VCUSER_HANDLE, DWORD);

	/* frame repeat function - if driver supports synch adjustment */
	ULONG (FAR *RepeatFrame)(VCUSER_HANDLE, LPUCHAR, ULONG, LPUCHAR, ULONG, LPULONG);

    /* returns TRUE if Capture Service should happen */
    BOOL  (FAR *InterruptAcknowledge)(VCUSER_HANDLE, BOOL FAR *, BOOL FAR *);

   /*
    * returns bytes written to buffer PUCHAR (whose length is ULONG)
    * - not called if no buffer
    */
    ULONG (FAR *CaptureService)(VCUSER_HANDLE, LPUCHAR, LPULONG, ULONG, BOOL FAR *);

    /*
     * writes a frame to the device for display
     */
    BOOL  (FAR *DrawFrameFunc)(VCUSER_HANDLE, LPDRAWBUFFER);

    /* called on driver-unload */
    BOOL  (FAR *CleanupFunc)(VCUSER_HANDLE);


} VC_CALLBACK, * PVC_CALLBACK, FAR* LPVC_CALLBACK;


/* --- support functions in vckernel.lib -------------------------- */


/*
 * VC_Init
 *
 * Create the device object, and any necessary related setup, and
 * allocate device extension data. The device extension data is
 * a VCUSER_HANDLE struct plus however much data the caller wants for
 * hardware-specific data.
 *
 * returns pointer to device extension data as VCUSER_HANDLE struct.
 */
VCUSER_HANDLE VC_Init(ULONG HWInfoSize);


/* call to unload or abort load of the driver */
VOID VC_Cleanup(VCUSER_HANDLE);

/*
 * get the hardware specific portion of the device extension
 */
LPVOID VC_GetHWInfo(VCUSER_HANDLE);

/* get a poiter to the device's callback table */
LPVC_CALLBACK VC_GetCallbackTable(VCUSER_HANDLE);

/*
 * report the expected image size. attempts to queue buffers with a size
 * smaller than this will be rejected by vckernel.
 */
VOID VC_SetImageSize(VCUSER_HANDLE, ULONG);

/*
 * delay for a number of milliseconds. This is accurate only to
 * +- 15msecs at best.
 */
VOID VC_Delay(UINT nMillisecs);

/* block for given number of microseconds by polling. Not recommended
 * for more than 25 usecs.
 */
VOID VC_Stall(int nMicrosecs);

/* 
 * Interrupt service 
 */
BOOL VC_ServiceInterrupt (VCUSER_HANDLE vh); 

/* 
 * Buffer Manipulation
 */
VOID VC_ReplaceReadyHead(VCUSER_HANDLE vh, LPVIDEOHDR lpvh);
VOID VC_AddReadyTail(VCUSER_HANDLE vh, LPVIDEOHDR lpvh);
LPVIDEOHDR VC_RemoveReadyHead(VCUSER_HANDLE vh);
VOID VC_ResetReadyQueue(VCUSER_HANDLE vh);

VOID VC_ReplaceDoneHead(VCUSER_HANDLE vh, LPVIDEOHDR lpvh);
VOID VC_AddDoneTail(VCUSER_HANDLE vh, LPVIDEOHDR lpvh);
LPVIDEOHDR VC_RemoveDoneHead(VCUSER_HANDLE vh);
VOID VC_ResetDoneQueue(VCUSER_HANDLE vh);

/* 
 * Start and stop high resolution timer
 */
BOOL VC_StartEventTimer (VCUSER_HANDLE vh, UINT Delay);
VOID VC_StopEventTimer (VCUSER_HANDLE vh);
void VC_TimerDisable (VCUSER_HANDLE vh);
void VC_TimerEnable (VCUSER_HANDLE vh);
                                                 

#ifdef __cplusplus
}
#endif


#endif // _VCDRIVER_
