/* VT_REGS.H */

#ifndef __VT_REGS
#define __VT_REGS

//#define DWORD unsigned long

//Status bits
#define MASTER_XFER_CNT_2		0x0003ffff
#define VRP_INT_EN				0x00000080
#define B_DONE_INT_EN			0x00000040
#define A_DONE_INT_EN			0x00000020
#define VSYNC_INT_EN			0x00000010
#define VRP_INT_FLAG			0x08000000
#define DONE_FLAG_2				0x04000000
#define DONE_FLAG_1				0x02000000
#define VSYNC_INT_FLAG			0x01000000
#define VSYNC_FLAG				0x01000000
#define FIFO_OVERFLOW			0x00800000

// not actual hw interupts
#define VRP_INT					0x00000008
#define B_DONE_INT				0x00000004
#define A_DONE_INT				0x00000002
#define VSYNC_INT				0x00000001


//  CSR bits
#define SOFT_RESET	(1u<<31)
#define LAST_EVEN_PIXEL	(1u<<30)
#define FIRST_EVEN_PIXEL (1u<<29)
#define FIFO_FULLN	(1u<<28)
#define FIFO_EMPTYN	(1u<<27)
#define FIFO_AFN	(1u<<26)
#define FIFO_AEN	(1u<<25)
#define PORTA_FLAG	(1u<<24)
#define VRP_LOCAL	(1u<<23)
#define COUNT_2S	(1u<<22)
#define COUNT_1S	(1u<<21)
#define COUNT_2C	(1u<<20)
#define COUNT_1C	(1u<<19)
#define STOP_2DONE_OUT	(1U<<13)
#define VRP_BUS_ERROR	(1u<<12)
#define SDI			(1u<<11)
#define SDO			(1u<<10)
#define SCLK		(1u<<9)
#define IREG_OP_BUSY		(1u<<8)
#define IREG_WR		(1u<<7)
#define VRP_SEL		(1u<<6)
#define STOP_1DONE_IN (1u<<5)
#define DISABLE_LARGEBURST (1u<<3)
#define MSTR_MODE_MASK	0x7u
#define MSTR_STOP	0u
#define MSTR_INPUT  1u  // in and out may be or'd together
#define MSTR_OUTPUT	2u
#define MSTR_TOGGLE 4u  // for passthrough only


struct _VT_REGS{ 
	DWORD Csr	;
	DWORD Status;
	DWORD IRegData;
	DWORD IRegAddr;
	DWORD MasterStart1;
	DWORD MasterSize1;
	DWORD RESERVED1;
	DWORD RESERVED2;
	DWORD MasterStart2;
	DWORD MasterSize2;
	DWORD RESERVED3;
	DWORD RESERVED4;
	DWORD FifoData;
	DWORD PortAIn;
    };

typedef struct _VT_REGS  VT_REGS;

typedef struct {
	DWORD Ctrl1;
	DWORD Ctrl2;
	DWORD FifoData;
	}  CONT_REGS;


#define CTRL1	0
#define CTRL2	1
#define FIFO_DATA 2

// CTRL1 BITS
#define VRP_RESET	0X80
#define SNGL_FRAME	0x40
#define VIP_RESET	0X20
#define VIP_OE		0X10
#define LAST_PIXEL	0X8
#define FIRST_PIXEL	0X4
#define VSYNC_IN	0X2
#define VIDEO_EN	0X1


// CTRL2 BITS
#define FIFO_RESET	1
#define FIFO_WR	2
#define FIFO_DATAH_SEL	4
#define FIFO_LD	0X40
#define FIFO_MODE_MASK	0x38
#define MAKE_FIFO_MODE(_x) (_x<<3)
#define FIFO_MODE_CAPTURE	(0<<3)
#define FIFO_MODE_PASSTHROUGH	(1<<3)
#define FIFO_MODE_ICAPTURE	(2<<3)
#define FIFO_MODE_RECODE	(3<<3)
#define FIFO_MODE_PROGRAM	(4<<3)
#define FIFO_MODE_WRITE		(5<<3)
#define FIFO_MODE_IONLY		(6<<3)


 struct _VIP_REGS{
	DWORD VipVer;
	DWORD Reserved1;
	DWORD VipInputCtrl1;
	DWORD VipInputCtrl2;
	DWORD VipGainCtrl1;
	DWORD VipGainCtrl2;
	DWORD VipHsb;
	DWORD VipHss;
	DWORD VipSyncCtrl;
	DWORD VipLumaCtrl;
	DWORD VipBrightCtrl;
	DWORD VipContCtrl;
	DWORD VipSatCtrl;
	long  VipHueCtrl;
	DWORD VipChromaCtrl;
	DWORD Reserved2;
	DWORD VipFmtDlyCtrl;
	DWORD VipOutCtrl1;
	DWORD VipOutCtrl2;
};

typedef struct _VIP_REGS VIP_REGS;



#define	VIP_VER			0x00	/*VIP Version reg.*/
#define	VIP_INPUT_CTRL1	0x02	//VIP Input Control register 1
#define	VIP_INPUT_CTRL2	0x03	//VIP Input Control register 2
#define	VIP_GAIN_CTRL1	0x04	//VIP Gain Control register 1
#define	VIP_GAIN_CTRL2	0x05	//VIP Gain Control register 2
#define	VIP_HSB			0x06	//VIP Horizontal Sync Begin reg.
#define	VIP_HSS			0x07	//VIP Horizontal Sync Stop reg.
#define	VIP_SYNC_CTRL	0x08	//VIP Sync Control reg.
#define	VIP_LUMA_CTRL	0x09	//VIP Luminance Control reg.
#define	VIP_BRIGHT_CTRL	0x0A	//VIP Luminance Brightness Control reg.
#define	VIP_CONT_CTRL	0x0B	//VIP Luminance Contrast Control reg.
#define	VIP_SAT_CTRL	0x0C	//VIP Chrominance Saturation Control reg.
#define	VIP_HUE_CTRL	0x0D	//VIP Chrominance Hue Control reg.
#define	VIP_CHROMA_CTRL	0x0E	//VIP Chrominance Control reg.
#define	VIP_FMT_DLY_CTRL	0x10	//VIP Format/Delay Control reg.
#define	VIP_OUT_CTRL1	0x11	//VIP Output Control reg.
#define	VIP_OUT_CTRL2	0x12	//VIP Output Control reg.
#define VIP_TEXT_STATUS 0x1A
#define VIP_TEXT1		0x1B
#define VIP_TEXT2		0x1C
#define VIP_STATUS		0x1F    //VIP Status register



typedef volatile VT_REGS  *PREGS;

struct _DEVSTRUCT{
	volatile VT_REGS *Regs;
	const char* szTablePath;
	VIP_REGS VipRegs; 
};
typedef struct _DEVSTRUCT DEVSTRUCT;
#if 0
#ifdef __cplusplus
class CDevice;
typedef CDevice *DEVID;
#else
typedef DEVSTRUCT *DEVID;
#endif
#endif
#define GET_STRUCT_PTR(_x) (_x->Regs)

//#define SET_CSR_BITS(_x) 	bd_addr->Csr|=_x
//#define CLEAR_CSR_BITS(_x) bd_addr->Csr&=~_x
#define SET_CSR_BITS(_x) 	Change32(&bd_addr->Csr,_x,_x)
#define CLEAR_CSR_BITS(_x) Change32(&bd_addr->Csr,_x,0)

#ifdef __cplusplus
extern "C" {
#endif

//DWORD Change32(volatile DWORD *addr,DWORD mask,DWORD data); // can't be inline unless asm

__inline void W32(volatile DWORD *addr,DWORD data)
{	*addr=data;}

__inline void Change32(volatile DWORD *addr,DWORD mask,DWORD value)
{
		W32(addr,(*addr & ~mask) | (value & mask));
}

#ifdef __cplusplus
}
#endif

#endif
