/*
 * vtdioc.h
 *
 * 32-bit Video Tool driver
 * Win95 kernel-mode helper library private definitions
 *
 * Mark Vrabel, October 95
 */

#ifndef _VTDIOC_H_
#define _VTDIOC_H_

/* Device Naming:
 *
 * a device name will be created in \Device and a symbolic link
 * in \DosDevices: the name will be the string DD_VIDCAP_DEVICE_NAME_U
 * followed by a number 0, 1.. to ensure uniqueness. There is no guarantee
 * that names will have consecutive numbers: if device 1 is deleted out of 3,
 * it would leave two names, \Device\VidCap0 and \Device\VidCap2
 *
 * In addition, each device records the name used (eg VidCap0) in the
 * REG_DEVNAME value of its PARMS_SUBKEY entry in the registry.
 */
#define DD_VIDCAP_DEVICE_NAME_U     TEXT("VidCap")

/*
 * name of subkey in which device parameters are stored
 */
#define PARMS_SUBKEY				TEXT("Parameters")

/*
 * value name within PARMS_SUBKEY where the device name created is stored.
 * This is the DD_VIDCAP_DEVICE_NAME_U string together with the
 * unique number.
 */
#define REG_DEVNAME					TEXT("DeviceName")

/*
 * value name within PARMS_SUBKEY where the kernel mode initialization i
 * status is stored.
 */
#define REG_STATUS					TEXT("KernelStatus")

/* 
 * maximum number of vidcap devices we allow
 */
#define	MAX_VIDCAP_DEVICES	    32

/*
 * maximum length (in characters) of the device name including \device and the
 * number (or \dosdevices and the number) and a null
 */
#define MAX_VIDCAP_NAME_LENGTH	    12+6+3+1

/*
 * device type - we are a multimedia device, so for now we will be a sound device
 */
#define DD_TYPE_VIDC	FILE_DEVICE_SOUND


/* 
 * base for IOCTL requests - just above the sound requests
 */
#define IOCTL_VIDC_BASE	0x200


/* Ioctl set */
#define IOCTL_VIDTOOL_GET_DEVICEINFO     CTL_CODE(DD_TYPE_VIDC, IOCTL_VIDC_BASE+0x1, METHOD_BUFFERED, FILE_ANY_ACCESS)
#define IOCTL_VIDTOOL_RELEASE_DEVICEINFO CTL_CODE(DD_TYPE_VIDC, IOCTL_VIDC_BASE+0x2, METHOD_BUFFERED, FILE_ANY_ACCESS)
#define IOCTL_VIDTOOL_GET_DEVICE_IDS     CTL_CODE(DD_TYPE_VIDC, IOCTL_VIDC_BASE+0x3, METHOD_BUFFERED, FILE_ANY_ACCESS)
#define IOCTL_VIDTOOL_FLUSHCACHE         CTL_CODE(DD_TYPE_VIDC, IOCTL_VIDC_BASE+0x4, METHOD_BUFFERED, FILE_ANY_ACCESS)   
/* 
 * define the device node used in Win95
 */
#ifndef DEVNODE 
    typedef ULONG DEVNODE;
#endif

/*
 * define device states
 */
typedef enum 
{   
    IDLE = 0,
    INITIALIZED,
    CAPTURE,
    CODEC_INITIALIZED,
    CODEC_BUSY,
    RECODE 
}  
BOARD_STATE;


#pragma pack(1) 

/* 
 * shared data structure allocated in kernel mode and
 * accessable by all processes.
 */
typedef struct _USER_INFO
{
    DWORD dwStructSize;				// important!!!
	volatile BOARD_STATE State;		// hardware state
	DWORD dwSizeReserved;			// size of the reserved data space
	char  cbReserved[1];			// used by vtaccess for shared data

} USER_INFO, *PUSER_INFO, FAR *LPUSER_INFO;


/*
 * device information structure
 */
typedef struct _DEVICE_INFO {

    DWORD dwStructSize;				// important!!!
    UCHAR RevisionID;				// hardware revision

	DWORD pPhysAddr;				// physical address of device memory
    DWORD dwMemoryBytes;			// Size of device memory in bytes  
    
    void *pRegs;					// 32-bit linear address of device's memory                                 
    char *pLinearAddr;				// 32-bit linear address of device memory
	PUSER_INFO pUserInfo;           // 32-bit linear address of shared user memory
		
    char FAR *lpLinearAddr16;		// (obsolete) segment/offset linear address of device memory
    void FAR *lpRegs16;				// (obsolete) segment/offset address of device's memory                                 

} DEVICE_INFO, *PDEVICE_INFO, FAR *LPDEVICE_INFO;


/*
 * device ID structure
 */
#define MAX_DEVNODES 64

typedef struct _DEVICE_IDS {
    DWORD dwNumDevNodes;
    DEVNODE DevNodes[MAX_DEVNODES];
} DEVICE_IDS, *PDEVICE_IDS, FAR *LPDEVICE_IDS;


#pragma pack()

#endif  //_VTDIOC_H_
