/*
 * vtoolipc.h
 *
 *
 * Provides stucture and constant declarations for
 * Video Tool InterProcess Comunication (IPC).  Used
 * to allow threads to be created within the context
 * of a 16-bit application. 
 *
 * Mark Vrabel, October 95
 */

#define MAX_ARGUMENT_SIZE   1024
#define MAX_STRING_SIZE     256

/* define the IPC control codes */
typedef enum ipc_controlcode_tag
{
    IPC_TERMINATEPROCESS = 0,
    IPC_OPENDEVICE,
    IPC_CLOSEDEVICE,
    IPC_CONFIGFORMAT,
    IPC_CONFIGSOURCE,
    IPC_CONFIGDISPLAY,
    IPC_GETOVERLAYMODE,
    IPC_SETKEYCOLOURRGB,
    IPC_SETKEYCOLOURPALIDX,
    IPC_GETKEYCOLOUR,
    IPC_SETOVERLAYWINDOW,
    IPC_SETOVERLAYRECT,
    IPC_SETOVERLAYOFFSET,
    IPC_OVERLAY,
    IPC_CAPTURE,
    IPC_DRAWFRAME,
    IPC_FRAME,
    IPC_STREAMINIT,
    IPC_STREAMFINI,
    IPC_STREAMSTART,
    IPC_STREAMSTOP,
    IPC_STREAMRESET,
    IPC_GETSTREAMERROR,
    IPC_STREAMADDBUFFER,
    IPC_GETSTREAMPOS,

    IPC_OPENPROFILEACCESS,
    IPC_CLOSEPROFILEACCESS,
    IPC_INSTALLDRIVER,
    IPC_WRITEPROFILE,
    IPC_READPROFILE,
    IPC_READPROFILEUSER,
    IPC_WRITEPROFILEUSER,
    IPC_READPROFILESTRING,
    IPC_REMOVEDRIVER,
    IPC_CONFIGACCESS,
    IPC_DRIVERINSTALLDIALOG,
    IPC_DRIVERPROPERTIESDIALOG,

} IPC_CONTROLCODE; 


/* 
 * Stuctures used in IPC message packet 
 * structure are typedef'ed here rather 
 * than embedded in the structure so
 * the debugger can veiw the contents 
 */

/* agument structure for VC_OpenDevice */
typedef struct _OPEN_DEVICE
{      
    char DriverName[MAX_STRING_SIZE+1];
    DWORD dnDevNode;
    int DeviceIndex;
} OPEN_DEVICE;


/* agument structure for VC_StreamInit */
typedef struct _STREAM_INIT
{       
    LPVCCALLBACK Callback;	
    ULONG ulFrameRate;
} STREAM_INIT;

/* agument structure for VC_OpenProfule 
 * 
 * The size of DriverName defines the size of the input buffer. 
 * This assures that there is plenty of room available for the 
 * structures of size unknown, such as CONFIG_INFO or OVERLAY_RECTS.
 * Error checking should be done to validate that the size of
 * such structures does not exceed MAX_ARGUMENT_SIZE and the length
 * of strings does not exceed MAX_STRING_SIZE+1. 
 */
typedef struct _OPEN_PROFILE
{      
    char DriverName[MAX_ARGUMENT_SIZE];
    DWORD dnDevNode;
} OPEN_PROFILE;

/* agument structure for VC_InstallDriver */
typedef struct _INSTALL_DRIVER
{       
    LPPROFILE_CALLBACK lpCallback;
    LPVOID lpContext;
} INSTALL_DRIVER;

/* agument structure for VC_WriteProfile and VC_WriteProfileUser */
typedef struct _WRITE_PROFILE
{   
    char SectionName[MAX_STRING_SIZE+1];
    char ValueName[MAX_STRING_SIZE+1];
    DWORD Value;
} WRITE_PROFILE;

/* agument structure for VC_ReadProfile and VC_ReadProfileUser */
typedef struct _READ_PROFILE 
{       
    char SectionName[MAX_STRING_SIZE+1];
    char ValueName[MAX_STRING_SIZE+1];
    DWORD dwDefault;
} READ_PROFILE;

/* agument structure for VC_ReadProfileString */
typedef struct _READ_PROFILE_STRING
{       
    char ValueName[MAX_STRING_SIZE+1];
    char ValueString[MAX_STRING_SIZE+1];
    DWORD ValueLength;
} READ_PROFILE_STRING;


/* 
 * argument structure for  VC_DriverInstallDialog and 
 * VC_DriverProperties Dialog
 */
typedef struct _DRIVER_DIALOG
{       
    CONFIG_INFO *lpGeneric;
    HWND hWnd;

} DRIVER_DIALOG;


/* IPC message packet structure */
typedef struct 
{
    IPC_CONTROLCODE IpcControlCode;   /* Input - IPC control code       */
    VCUSER_HANDLE vh;                 /* I/O - vcuser handle            */
    LPVC_PROFILE_INFO lpProf;         /* I/O - vc_profile info          */
    LRESULT lResult;                  /* Output - function return value */

    union 
    {        
        /* 
         * Input arguments for functions passing 
         * single arguments other than VCUSER_HANDLE 
         * or LPVC_PROFILE_INFO.
         */
        CONFIG_INFO *lpGeneric;
        RGBQUAD *lprgbQuad;
        WORD PalIndex;
        HWND hOverlayWnd;
        OVERLAY_RECTS *lpOverlayRect;
        RECT *lprect;
        BOOL bOverlay;
        BOOL bAcquire;
        DRAWBUFFER *lpDrawBuf;
        LPVIDEOHDR lpVideoHdr;
        MMTIME *lpTime;

        /* 
         * arguments for functions passing multipe arguments 
         * packed in structures for readability and ease of 
         * use
         */
        OPEN_DEVICE OpenDevice;
        STREAM_INIT StreamInit;
        OPEN_PROFILE OpenProfile;
        INSTALL_DRIVER InstallDriver;
        WRITE_PROFILE WriteProfile;
        READ_PROFILE ReadProfile;
        READ_PROFILE_STRING ReadProfileString;
		DRIVER_DIALOG DriverDialog;
   };
} IPC_PACKET;                 

 

