/*
 * vtkernel.h
 *
 *
 * 32-bit Video Capture driver
 * kernel-mode support library 
 *
 * Mark Vrabel, June 96
 */

#ifndef _VTKERNEL_
#define _VTKERNEL_

/* include necessary headers so that hardware-specific callers do not
 * explicitly reference NT-specific headers.
 */
#include <ntddk.h>
#include <windef.h>
#include <vcerrors.h>

#ifdef __TEXT
#undef __TEXT
#endif

#define __TEXT(quote) L##quote

#include <vtdioc.h>

#define SIZE_PAGE		(4096)
#define MAX_PCI_BUSES	(255)
#define MAX_PCI_SLOTS	(255)
#define BUS_NOTFOUND	(0)
#define MAX_NAME_SIZE   (256*sizeof(WCHAR))
#define MAX_INSTANCES   (256)


typedef struct _INSTANCE_INFO
{
	PEPROCESS pProcess;             // process that owns this mapping
	ULONG UsageCount;               // process usage count

    PVOID pRegs;					// 32-bit linear address of device's memory                                 
    PVOID pLinearAddr;				// used by vtaccess for shared data
	PUSER_INFO pUserInfo;			// 32-bit linear address of shared user memory

    PMDL pMdlLinearAddr;			// 32-bit linear address of device memory
    PMDL pMdlRegs;					// 32-bit linear address of device's memory                                 
    PMDL pMdlInfo;				// used by vtaccess for shared data

} INSTANCE_INFO, *PINSTANCE_INFO;


/*
 * definition of _DEVICE_INFO structure. This is declared in vckernel.h
 * so that the hw/specific code can use it as an opaque pointer.
 */
typedef struct _DEVICE_EXT 
{

    PDEVICE_OBJECT pDeviceObject;	// parent device object
    int DeviceNumber;				// index to this device

    ULONG BusType;					// bus type (isa, eisa, pci, ect)
    ULONG BusNumber;				// bus number
    PCI_SLOT_NUMBER SlotNumber;		// bus slot number
								
    char *pLinearAddr;				// 32-bit linear address of device memory
	PHYSICAL_ADDRESS pPhysAddr;		// physical address of device frame buffer memory
    DWORD dwMemoryBytes;			// Size of device frame buffer memory in bytes  

    ULONG PortMemType;				// memory mapped or I/O
    void *pRegs;					// 32-bit linear address of device's memory mapped registers                                 
    ULONG NrOfPorts;				// size of device memory mapped registers
    
	PUSER_INFO pUserInfo;			// user info structure
	INSTANCE_INFO InstanceList[MAX_INSTANCES]; // user instance list

    PWCHAR ParametersKey;			// device parameter registry key
    UNICODE_STRING RegistryPath;	// saved registry path for cleanup

	ULONG DeviceInUse;              // device usage count
    UCHAR RevisionID;               // hardware revision

} DEVICE_EXT, *PDEVICE_EXT, FAR *LPDEVICE_EXT;



/* the dispatch routine to which all IRPs go */
NTSTATUS 
VT_Dispatch(
	IN PDEVICE_OBJECT pDeviceObject, 
	IN PIRP pIrp
	);

/* device initialization routine */
PDEVICE_EXT 
VT_Init(
    PDRIVER_OBJECT pDriverObject,
    PUNICODE_STRING RegistryPathName,
	USHORT DevId,
	USHORT VenId,
	ULONG FrameBufferSize
	);

/* write a value to the registry */
BOOL 
VT_WriteProfile(
	PDEVICE_EXT pDevExt, 
	PWCHAR ValueName, 
	DWORD ValueData);

/* Cleanup on driver unload or aborted load. */
VOID
VT_Cleanup(
	PDRIVER_OBJECT pDriverObject
	);

/* 
 * map device information into address space 
 * of current process 
 */ 
NTSTATUS 
VT_MapDevInfoToUser(
PDEVICE_EXT pDevExt
);


/* Unmap device info from current process */
VOID 
VT_UnMapDevInfo(
	PDEVICE_EXT pDevExt
	);


/* retrieve current process mapping of device info */
NTSTATUS 
VT_GetDevInfoForUser(
	PDEVICE_EXT pDevExt, 
	PDEVICE_INFO pDevInfo
	);

#endif // _VTKERNEL_
