/*
 * realtime.c
 *
 *
 * 32-bit Video Tool driver
 * realtime mpeg capture window subclass 
 * routines for Data Translation Video Tool.
 *
 * Mark Vrabel, October 96
 */

#include <vcdriver.h>
#include "hardware.h"
#include <bwdlgs.h>
#include <bwuser.h>
#include <rcuser.h>
#include <Tlhelp32.h>

#include "realtime.h"


/*
 * needed to bybass subclassed window proc
 */
extern FARPROC gfpOrigCapWndProc; 


/* declare local functions */
LRESULT RT_Capture (LPHWINFO lpHw);
DWORD RT_ThreadProc(DWORD arg);


LRESULT 
RT_CapFileGetCaptureFile (LPHWINFO lpHw, LPSTR lpszFileName, UINT nSize)
{
	HWND hWnd = lpHw->hWndCap;
	char szFile[MAX_PATH];
	char *ptr = szFile;
	int len;

	LRESULT lResult;

	/* get the file name to capture to */
	lResult = bcapFileGetCaptureFile(hWnd, szFile, sizeof(szFile));

	if (lResult) {

		/* make sure we have enough space for ext */
		szFile[sizeof(szFile)-5] = '\0';
		
		/* find extention */		
		len = lstrlen (ptr);
		ptr += len;
		while (*ptr != '.' && len != 0)	{
			ptr--;
			len--;
		}

		if (lResult == CAPMODE_RTMPEG) {
			if (len==0) {
				lstrcat (szFile, ".mpg");  			
			} else {
				wsprintf (ptr, ".mpg");  			
			}
		} else {
			if (len==0) {
				lstrcat (szFile, ".avi");  			
			} else {
				wsprintf (ptr, ".avi");  			
			}
		}

		lstrcpyn(lpszFileName, szFile, nSize-1);
	}

	return lResult;
}


LRESULT 
RT_CapFileSetCaptureFile (LPHWINFO lpHw, LPSTR lpszFileName)
{
	HWND hWnd = lpHw->hWndCap;
	char szFile[MAX_PATH];
	char *ptr = szFile;
	int len;

	LRESULT lResult;

	/* get the file name to capture to */
	lstrcpyn(szFile, lpszFileName, sizeof(szFile)-1);

	/* make sure we have enough space for ext */
	szFile[sizeof(szFile)-5] = '\0';
		
	/* find extention */		
	len = lstrlen (ptr);
	ptr += len;
	while (*ptr != '.' && len != 0)	{
		ptr--;
		len--;
	}

	if (lpHw->CaptureMode == CAPMODE_RTMPEG) {
		if (len==0) {
			lstrcat (szFile, ".mpg");  			
		} else {
			wsprintf (ptr, ".mpg");  			
		}
	} else {
		if (len==0) {
			lstrcat (szFile, ".avi");  			
		} else {
			wsprintf (ptr, ".avi");  			
		}
	}

	lResult = bcapFileSetCaptureFile (hWnd, szFile);

	return lResult;
}


LRESULT 
RT_CapDriverGetVersion (UINT wSize, LPSTR lpVersion) {

    UINT    VersionLen;
    BOOL   	bRetCode;
    DWORD 	dwVerInfoSize;
    DWORD 	dwVerHnd;
    TCHAR 	szBuf[MAX_PATH];
    
	GetSystemDirectory (szBuf, sizeof(szBuf));
	lstrcat (szBuf, "\\vtool.dll");

    /* You must find the file info size first before getting any file info */
    dwVerInfoSize = GetFileVersionInfoSize(szBuf, &dwVerHnd);

    if (dwVerInfoSize) {
		LPBYTE  lpstrVffInfo;             // Pointer to block to hold info

		/* Get a block big enough to hold version info */
		lpstrVffInfo = GlobalAllocPtr(GMEM_MOVEABLE, dwVerInfoSize);

		/* Get the File Version first */
		if(GetFileVersionInfo(szBuf, 0L, dwVerInfoSize, lpstrVffInfo)) {
			VS_FIXEDFILEINFO FAR* lpVerInfo;

		    /*
		     * get the VS_FIXEDFILEINFO in the root block
	    	 */
	    	bRetCode = VerQueryValue(
				lpstrVffInfo,				 // version resource
				TEXT("\\"),					 // fetch root VS_FIXEDFILEINFO	
				(void FAR* FAR*) &lpVerInfo, // pointer to it goes here
				&VersionLen);				 // length of it goes here

	    	if (bRetCode && (VersionLen > 0) && (lpVerInfo != NULL)) {
                 
				/* fill in the file version */
				wsprintf(lpVersion
					,TEXT("Version: %d.%d.%d.%d")
						 ,HIWORD(lpVerInfo->dwFileVersionMS)
						 ,LOWORD(lpVerInfo->dwFileVersionMS)
						 ,HIWORD(lpVerInfo->dwFileVersionLS)
						 ,LOWORD(lpVerInfo->dwFileVersionLS)
					 	);
    		}
		}

		/* Let go of the memory */
		GlobalFreePtr(lpstrVffInfo);
	}
	
	return TRUE;
}


LRESULT 
RT_CapSetCallbackOnError (LPHWINFO lpHw, FARPROC lpfnCallback)
{
	HWND hWnd = lpHw->hWndCap;
	LRESULT lResult;

	lResult = bcapSetCallbackOnError (hWnd, lpfnCallback);
	if (lResult) {
		lpHw->lpfnErrorCallback = lpfnCallback;
	}
	return lResult;
}


LRESULT 
RT_CapSetCallbackOnStatus (LPHWINFO lpHw, FARPROC lpfnCallback)
{
	HWND hWnd = lpHw->hWndCap;
	LRESULT lResult;

	lResult = bcapSetCallbackOnStatus (hWnd, lpfnCallback);
	if (lResult) {
		lpHw->lpfnStatusCallback = lpfnCallback;
	}
	return lResult;
}


LRESULT 
RT_CapSetCallbackOnYield (LPHWINFO lpHw, FARPROC lpfnCallback)
{
	HWND hWnd = lpHw->hWndCap;
	LRESULT lResult;

	lResult = bcapSetCallbackOnYield (hWnd, lpfnCallback);
	if (lResult) {
		lpHw->lpfnYieldCallback = lpfnCallback;
	}
	return lResult;
}


LRESULT 
RT_CapSetCallbackOnVideoStream (LPHWINFO lpHw, FARPROC lpfnCallback)
{
	HWND hWnd = lpHw->hWndCap;
	LRESULT lResult;

	lResult = bcapSetCallbackOnVideoStream (hWnd, lpfnCallback);
	if (lResult) {
		lpHw->lpfnVideoStreamCallback = lpfnCallback;
	}
	return lResult;
}


LRESULT 
RT_CapSetCallbackOnWaveStream (LPHWINFO lpHw, FARPROC lpfnCallback)
{
	HWND hWnd = lpHw->hWndCap;
	LRESULT lResult;

	lResult = bcapSetCallbackOnWaveStream (hWnd, lpfnCallback);
	if (lResult) {
		lpHw->lpfnWaveStreamCallback = lpfnCallback;
	}
	return lResult;
}


LRESULT 
RT_CapSetCallbackOnCapControl (LPHWINFO lpHw, FARPROC lpfnCallback)
{
	HWND hWnd = lpHw->hWndCap;
	LRESULT lResult;

	lResult = bcapSetCallbackOnCapControl (hWnd, lpfnCallback);
	if (lResult) {
		lpHw->lpfnCapControlCallback = lpfnCallback;
	}
	return lResult;
}


LRESULT 
RT_CapGetStatus (LPHWINFO lpHw, CAPSTATUS* lpCapStatus, UINT size)
{
	HWND hWnd = lpHw->hWndCap;
	LRESULT lResult;

	if (lpHw->CaptureMode == CAPMODE_RTMPEG) {
		if (size < sizeof(CAPSTATUS)) {
			return FALSE;
		}
	}

	lResult = bcapGetStatus (hWnd, lpCapStatus, size);

	if (lResult && lpHw->CaptureMode == CAPMODE_RTMPEG) {
		
	    lpCapStatus->dwCurrentVideoFrame = lpHw->dwCurrentVideoFrame; 
		lpCapStatus->dwCurrentVideoFramesDropped = lpHw->dwCurrentVideoFramesDropped; 
		lpCapStatus->dwCurrentWaveSamples = lpHw->dwCurrentWaveSamples; 
		lpCapStatus->dwCurrentTimeElapsedMS = lpHw->dwCurrentTimeElapsedMS; 
		lpCapStatus->fCapturingNow = lpHw->fCapturingNow; 
		lpCapStatus->wNumVideoAllocated = lpHw->wNumVideoAllocated; 
		lpCapStatus->wNumAudioAllocated = lpHw->wNumAudioAllocated; 

		lpCapStatus->uiImageWidth = lpHw->CaptureInfo.FrameWidth;
		lpCapStatus->uiImageHeight = lpHw->CaptureInfo.FrameHeight;

	}

	return lResult;
}


LRESULT 
RT_CapGetVideoFormat (LPHWINFO lpHw, LPBITMAPINFOHEADER lpbi, UINT size)
{
	HWND hWnd = lpHw->hWndCap;
	LRESULT lResult;

	lResult = bcapGetVideoFormat (hWnd, lpbi, size);

	if (lpHw->CaptureMode == CAPMODE_RTMPEG ) {

		if (size >= sizeof(BITMAPINFOHEADER) && lpbi != NULL) {
			lpbi->biWidth = lpHw->CaptureInfo.FrameWidth;
			lpbi->biHeight = lpHw->CaptureInfo.FrameHeight;
		}
	}

	return lResult;
}


LRESULT 
RT_CapPreview (LPHWINFO lpHw, BOOL bPreview)
{
	HWND hWnd = lpHw->hWndCap;
	LRESULT lResult;

	lResult = bcapPreview (hWnd, bPreview);
	if (lResult) {
		lpHw->bPreview = bPreview;
	}

	return lResult;
}


LRESULT 
RT_CapCaptureSequence (LPHWINFO lpHw)
{
	HWND hWnd = lpHw->hWndCap;
	LRESULT lResult;

	if (lpHw->CaptureMode == CAPMODE_RTMPEG) {
		lpHw->bCreateNoFile = FALSE;
		return RT_Capture (lpHw);
	} else {
		lResult = bcapCaptureSequence (hWnd);
		return lResult;
	}
}


BOOL
RT_IsStopping (LPHWINFO lpHw)
{
	return lpHw->bIsStopping;
}


BOOL
RT_IsCapturing (LPHWINFO lpHw)
{
	return lpHw->fCapturingNow;
}

	
LRESULT
RT_CapCaptureSequenceNoFile (LPHWINFO lpHw)
{
	HWND hWnd = lpHw->hWndCap;
	LRESULT lResult;

	if (lpHw->CaptureMode == CAPMODE_RTMPEG) {
		lpHw->bCreateNoFile = TRUE;
		return RT_Capture (lpHw);
	} else {
		lResult = bcapCaptureSequenceNoFile (hWnd);
		return lResult;
	}
}


LRESULT
RT_CapCaptureStop (LPHWINFO lpHw)
{
	HWND hWnd = lpHw->hWndCap;
	LRESULT lResult;

	if (lpHw->CaptureMode == CAPMODE_RTMPEG) {

		if (lpHw->hCaptureThread) {

			HANDLE hCaptureThread = lpHw->hCaptureThread;
			MSG Msg;

			/* signal the worker thread it's time to stop */
			SetEvent (lpHw->hCaptureComplete);

			/* if we're being called from a callback in the capture thread, 
			 * this will allow the calling thread to run.
			 */
			ReplyMessage(TRUE);
                  
			while (WaitForSingleObject(hCaptureThread, 0) != WAIT_OBJECT_0)  {
	
				/* 
				 * NOTE: we need to do a peek message on the capture window
				 * while waiting so the worker thread can comunicate with
				 * the capture window using the SendMessage interface. if
				 * we don't do the PeekMeesage thing - severe deadlock. 
				 */
				if (PeekMessage (&Msg, hWnd, 0, 0, PM_REMOVE)) {
					TranslateMessage (&Msg);
					DispatchMessage (&Msg);
				} 

				/* just let paint messages get through to the main window */
				if (PeekMessage (&Msg, NULL, WM_PAINT, WM_PAINT, PM_REMOVE)) {
					TranslateMessage (&Msg);
					DispatchMessage (&Msg);
				} 

 			}

			CloseHandle (hCaptureThread);
		} 

		return TRUE;
		
	} else {
		lResult = bcapCaptureStop (hWnd);
		return lResult;
	}
}


LRESULT
RT_CapCaptureAbort (LPHWINFO lpHw)
{
	HWND hWnd = lpHw->hWndCap;
	LRESULT lResult;

	if (lpHw->CaptureMode == CAPMODE_RTMPEG) {
		return RT_CapCaptureStop (lpHw);
	} else {
		lResult = bcapCaptureAbort (hWnd);
		return lResult;
	}
}

DWORD RT_ThreadProc(DWORD arg)
{	
	LPHWINFO lpHw = (LPHWINFO)arg;	

	RT_CaptureLoop (lpHw);	
	
	return 0;
}


DWORD ghThreadId;

LRESULT 
RT_Capture (LPHWINFO lpHw)
{
	HWND hWnd = lpHw->hWndCap;

	CAPTUREPARMS CapParams;
	char message[MAX_PATH];

	if (!bcapCaptureGetSetup (hWnd, &CapParams, sizeof(CapParams))) {
		if (lpHw->lpfnErrorCallback) {
			RT_LoadAviCapString(lpHw, IDS_CAP_AVI_INIT_ERROR, message, sizeof(message));
			lpHw->lpfnErrorCallback(hWnd, IDS_CAP_AVI_INIT_ERROR, message);
		}
		return FALSE;
	}

	ghThreadId = GetCurrentThreadId();

	if (CapParams.fYield) {

		DWORD dwThreadId;

		/* create the thread */
		lpHw->hCaptureThread = CreateThread(
		    NULL,
			0,
			(LPTHREAD_START_ROUTINE) RT_ThreadProc,
			lpHw,
			0,
			&dwThreadId);

		if (lpHw->hCaptureThread == NULL) {
			if (lpHw->lpfnErrorCallback) {
				RT_LoadAviCapString(lpHw, IDS_CAP_AVI_INIT_ERROR, message, sizeof(message));
				lpHw->lpfnErrorCallback(hWnd, IDS_CAP_AVI_INIT_ERROR, message);
			}
			
			return FALSE;
		}


		return TRUE;

	} else {

		if (RT_CaptureLoop (lpHw)) {
			return FALSE;
		}
	}

	return TRUE;

}


