/*
 * vcupriv.h
 *
 * 32-bit Video Capture driver
 * User-mode support library - private definitions
 *
 *
 * Mark Vrabel, October 95:
 */

#ifdef __cplusplus
extern "C" {
#endif


/*
 * we track stream init/fini, start/stop requests by maintaining
 * the current state of the driver, and only allowing requests that
 * move to the next logical state.
 * here are the valid states - current state is in DEVICE_INFO.State
 */
typedef enum _VCSTATE {
    State_Idle = 0,        // device is Idle
    State_Init,            // stream-init done
    State_Start            // streaming started
} VCSTATE, * PVCSTATE;


typedef enum _VCFUNC {
    StreamFini = 0,
    StreamEvent,
    InvalidFunction

} VCFUNC, *PVCFUNC;


/*
 * a capture device handle VCUSER_HANDLE is an opaque pointer
 * to one of these structures.
 */
struct _VCUSER_HANDLE {

    DWORD SizeStruct;                   // sizeof this structure

    /* common data */             
    HANDLE hDriver;                     // handle to kernel driver
    VCCALLBACK UserCallback;            // user callback info
    VCSTATE State;                      // device state
    BOOL bOverlay;                      // overlay flag
    VC_CALLBACK Callback;               // callback to h/w specific functions


    /* inter-thread sync data */

    DWORD ThreadId;                     // worker thread ID
    HANDLE hThread;                     // worker thread handle

    DWORD FunctionArg;                  // passed to thread init proc
    DWORD FunctionResult;               // result from worker thread init

    HANDLE hTimerEvent;                 // timer event

    HANDLE hWorkEvent;                  // terminate thread event
    HANDLE hCompleteEvent;              // thread init complete event
    HANDLE hErrorSemaphore;             // semaphore for skipped frames
    HANDLE hBufferSemaphore;            // semaphore for finished buffers
    HANDLE hWaitObjects[3];             // wait objects for worker thread

    DWORD SkipCount;                    // count of skipped frames
    DWORD ImageSize;                    // size of DIB

    LPVIDEOHDR ReadyListHead, ReadyListTail;    // local queue of add-buffers
    LPVIDEOHDR DoneListHead, DoneListTail;      // local queue of filled-buffers
    
    TIMECAPS tc;                     // timer capabilities
    UINT TimerId;                    // timer id
    UINT Delay;                      // timer delay
    UINT Resolution;                 // timer resolution
    BOOL bTimerPaused;               // timer pause flag

	CRITICAL_SECTION BufferCritical; // critical section for buffer management    
	BOOL bCriticalInit;				 // critical section init flag

} VCUSER_STRUCT;


/*
 * Inline handle verification
 */
#define IsBadVCUSER_HANDLE(v) ((v) == NULL || IsBadWritePtr((v), sizeof(VCUSER_STRUCT)) || (v)->SizeStruct != sizeof(VCUSER_STRUCT))


DWORD VC_ThreadInit(DWORD arg);

VOID VC_Callback(LPVCCALLBACK lpCallback, UINT msg, DWORD data);
VOID VC_ProcessBuffer(VCUSER_HANDLE vh, LPVIDEOHDR lpVideoHdr);

VOID VC_SkipCount(VCUSER_HANDLE vh, DWORD dwSkips);


#ifdef __cplusplus
}
#endif


